=begin
#-------------------------------
Author: Tax, 2022
Description: Displace an object copies along path edges
#-------------------------------
=end


#-------------------------------------------------------------------------------
module Taxsola
	module Tax_Engineering


def self.displacer_dialog

#Get selection array
model = Sketchup.active_model
entities = model.active_entities
@Selection = model.selection.to_a

#BASIC CHECK
if (@Selection.length < 1) #no selection
	UI.messagebox("Select 1 group and a set of edges.")
	raise Exception.new("Displacer error: no selection.")
end

#GET GROUP ARRAY
@Groups = @Selection.grep(Sketchup::Group).uniq
@Components = @Selection.grep(Sketchup::ComponentInstance).uniq
if ( (@Groups.length + @Components.length) != 1)
	UI.messagebox("Select 1 group/component.")
	raise Exception.new("Displacer error: groups ammount error.")
end

#COMPONENT DEFINITION
if (@Groups.length > 0)
	@Instance = @Groups[0].to_component
	@Definition = @Instance.definition
else
	@Instance = @Components[0]
	@Definition = @Instance.definition
end



#GET EDGES ARRAY
@Path_Edges = @Selection.grep(Sketchup::Edge).uniq
if (@Path_Edges.length < 1)
	UI.messagebox("Select a set of edges as path.")
	raise Exception.new("Displacer error: no path selected.")
end


#MAKE PATH SEGMENTS
@Path_Segments = Taxsola.generate_segments(@Path_Edges)

#CONTINUE CHECK
if UI.messagebox(@Path_Segments.length.to_s + " path segments.\n\nContinue?", type = MB_YESNO) == IDNO
	raise Exception.new("Displacer - Operation canceled.")
end

######################################################################################################



#DEFAULT DISPLACER INPUTS
@Section_Interval = 5.0.cm

@Rotation_Interp = "Linear Interval"
@Rotation_Parameter = 0.0.degrees
@Rot_Phase = 0.0.degrees
@Rot_Cycles = 1.0
@Rot_Exp_Log = 3.0 #Exponential/Logarithm mode

@Scale_Interp = "Linear Total"
@Scale_Parameter = 1.0 #Linear, Parabole and Sine wave modes
@Sca_Phase = 0.0.degrees
@Sca_Cycles = 1.0
@Sca_Exp_Log = 3.0 #Exponential/Logarithm mode

@Rev_Segs = "No"

#For rotation
@Interpolations1 = "Linear Interval|Linear Total|Parabole|SineWave-Cycles|Exponential|Logarithm"
#For scale
@Interpolations2 = "Linear Total|Parabole|SineWave-Cycles|Exponential|Logarithm"
#------------------

puts("---------------------------------------")
@Verification = "NOT OK"
until @Verification == "OK"
#-------------------------------------------------------------------------------
@Verification = "OK"

#INPUT BOX
@Rotation_Parameter = @Rotation_Parameter.radians.round(3).to_s + "degrees"
@Rot_Phase = @Rot_Phase.radians.round(3).to_s + "degrees"
@Sca_Phase = @Sca_Phase.radians.round(3).to_s + "degrees"

@prompts = ["Section interval: ", "Rotation interp: ", "Rotation param: ", "Rotation phase: ", "Rotation Cycles: ", "Rot Exp/Log: ", "Scale interp: ", "Scale param: ", "Scale phase: ", "Scale Cycles: ", "Scale Exp/Log: ", "Reverse segments? "]
@defaults = [@Section_Interval, @Rotation_Interp, @Rotation_Parameter, @Rot_Phase, @Rot_Cycles, @Rot_Exp_Log, @Scale_Interp, @Scale_Parameter, @Sca_Phase, @Sca_Cycles, @Sca_Exp_Log, @Rev_Segs]
@list = ["", @Interpolations1, "", "", "", "", @Interpolations2, "", "", "", "", "Yes|No"]

@input = UI.inputbox(@prompts, @defaults, @list, "Displacer")
#---------

#GET INPUTS
if @input
	@Section_Interval, @Rotation_Interp, @Rotation_Parameter, @Rot_Phase, @Rot_Cycles, @Rot_Exp_Log, @Scale_Interp, @Scale_Parameter, @Sca_Phase, @Sca_Cycles, @Sca_Exp_Log, @Rev_Segs = @input;
else
	raise Exception.new("FRS - Operation canceled. (Inputbox = false)")
end

#Personal method for improving UI experience with angles units
@Rotation_Parameter = Taxsola.to_angle(@Rotation_Parameter)
@Rot_Phase = Taxsola.to_angle(@Rot_Phase)
@Sca_Phase = Taxsola.to_angle(@Sca_Phase)


#INPUT LOG
@Input_Log = "Input log:\n"
@input.each {|field| @Input_Log += field.to_s + " | "}
print @Input_Log


#VERIFICATIONS
@V_Cod = "DISxV."
@V_Num = 0

(
#1st verification
if (@Section_Interval <= 0)
@V_ID = "error: " + @V_Cod + @V_Num.to_s #error: FRSxV.xxx
@msg = @V_ID + " - Section interval must be higher than 0."
UI.messagebox(@msg); warn(@msg)
@Verification = "NOT OK"
end
@V_Num += 1

#2nd verification
if ((@Rotation_Parameter > 60.degrees) || (@Rotation_Parameter < -60.degrees)) && (@Rotation_Interp == "Linear Interval")
@V_ID = "error: " + @V_Cod + @V_Num.to_s #error: FRSxV.xxx
@msg = @V_ID + " - Rotation is out of range. (max +-60 for 'by interval' method)"
UI.messagebox(@msg); warn(@msg)
@Verification = "NOT OK"
end
@V_Num += 1

#3rd verification
if (@Rot_Cycles <= 0)
@V_ID = "error: " + @V_Cod + @V_Num.to_s #error: FRSxV.xxx
@msg = @V_ID + " - Rot Cycles must be > 0."
UI.messagebox(@msg); warn(@msg)
@Verification = "NOT OK"
end
@V_Num += 1

#5th verification
if (@Rot_Exp_Log > 32.0)
@V_ID = "error: " + @V_Cod + @V_Num.to_s #error: FRSxV.xxx
@msg = @V_ID + " - max. Exp/Log Factor = 32."
UI.messagebox(@msg); warn(@msg)
@Verification = "NOT OK"
end
@V_Num += 1

#6th verification
if (@Rot_Exp_Log < -32.0)
@V_ID = "error: " + @V_Cod + @V_Num.to_s #error: FRSxV.xxx
@msg = @V_ID + " - min. Exp/Log Factor = -32."
UI.messagebox(@msg); warn(@msg)
@Verification = "NOT OK"
end
@V_Num += 1



#7th verification
if (@Scale_Parameter > 100)
@V_ID = "error: " + @V_Cod + @V_Num.to_s #error: FRSxV.xxx
@msg = @V_ID + " - Scale parameter is too high. (max 100)"
UI.messagebox(@msg); warn(@msg)
@Verification = "NOT OK"
end
@V_Num += 1

#8th verification
if (@Scale_Parameter <= 0)
@V_ID = "error: " + @V_Cod + @V_Num.to_s #error: FRSxV.xxx
@msg = @V_ID + " - Scale parameter must be higher than 0."
UI.messagebox(@msg); warn(@msg)
@Verification = "NOT OK"
end
@V_Num += 1

#9th verification
if (@Sca_Cycles <= 0)
@V_ID = "error: " + @V_Cod + @V_Num.to_s #error: FRSxV.xxx
@msg = @V_ID + " - Sca Cycles must be > 0."
UI.messagebox(@msg); warn(@msg)
@Verification = "NOT OK"
end
@V_Num += 1

#11th verification
if (@Sca_Exp_Log > 32.0)
@V_ID = "error: " + @V_Cod + @V_Num.to_s #error: FRSxV.xxx
@msg = @V_ID + " - max. Exp/Log Factor = 32."
UI.messagebox(@msg); warn(@msg)
@Verification = "NOT OK"
end
@V_Num += 1

#12th verification
if (@Sca_Exp_Log < -32.0)
@V_ID = "error: " + @V_Cod + @V_Num.to_s #error: FRSxV.xxx
@msg = @V_ID + " - min. Exp/Log Factor = -32."
UI.messagebox(@msg); warn(@msg)
@Verification = "NOT OK"
end
@V_Num += 1

)
#-------------------------------------------------------------------------------
end
print("Verifications OK" + "\n\n")

#Conversions
@Section_Interval = @Section_Interval.to_f

#@Rotation_Parameter is already float radians or cycles
#@Rot_Phase is already float radians or cycles
@Rot_Cycles = @Rot_Cycles.to_f

@Rot_Exp_Log = @Rot_Exp_Log.to_f

@Scale_Parameter = @Scale_Parameter.to_f
#@Sca_Phase is already float radians or cycles
@Sca_Cycles = @Sca_Cycles.to_f

@Sca_Exp_Log = @Sca_Exp_Log.to_f



#Speed boost by disable UI updates ---- <<<
model.start_operation('FRS - Mesh generation', true)

# TIMER ------------------------------------------------------------------------------------
@start_time = Time.now



#Normalizing path segments based on section interval
@Path_Segments = Taxsola.normalize_segments2(@Path_Segments, @Section_Interval, @Rev_Segs)



#CONTINUE CHECK
@Objects = 0
@Path_Segments.each do |segment|
@Objects += (segment.length-1)
end
if UI.messagebox(@Objects.to_s + " objects to be created.\nContinue?", type = MB_OKCANCEL) != IDOK
 UI.messagebox("Operation canceled.", type = MB_OK)
 raise Exception.new("Displacer: Operation canceled.")
end
puts ("Displacer - " + @Objects.to_s + " chain links to be created.")



#Sections properties definition
@Paths = []
@Path_Segments.each do |segment|
@Segment = []

#Segment mode definition
if (segment.first == segment.last) then @Mode = "Loop" else @Mode = "Open" end


for section in 0..(segment.length-1)
@Section = {}
@Section[:position] = segment[section]


@Last_Distance = Taxsola.distance(segment[segment.length-2], (segment[segment.length-1]))

if (@Last_Distance.to_f < @Section_Interval.to_f) #if last section isn't complete
	@Interval_Fraction = @Last_Distance.to_f / @Section_Interval.to_f
	@Seg_Ammount = (segment.length.to_f - 2.0) + @Interval_Fraction

	if (section == (segment.length-1))
		@Curr_Sec = @Seg_Ammount.to_f
	else
		@Curr_Sec = section
	end

else
	@Seg_Ammount = (segment.length-1.0).to_f
	@Curr_Sec = section
end



	#ROTATION definition
	case @Rotation_Interp
		when "Linear Interval"
			@Compass_Rotation = @Rotation_Parameter.to_f
			@Section[:rotation] = (@Compass_Rotation.to_f * @Curr_Sec.to_f) + @Rot_Phase

		when "Linear Total"
			@Compass_Rotation = (@Rotation_Parameter.to_f)/ @Seg_Ammount.to_f
			@Section[:rotation] = (@Compass_Rotation.to_f * @Curr_Sec.to_f) + @Rot_Phase

		when "Parabole"
			@Freq = 180.0.degrees / @Seg_Ammount
			@Angle = (@Freq.to_f * @Curr_Sec.to_f) + @Rot_Phase
			@Rot_Factor = Math.sin(@Angle.to_f)
			@Section[:rotation] = (@Rot_Factor * @Rotation_Parameter.to_f)

		when "SineWave-Cycles"
			@Angle = ((@Rot_Cycles.to_f/@Seg_Ammount.to_f) * @Curr_Sec.to_f * 360.0.degrees) + @Rot_Phase
			@Rot_Factor = Math.sin(@Angle).to_f
			@Section[:rotation] = (@Rot_Factor * @Rotation_Parameter.to_f)

		when "Exponential"
			@Exp = Taxsola.unitary_scale(0, @Curr_Sec, @Seg_Ammount) #From 0 to 1 scale
			@Exp *= 2.0
			@Section[:rotation] = (@Rot_Exp_Log**@Exp).to_f + @Rot_Phase

		when "Logarithm"
			@Exp = Taxsola.unitary_scale(0, @Curr_Sec, @Seg_Ammount) #From 0 to 1 scale
			@Exp *= 2.0
			@Section[:rotation] = (1.0/(@Rot_Exp_Log**@Exp).to_f) + @Rot_Phase

		else
			UI.messagebox("Error: Rotation interpolation not identifyed.")
			raise Exception.new("Error: Rotation interpolation not identifyed.")
	end



	#SCALE definition
	case @Scale_Interp
		when "Linear Total"
			@Compass_Scale = (@Scale_Parameter-1.0)/@Seg_Ammount
			@Section[:scale] = (@Compass_Scale * @Curr_Sec) + 1.0

		when "Parabole"
			@Freq = 180.0.degrees / @Seg_Ammount
			@Angle = (@Freq.to_f * @Curr_Sec.to_f) + @Sca_Phase
			@Scale_Factor = Math.sin(@Angle.to_f)
			@Section[:scale] = (@Scale_Factor * @Scale_Parameter.to_f) + 1.0

		when "SineWave-Cycles"
			@Angle = ((@Sca_Cycles.to_f/@Seg_Ammount.to_f) * @Curr_Sec.to_f * 360.0.degrees) + @Sca_Phase
			@Scale_Factor = (Math.sin(@Angle.to_f)+1.0)/2.0
			@Section[:scale] = (@Scale_Factor * @Scale_Parameter.to_f) + 1.0

		when "Exponential"
			@Exp = Taxsola.unitary_scale(0, @Curr_Sec, @Seg_Ammount) #From 0 to 1 scale
			@Exp *= 2.0
			@Section[:scale] = (@Sca_Exp_Log**@Exp).to_f

		when "Logarithm"
			@Exp = Taxsola.unitary_scale(0, @Curr_Sec, @Seg_Ammount) #From 0 to 1 scale
			@Exp *= 2.0
			@Section[:scale] = 1.0/(@Sca_Exp_Log**@Exp).to_f

		else
			UI.messagebox("Error: Scale interpolation not identifyed.")
			raise Exception.new("Error: Scale interpolation not identifyed.")
	end



	#NORMAL definition
	case section
		when 0 #First section
			if (@Mode == "Open")
				@Section[:normal] = Taxsola.find_normal(segment[0], segment[1], "") #First on open
			else
				@Section[:normal] = Taxsola.find_normal(segment[-2], segment[0], segment[1]) #First on loop
			end

		when (segment.length-1) #Last section
			if (@Mode == "Open")
				@Section[:normal] = Taxsola.find_normal(segment[-2], segment[section], "") #Last on open
			else
				@Section[:normal] = Taxsola.find_normal(segment[-2], segment[section], segment[1]) #Last on loop
			end

		else #Middle sections
			@Section[:normal] = Taxsola.find_normal(segment[section-1], segment[section], segment[section+1])
	end



@Segment << @Section
end

@Paths << @Segment
end



@FRS_global = Sketchup.active_model.entities.add_group #Geral group
@Paths.each do |seg|

	if (@Paths.length > 1)
		@FRS = @FRS_global.entities.add_group #Sub groups in case there is more than 1 segment
	else
		@FRS = @FRS_global
	end

@Last_Sec_Up = Z_AXIS
@Seg_ID = 1
	seg.each do |sec|

		#Get section parameters
		@Sec_P = sec[:position]
		@Sec_N = sec[:normal].normalize
		@Sec_R = sec[:rotation]
		@Sec_S = sec[:scale]
		
		if (@Seg_ID == 1) && ( (@Sec_N == Z_AXIS) || (@Sec_N == Z_AXIS.reverse) )
			UI.messagebox("First segment cannot be vertical.")
			raise Exception.new("First segment cannot be vertical.")
		else

			if ( (sec[:normal].normalize == Z_AXIS) || (sec[:normal].normalize == Z_AXIS.reverse) )
				@Sec_Up = @Last_Sec_Up
				@Sec_Side = @Sec_Up.cross(@Sec_N).normalize
				@Sec_Up = @Sec_Side.cross(@Sec_N).normalize
			else
				@Sec_Up = Z_AXIS
				@Sec_Side = @Sec_Up.cross(@Sec_N).normalize
				@Sec_Up = @Sec_Side.cross(@Sec_N).normalize
			end

			if (@Last_Sec_Up.angle_between @Sec_Up).radians > 90 then @Sec_Up.reverse! end

		end

		#Add new instance
		@Inst = @FRS.entities.add_instance @Instance.definition, Geom::Transformation.scaling(@Sec_P, 1)

		#Transformations definitions
		@transform1 = Geom::Transformation.axes( @Sec_P, @Sec_N, @Sec_Side, @Sec_Up )
		@transform2 = Geom::Transformation.rotation([0,0,0], X_AXIS, @Sec_R)
		@transform3 = Geom::Transformation.scaling([0,0,0], @Sec_S)

		@Inst.transformation *= @transform1
		@Inst.transformation *= @transform2
		@Inst.transformation *= @transform3


		@Last_Sec_Up = @Sec_Up
		@Seg_ID += 1
	end
end



# TIMER END -------------------------------------------------------------------------
@end_time = Time.now
@Total_Time = @end_time - @start_time

Sketchup.active_model.commit_operation
#-------------------------------------- <<<
@msg = "Success.\n" + "Time: " + @Total_Time.round(3).to_s + "s."
puts(@msg)
puts("---------------------------------------")
Taxsola.soundeffect(1) #0: Installation | 1: Success | 2: Notification
end


	end
end
#-------------------------------------------------------------------------------