=begin
#-------------------------------
Author: Tax, 2021
Description:
#-------------------------------
=end


#-------------------------------------------------------------------------------
module Taxsola
	module Tax_Engineering

def self.linear_spiral_dialog
#-------------------------------------------------------------------------------
prompts = ["Radius1 (bottom): ", "Radius2 (top): ", "Height: ", "Coils: ", "Segments per coil: ", "Draw wire?: ", "Wire diameter: ", "Wire profile segments: ", "Draw reference?: "]
defaults = [ 20.00.mm, 10.00.mm, 100.00.mm, 10, 36, "Yes", 5.00.mm, 12, "Yes" ]
list = ["", "", "", "", "", "Yes|No", "", "", "Yes|No"]
input = UI.inputbox(prompts, defaults, list, "Linear Helix")
if input
@Radius1, @Radius2, @Height, @Revolutions, @Edges_per_revolution, @Make_Tube, @Tube_diameter, @Tube_detail, @MakeRef = input;
else
 raise Exception.new("Make Linear Spring error: LSxV.000 - Operation canceled.")
end
#-------------------------------------------------------------------------------

#Timer start
@start_time = Time.now

#Log
print 'Make Linear Spring operation:'
print 'Input log: ' + input[0].to_s + ' | ' + input[1].to_s + ' | ' + input[2].to_s + ' | ' + input[3].to_s + ' | ' + input[4].to_s + ' | ' + input[5].to_s + ' | ' + input[6].to_s + ' | ' + input[7].to_s + ' | ' + input[8].to_s


(#Verifications

# Sphere radius > 0
if (@Radius1 <= 0) or (@Radius2 <= 0)
UI.messagebox("error: LSxV.001 - Both radius must be higher than 0.")
raise Exception.new("Make Parts error: LSxV.001")
end

# Revolutions > 0
if (@Revolutions <= 0)
UI.messagebox("error: LSxV.002 - Revolutions must be higher than 0.")
raise Exception.new("Make Parts error: LSxV.002")
end

# Edges_per_revolution > 0
if (@Edges_per_revolution <= 0)
UI.messagebox("error: LSxV.003 - Edges per revolution must be higher than 0.")
raise Exception.new("Make Parts error: LSxV.003")
end

# Tube_diameter > 0
if (@Tube_diameter <= 0)
UI.messagebox("error: LSxV.004 - Tube diameter must be higher than 0.")
raise Exception.new("Make Parts error: LSxV.004")
end

# Tube_diameter limit
if ( @Tube_diameter >= (@Height / @Revolutions))
UI.messagebox("error: LSxV.005 - Tube diameter is too large.")
raise Exception.new("Make Parts error: LSxV.005")
end

# Tube_detail > 0
if (@Tube_detail <= 0)
UI.messagebox("error: LSxV.006 - Tube detail must be higher than 0.")
raise Exception.new("Make Parts error: LSxV.006")
end
)

print 'Make Linear Spring - Debug point 1/2 ok'

(#Camera and Boost -------------------------------------------------------------

#Define camera view
camera = Sketchup::Camera.new([2, -2, 2], [0, 0, 0], Z_AXIS)
view = Sketchup.active_model.active_view
view.camera = camera

#Speed boost by disable UI updates
model = Sketchup.active_model
entities = model.active_entities
model.start_operation('Make Parts', true)

#-------------------------------------------------------------------------------
)

@Scale = 1000.00
@scale_transform = Geom::Transformation.scaling([0, 0, 0], 1/@Scale)

#-------------------------------------------------------------------------------
if @Radius1 == @Radius2

   print 'Make Linear Spring - Debug point (1 radius) ok'
   #1 radius spiral
   def self.spiral1(radius, height, laps, edgesperlap, tube_diameter, tube_detail, make_reference)
   model = Sketchup.active_model
   entities = model.active_entities

   @Spiral1 = Sketchup.active_model.entities.add_group
   @Reference = @Spiral1.entities.add_group


   #Defining variables
   radius = radius.to_f * @Scale
   height = height.to_f * @Scale
   laps = laps.to_i
   edgesperlap = edgesperlap.to_i
   tube_diameter = tube_diameter.to_f * @Scale
   tube_detail = tube_detail.to_i
   @SpringColor = [40, 40, 40]

   @TotalEdges = laps * edgesperlap
   @Zincrement = height / @TotalEdges
   @EdgeAngle = 360.00.degrees / edgesperlap
   @LapHeight = height / laps

    #Make path reference
    @point1 = [radius, 0, 0]
    for step in 0..@TotalEdges
     @x = Math.cos(@EdgeAngle * step) * radius
     @y = Math.sin(@EdgeAngle * step) * radius
     @z = step * @Zincrement
     @point2 = [@x, @y, @z]
     @line = @Reference.entities.add_line @point1, @point2
     @point1 = @point2
    end
    
     #Get edge and path reference
   @RefEdge = @Reference.entities[0]
   @Vector = @RefEdge.start.position.vector_to(@RefEdge.end.position)
   @Path = @RefEdge.all_connected
   
   if (@Make_Tube == "Yes")
    @Tube = @Spiral1.entities.add_group
   #Make tube face
   @C1 = @Tube.entities.add_circle(@RefEdge.start.position, @Vector , tube_diameter/2, tube_detail)
   @F1 = @Tube.entities.add_face(@C1)

   #-------------------------------------------------------------------------------
   #Set face direction
   @D = 3
   @VectorModule = ((@Vector.x**2) + (@Vector.y**2) + (@Vector.z**2))**0.5
   @UnitVector = [ (@Vector.x/@VectorModule).round(@D), (@Vector.y/@VectorModule).round(@D), (@Vector.z/@VectorModule).round(@D) ]

   @FaceVectorModule = ((@F1.normal.x**2) + (@F1.normal.y**2) + (@F1.normal.z**2))**0.5
   @FaceUnitVector = [ (@F1.normal.x/@FaceVectorModule).round(@D), (@F1.normal.y/@FaceVectorModule).round(@D), (@F1.normal.z/@FaceVectorModule).round(@D) ]

   if @UnitVector == @FaceUnitVector
   @F1.reverse!
   end
   #-------------------------------------------------------------------------------

   #Apply face color
   @F1.material = @SpringColor
   @F1.back_material = @SpringColor

   #Make tube
   @Structure = @F1.followme(@Path)
   @Tube.transformation *= @scale_transform
   
   @Reference.erase!
   end
   
   
    if make_reference == "Yes"
     
     if (@Make_Tube == "Yes")
      @Reference = @Spiral1.entities.add_group
     end
     
     @Pt1 = @Reference.entities.add_cpoint [0, 0, 0]
     @Pt2 = @Reference.entities.add_cpoint [0, 0, height]
     @Cp = @Reference.entities.add_cpoint [0, 0, height/2]
     @Rl = @Reference.entities.add_line [0, 0, 0], [0, 0, height]
     @Reference.transformation *= @scale_transform    
    else
    
     if (@Make_Tube == "No")
      @Reference.transformation *= @scale_transform
     end    
    end


   end

   self.spiral1(@Radius1, @Height, @Revolutions, @Edges_per_revolution, @Tube_diameter, @Tube_detail, @MakeRef)

else #--------------------------------------------------------------------------

   print 'Make Linear Spring - Debug point (2 radius) ok'
   #2 radius spiral
   def self.spiral2(radius1, radius2, height, laps, edgesperlap, tube_diameter, tube_detail, make_reference)
   model = Sketchup.active_model
   entities = model.active_entities

   @Spiral2 = Sketchup.active_model.entities.add_group
   @Reference = @Spiral2.entities.add_group


   #Defining variables
   radius1 = (radius1 * @Scale).to_f
   radius2 = (radius2 * @Scale).to_f
   height = (height * @Scale).to_f
   laps = laps.to_i
   edgesperlap = edgesperlap.to_i
   tube_diameter = (tube_diameter * @Scale).to_f
   tube_detail = tube_detail.to_i
   @SpringColor = [40, 40, 40]

   @TotalEdges = laps * edgesperlap
   @Zincrement = height / @TotalEdges
   @EdgeAngle = 360.00.degrees / edgesperlap
   @LapHeight = height / laps

   @Reduction = radius2 - radius1
   @ReductionPerEdge = @Reduction / @TotalEdges

    @point1 = [radius1, 0, 0]
    for step in 0..@TotalEdges
     @Radius = (radius1 + (@ReductionPerEdge * step))
     @x = Math.cos(@EdgeAngle * step) * @Radius
     @y = Math.sin(@EdgeAngle * step) * @Radius
     @z = step * @Zincrement
     @point2 = [@x, @y, @z]
     @line = @Reference.entities.add_line @point1, @point2
     @point1 = @point2
    end
    
    #Get edge and path reference
   @RefEdge = @Reference.entities[0]
   @Vector = @RefEdge.start.position.vector_to(@RefEdge.end.position)
   @Path = @RefEdge.all_connected
   
   if (@Make_Tube == "Yes")
    @Tube = @Spiral2.entities.add_group
   #Make tube face
   @C1 = @Tube.entities.add_circle(@RefEdge.start.position, @Vector , tube_diameter/2, tube_detail)
   @F1 = @Tube.entities.add_face(@C1)

   #-------------------------------------------------------------------------------
   #Set face direction
   @D = 3
   @VectorModule = ((@Vector.x**2) + (@Vector.y**2) + (@Vector.z**2))**0.5
   @UnitVector = [ (@Vector.x/@VectorModule).round(@D), (@Vector.y/@VectorModule).round(@D), (@Vector.z/@VectorModule).round(@D) ]

   @FaceVectorModule = ((@F1.normal.x**2) + (@F1.normal.y**2) + (@F1.normal.z**2))**0.5
   @FaceUnitVector = [ (@F1.normal.x/@FaceVectorModule).round(@D), (@F1.normal.y/@FaceVectorModule).round(@D), (@F1.normal.z/@FaceVectorModule).round(@D) ]

   if @UnitVector == @FaceUnitVector
   @F1.reverse!
   end
   #-------------------------------------------------------------------------------

   #Apply face color
   @F1.material = @SpringColor
   @F1.back_material = @SpringColor

   #Make tube
   @Structure = @F1.followme(@Path)
   @Tube.transformation *= @scale_transform
   
   @Reference.erase!
   end


    if make_reference == "Yes"
     
     if (@Make_Tube == "Yes")
      @Reference = @Spiral2.entities.add_group
     end
     
     @Pt1 = @Reference.entities.add_cpoint [0, 0, 0]
     @Pt2 = @Reference.entities.add_cpoint [0, 0, height]
     @Cp = @Reference.entities.add_cpoint [0, 0, height/2]
     @Rl = @Reference.entities.add_line [0, 0, 0], [0, 0, height]
     @Reference.transformation *= @scale_transform    
    else
    
     if (@Make_Tube == "No")
      @Reference.transformation *= @scale_transform
     end    
    end

   end

   self.spiral2(@Radius1, @Radius2, @Height, @Revolutions, @Edges_per_revolution, @Tube_diameter, @Tube_detail, @MakeRef)

end

#-------------------------------------------------------------------------------
print 'Make Linear Spring - Debug point 2/2 ok'
 #Timer end
@end_time = Time.now
@Total_Time = @end_time - @start_time
print "Success. (" + @Total_Time.to_s + " seconds) \n \n"
Taxsola.soundeffect(1) #0: Installation | 1: Success | 2: Notification
#-------------------------------------------------------------------------------

end

	end
end
#-------------------------------------------------------------------------------