=begin
#-------------------------------
Author: Tax, 2021
Description:
#-------------------------------
=end


#-------------------------------------------------------------------------------
module Taxsola
	module Tax_Engineering

def self.mariner_dialog

(#BASIC INITIAL HANDLERS -----------------------------------------------------
model = Sketchup.active_model
entities = model.active_entities

@selection = model.selection.to_a
@edges = @selection.grep(Sketchup::Edge).uniq
@Unchecked_Edges = @edges
@Checked_Edges = []
@Segments = []

@MODE = ""
if (@edges.length == 0)
print ("Make Mariner Chain debug: MxD.000 - No edges selected. Default mode operation.")
UI.messagebox("No path selected.\nDefault mode operation.")
@MODE = "no path"
else
print ("Make Mariner Chain debug: MxD.001 - Follow path along operation.")
@MODE = "follow path"
end
#-----------------------------------------------------------------------------
)


if (@MODE == "follow path")

(#VERTEX TYPE CLASSIFICATION --------------------------------------------------
@Raw_vertexes = []
@edges.each { |edge| @Raw_vertexes << edge.start; @Raw_vertexes << edge.end;}

@Single_vertexes = @Raw_vertexes.uniq
@Vertex_Counter = @Single_vertexes.collect { |v| @Raw_vertexes.count(v) }


@End_Vertexes = []
@Middle_Vertexes = [] 
@Intersection_Vertexes = []

@ID = 0
@Vertex_Counter.each do |counter|
	if (counter == 1) then @End_Vertexes << @Single_vertexes[@ID]; end
	if (counter == 2) then @Middle_Vertexes << @Single_vertexes[@ID]; end
	if (counter > 2) then @Intersection_Vertexes << @Single_vertexes[@ID]; end
	@ID += 1
end
#-----------------------------------------------------------------------------
print 'Make Mariner Chain - VERTEX TYPE CLASSIFICATION OK'
)

#SEGMENT MAKER ----------------------------------------------------
def self.make_segment(initialvertex, initialedge)
@Segment_Vertexes = []

@Vertex = initialvertex
@Edge = initialedge

	while ((@Middle_Vertexes.include?(@Vertex) == true) || (@Vertex == initialvertex)) && (@Checked_Edges.include?(@Edge) == false)
		@Segment_Vertexes << @Vertex
		@Checked_Edges << @Edge
		@Unchecked_Edges -= [@Edge]

		#Define new vertex
		@Vertex = @Edge.other_vertex @Vertex

		#Finds new edge
		@n = 0
		loop do
			if (@Unchecked_Edges.length == 0) then break end
			if (@Unchecked_Edges[@n].used_by? @Vertex) && (@Unchecked_Edges[@n] != @Edge) then @Edge = @Unchecked_Edges[@n]; break; end
			if (@n == @Unchecked_Edges.length-1) then break end
			@n += 1
		end
	end
@Segment_Vertexes << @Vertex

return @Segment_Vertexes
end
#------------------------------------------------------------------

#1st round - segments making from intersections -------------------------------
@Int_Seg_Ind = []
@Unchecked_Edges_Cache = @Unchecked_Edges

if (@Intersection_Vertexes.length > 0)
	@Intersection_Vertexes.each do |vertex|
		@Edges_Cache = []
		@Unchecked_Edges_Cache.each do |edge| #Didn't know about '.edges' method. Could work better.
			if (edge.used_by? vertex)
				@Edges_Cache << edge
				@Unchecked_Edges_Cache -= [edge]
			end
		end

		@Int_Seg_Ind << @Edges_Cache
	end

	@Int_Seg_Ind.each do |intersection|
		intersection.each do |segment_ind|
			if (@Checked_Edges.include?(segment_ind) == false)
				@Segments << self.make_segment(@Intersection_Vertexes[@Int_Seg_Ind.index(intersection)],segment_ind)
			end
		end
	end

end
print 'Make Mariner Chain - Segments making (1st round OK)'
#------------------------------------------------------------------------------


#2nd round - segments making from end points ----------------------------------
if (@End_Vertexes.length > 0)
	@End_Vertexes.each do |vertex|
		@Unchecked_Edges.each do |edge|
			if (edge.used_by? vertex)
				@Segments << self.make_segment(vertex, edge)
				break
			end
		end
	end
end
print 'Make Mariner Chain - Segments making (2nd round OK)'
#------------------------------------------------------------------------------


#3rd round - segments making loose loops --------------------------------------
until (@Unchecked_Edges.length == 0)
@Segments << self.make_segment(@Unchecked_Edges[0].start,@Unchecked_Edges[0])
end
print 'Make Mariner Chain - Segments making (3rd round OK)'
#------------------------------------------------------------------------------


#Remove aligned (in line) vertexes
@Seg_Num = 0
@Segments.each do |segment|
	@To_Remove = []
	for n in 1..(segment.length-2)
		if @Middle_Vertexes.include?(segment[n])
			@Last_Vector = segment[n-1].position.vector_to(segment[n].position)
			@Next_Vector = segment[n].position.vector_to(segment[n+1].position)
			@Last_Vector.length = 10000 #High value to prevent precision issues
			@Next_Vector.length = 10000

			if (@Last_Vector == @Next_Vector) then @To_Remove << segment[n]; end
		end
	end
	@Segments[@Seg_Num] = segment - @To_Remove
	@Seg_Num += 1
end
print 'Make Mariner Chain - Remove aligned OK'
#---------------------------------

UI.messagebox("There are " + @Segments.length.to_s + " path segments on selection.")
end



#VARIABLES DEFAULT VALUES
@Choosen_Preset = "Custom"
@ND = 5.00
@ILength = 35.00
@IWidth = 10.00
@Type = "Stud"
@Detail = 12
@Copies = 5

#Presets setup
@Presets = {
'Custom': [],
'G30 - 1/8': [3.962,23.876,6.35],
'G30 - 3/16': [5.512,24.892,9.906],
'G30 - 1/4': [7.01,31.496,12.7],
'G30 - 5/16': [8.407,32.766,11.176],
'G30 - 3/8': [10.008,35.052,13.97],
'G30 - 1/2': [13.005,45.466,18.288],
'G80 - 1/4"': [6,18,7.8],
'G80 - 5/16"': [8,24,10.4],
'G80 - 3/8"': [10,30,13],
'G80 - 1/2"': [13,39,17],
'G80 - 5/8"': [16,48,20.8],
'G80 - 3/4"': [19,57,24.7],
'G80 - 7/8"': [22,66,28.6],
'G80 - 1"': [26,78,33.8],
'G80 - 1 1/4"': [32,96,41.6],
'G80 - 1 3/8"': [36,108,46.8]
}
@Presets_String = ''
@Presets.each {|key,value| @Presets_String += key.to_s; @Presets_String += '|'}
@Presets_String.chomp!'|'



@Verification = "NOT OK"
until @Verification == "OK" #INPUT DIALOG BOX
#-------------------------------------------------------------------------------
@Verification = "OK"
prompts = ["Preset:", "Nominal Diameter: ", "Inside Length:", "Inside Width:", "Chain type:", "Detail (sides):", "Copies in row:"]
defaults = [@Choosen_Preset, @ND, @ILength, @IWidth, @Type, @Detail, @Copies]
list = [@Presets_String, "", "", "", "Open|Stud", "", ""]
input = UI.inputbox(prompts, defaults, list, "Make Mariner Chain")

#Get input values
if input
@Choosen_Preset, user_input1, user_input2, user_input3, user_input4, user_input5, user_input6 = input;
else
 raise Exception.new("Make Mariner Chain error: MxV.000 - Operation canceled.")
end

#LOAD PRESET
if (@Choosen_Preset != "Custom") #Get selected preset values into the variables
	puts ("Loading preset")
	@Preset_Values = @Presets[@Choosen_Preset.to_sym]
	user_input1 = @Preset_Values[0].to_f
	user_input2 = @Preset_Values[1].to_f
	user_input3 = @Preset_Values[2].to_f
end



#Print input log
print 'Make Mariner Chain operation:'
@Input_Log = ""
@Input_Log += @Choosen_Preset.to_s + ' | '
@Input_Log += user_input1.to_s + ' | '
@Input_Log += user_input2.to_s + ' | '
@Input_Log += user_input3.to_s + ' | '
@Input_Log += user_input4.to_s + ' | '
@Input_Log += user_input5.to_s + ' | '
@Input_Log += user_input6.to_s
print 'Input log: ' + @Input_Log.to_s

print 'Make Mariner Chain - Debug point 1/5 ok'


#Define camera view
if (@MODE == "no path")
camera = Sketchup::Camera.new([0, 0, 10], [0, 0, 0], Y_AXIS)
view = Sketchup.active_model.active_view
view.camera = camera
end


  #Get variables input
  @ND = user_input1.to_f
  @ILength = user_input2.to_f
  @IWidth = user_input3.to_f
  @Detail = user_input5.to_i
  @Type = user_input4.to_s
  @Copies = user_input6.to_f

(#VERIFICATIONS
#ND <= 0
if (@ND <= 0)
UI.messagebox("Nominal diameter must be higher than 0.")
print("Make Parts error: Nominal diameter must be higher than 0.")
@Verification = "NOT OK"
end

#Length < width
if (@ILength < @IWidth)
UI.messagebox("Length can't be smaller than width.")
print("Make Parts error: Length can't be smaller than width.")
@Verification = "NOT OK"
end

#ND mustbe smaller than width
if (@ND > @IWidth)
UI.messagebox("Diameter can't be bigger than width.")
print("Make Parts error: Diameter can't be bigger than width.")
@Verification = "NOT OK"
end

#length must be at least 2*ND
if (@ILength < (@ND*2))
UI.messagebox("Length must be at least 2 * diameter.")
print("Make Parts error: Length must be at least 2 * diameter.")
@Verification = "NOT OK"
end

#if STUD, length must be at least 3*ND
if (@Type == "Stud") and (@ILength < (@ND*3))
UI.messagebox("When using stud, length must be at least 3 * diameter.")
print("Make Parts error: When using stud, length must be at least 3 * diameter.")
@Verification = "NOT OK"
end
)

print ("Make Mariner Chain - Verifications: " + @Verification.to_s)
end

print 'Make Mariner Chain - Debug point 2/5 ok'

#Timer start ------------------------------------------------------------------------------
@start_time = Time.now

#Speed boost by disable UI updates ---- <<<
model.start_operation('Chain link - base model', true)

#Conversion
@Scale = 100.000
@ND = (@ND * @Scale).mm
@ILength = (@ILength * @Scale).mm
@IWidth = (@IWidth * @Scale).mm

(#Math --- 

#PT1
@pt1x = ((@ILength - @IWidth) / 2) * (-1)
@pt1y = (@IWidth + @ND) / 2

#PT2
@pt2x = (@ILength - @IWidth) / 2 
@pt2y = (@IWidth + @ND) / 2

#PT3
@pt3x = ((@ILength - @IWidth) / 2) * (-1)
@pt3y = ((@IWidth + @ND) / 2) * (-1)

#PT4
@pt4x = (@ILength - @IWidth) / 2 
@pt4y = ((@IWidth + @ND) / 2) * (-1)

@raio = (@IWidth + @ND) / 2

@line1 = []
@line1[0] = [@pt1x, @pt1y, 0]
@line1[1] = [@pt2x, @pt2y, 0]

@line2 = []
@line2[0] = [@pt3x, @pt3y, 0]
@line2[1] = [@pt4x, @pt4y, 0]
)

model = Sketchup.active_model
entities = model.active_entities


(#MODELING

#Group 1 - Construction 
@ConstPath = Sketchup.active_model.entities.add_group
@L1 = @ConstPath.entities.add_line @line1
@L2 = @ConstPath.entities.add_line @line2
@A1 = @ConstPath.entities.add_arc [-@pt1x, 0, 0], [0,@pt3y,0], [0,0,1], @raio, 0.degrees, 180.degrees, @Detail
@A2 = @ConstPath.entities.add_arc [@pt1x, 0, 0], [0,@pt1y,0], [0,0,1], @raio, 0.degrees, 180.degrees, @Detail

#Group 2 - BASIC ELO
@EloGroup = Sketchup.active_model.entities.add_group
@C1 = @EloGroup.entities.add_circle([0,@pt1y,0], [1, 0, 0] , @ND/2, @Detail)
@F1 = @EloGroup.entities.add_face(@C1)
status = @F1.material = [153, 153, 255]
status = @F1.back_material = [153, 153, 255]
@Path = @L1.all_connected
@ELO = @F1.followme(@Path)
status = @ConstPath.erase!
@EloFinal = @EloGroup

print 'Make Mariner Chain - Debug point 3/5 ok'

#Making STUD link
if (@Type == "Stud")
   #Group 3 - STUD
   @StudGroup = Sketchup.active_model.entities.add_group
   @C2 = @StudGroup.entities.add_circle([0,@pt3y,0], [0, 1, 0] , @ND/2, @Detail)
   @F2 = @StudGroup.entities.add_face(@C2)
   status = @F2.material = [153, 153, 255]
   status = @F2.back_material = [153, 153, 255]
   status = @F2.pushpull((@IWidth + @ND), true)

   model = Sketchup.active_model
   entities = model.active_entities
   @Temp = @StudGroup.union(@EloGroup)

        #Changing axis
        @EloFinal = Sketchup.active_model.entities.add_group
        @L1 = @EloFinal.entities.add_line @line1
        @EloFinal.entities.add_instance(@Temp.entities.parent, @Temp.transformation*@EloFinal.transformation)
        status = @L1.erase!
        status = @Temp.erase!
        @EloFinal.entities[0].explode
end

@EloFinal = @EloFinal.to_component #Object to component conversion
@EloFinal.name = "Mariner chain link"
@Elo_def = @EloFinal.definition
)


Sketchup.active_model.commit_operation
#-------------------------------------- <<<
print 'Make Mariner Chain - Base model OK'

if (@MODE == "no path")
print 'Make Mariner Chain - No path mode'
#Putting 2nd chain link
@EloFinal2 = @EloFinal.copy
@EloFinal2 = @EloFinal2.move! [(@ILength), 0, 0]


   model = Sketchup.active_model
   entities = model.active_entities
   @MarinerDuo = Sketchup.active_model.entities.add_group
   @MarinerDuo.entities.add_instance(@EloFinal.definition, @EloFinal.transformation*@MarinerDuo.transformation)
   @MarinerDuo.entities.add_instance(@EloFinal2.definition, @EloFinal2.transformation*@MarinerDuo.transformation)
   @EloFinal.erase!
   @EloFinal2.erase!
   entities.transform_entities(Geom::Transformation.rotation([0,0,0],[1,0,0],90.degrees), @MarinerDuo.entities[1])

   @Elo1 = @MarinerDuo.entities[0]
   @Elo2 = @MarinerDuo.entities[1]

print 'Make Mariner Chain - Debug point 4/5 ok'

	if (@Copies > 0)
	 model = Sketchup.active_model
	 entities = model.active_entities
	 @WholeChain = Sketchup.active_model.entities.add_group
	 @WholeChain.entities.add_instance(@MarinerDuo.definition, @MarinerDuo.transformation*@WholeChain.transformation)
	 @MarinerDuo.erase!

	  for step in 1..@Copies
	  @Extend = @WholeChain.entities[0].copy
	  @Extend = @Extend.move! [(@ILength)*2*step, 0, 0]
	  end
	 
	 @constpoint = @WholeChain.entities.add_cpoint [0, 0, 0]
	 #Reset scale
	 scale_transform = Geom::Transformation.scaling([0,0,0], 1/@Scale)
	 @WholeChain.transformation *= scale_transform
	 
	else
	 @constpoint = @MarinerDuo.entities.add_cpoint [0, 0, 0]
	 #Reset scale
	 scale_transform = Geom::Transformation.scaling([0,0,0], 1/@Scale)
	 @MarinerDuo.transformation *= scale_transform
	end

else #Put chain links along path
print 'Make Mariner Chain - Follow path mode'

@transform = Geom::Transformation.scaling([0,0,0], 1/@Scale)
@EloFinal.transformation *= @transform


#Angled offset math -----------------------------------------------
def self.find_new_point(current_point, reference_point, next_point, interval)

#Vectors
@Back_Vec = reference_point.vector_to(current_point)
@Front_Vec = reference_point.vector_to(next_point)

#Math
@Angle_A = @Back_Vec.angle_between @Front_Vec
@Angle_B = Math.asin((Math.sin(@Angle_A)*@Back_Vec.length)/interval)
@Angle_C = (180.0 - (@Angle_A.radians + @Angle_B.radians)).degrees
@Ref_Offset = (Math.sin(@Angle_C)*interval)/Math.sin(@Angle_A)

#Final result
@New_Point_Vector = @Front_Vec
@New_Point_Vector.length = @Ref_Offset
@New_Point = reference_point.offset(@New_Point_Vector)

return @New_Point
end
#------------------------------------------------------------------

def self.collinear_check(point1, point2, point3)
	@Array = [point1, point2, point3].uniq
	if (@Array.length == 2) then return true end
	@Vector1 = point1.vector_to(point2)
	@Vector2 = point1.vector_to(point3)
	@Vector1.length = 1
	@Vector2.length = 1
	if (@Vector1 == @Vector2) || (@Vector1 == @Vector2.reverse) then return true end
	return false
end


#Normalizes path segments
print 'Make Mariner Chain - Starting segments normalizing'
@Link_Interval = @ILength/@Scale
@Normalized_Segments = []

@Segments.each do |segment|
	@Seg_Points = []
	segment.each {|vertex| @Seg_Points << vertex.position}

	@Current_Point = @Seg_Points[0]
	@Next_Point = @Seg_Points[1]
	
	@Normalized_Segment = []
	@Normalized_Segment << Geom::Point3d.new(@Current_Point)

	@STATUS = "STARTING"
	until (@STATUS == "FINISHED")
		@Distance = @Current_Point.distance(@Next_Point)

		until (@Link_Interval <= @Distance) || ( @Seg_Points.index(@Next_Point) == (@Seg_Points.length-1) ) || ( @Seg_Points.index(@Next_Point) == 0 )
		@Next_Point = @Seg_Points[@Seg_Points.index(@Next_Point)+1]
		@Distance = @Current_Point.distance(@Next_Point)
		end

		if (@Link_Interval > @Distance) then
			#End projection
			if self.collinear_check(@Current_Point, @Seg_Points[@Seg_Points.index(@Next_Point)-1], @Next_Point)
				#Linear
				@New_Point_Offset = @Current_Point.vector_to(@Next_Point)
				@New_Point_Offset.length = @Link_Interval
				@New_Point = @Current_Point.offset(@New_Point_Offset)
				@Normalized_Segment << Geom::Point3d.new(@New_Point)
				@Current_Point = @New_Point
				@STATUS = "FINISHED"
			else
				#Angular
				@New_Point = self.find_new_point(@Current_Point, @Seg_Points[@Seg_Points.index(@Next_Point)-1], @Next_Point, @Link_Interval)
				@Normalized_Segment << Geom::Point3d.new(@New_Point)
				@Current_Point = @New_Point
				@STATUS = "FINISHED"
			end
		end

		if (@Link_Interval < @Distance)
			if self.collinear_check(@Current_Point, @Seg_Points[@Seg_Points.index(@Next_Point)-1], @Next_Point)
				#Linear
				@New_Point_Offset = @Current_Point.vector_to(@Next_Point)
				@New_Point_Offset.length = @Link_Interval
				@New_Point = @Current_Point.offset(@New_Point_Offset)
				@Normalized_Segment << Geom::Point3d.new(@New_Point)
				@Current_Point = @New_Point
			else
				#Angular
				@New_Point = self.find_new_point(@Current_Point, @Seg_Points[@Seg_Points.index(@Next_Point)-1], @Next_Point, @Link_Interval)
				@Normalized_Segment << Geom::Point3d.new(@New_Point)
				@Current_Point = @New_Point
			end
		end

		if (@Link_Interval == @Distance)
			@New_Point = @Next_Point
			@Normalized_Segment << Geom::Point3d.new(@New_Point)
			@Current_Point = @New_Point
			
			if (@Seg_Points.index(@Next_Point) == (@Seg_Points.length-1)) then @STATUS = "FINISHED"; end
		end
	end


@Normalized_Segments << @Normalized_Segment
end
print 'Make Mariner Chain - Segments normalized'
#--------------------------------


#Chain links ammount - CONTINUE CHECK
@Chain_Links = 0
@Normalized_Segments.each do |segment|
@Chain_Links += (segment.length-1)
end

if UI.messagebox(@Chain_Links.to_s + " chain links to be created.", type = MB_OKCANCEL) != IDOK
 UI.messagebox("Operation canceled.", type = MB_OK)
 raise Exception.new("Make Mariner Chain: Operation canceled.")
end
print ("Make Mariner Chain - " + @Chain_Links.to_s + " chain links to be created.")
#-----------------------------------------------------------------------------


#Speed boost by disable UI updates ---- <<<
model.start_operation('Chain link - segments instances', true)

#Puts chain link instances along normalized segments
@All_Chains = model.entities.add_group
@Normalized_Segments.each do |segment|
	
	if (@Normalized_Segments.length > 1)
	@Group = @All_Chains.entities.add_group
	else
	@Group = model.entities.add_group
	end
	
	@UP_Vector = Z_AXIS
	for i in 0..(segment.length-2)

		@Direction = segment[i].vector_to(segment[i+1])
		@Direction.normalize!

		@Offset = segment[i].vector_to(segment[i+1])
		@Offset.length = @Offset.length/2.0
		@Position = segment[i].offset(@Offset)


#-------------- Seach for a better aproach on defining Up_vector
		if ((@Direction == Z_AXIS) || (@Direction == Z_AXIS.reverse)) && (@i == 0)
		@UP_Vector = X_AXIS
		end

		if ((@UP_Vector == Z_AXIS) || (@UP_Vector == Z_AXIS.reverse)) && ((@Direction == Z_AXIS) || (@Direction == Z_AXIS.reverse))
		@UP_Vector = Geom::Vector3d.new(1.022,1.011,10) #Random value close to Z_AXIS to prevent glitch
		@UP_Vector.normalize!
		end

		@UP_Vector = @Direction.cross(@Direction.cross(@UP_Vector))
		@UP_Vector.normalize!
#--------------


		@Side_Vector = @Direction.cross(@UP_Vector)
		
		@Elo_Copy = @Group.entities.add_instance @Elo_def, Geom::Transformation.scaling(@Position, 1/@Scale) #Position & Scale <<<

		@transform2 = Geom::Transformation.axes( @Position, @Direction, @Side_Vector, @UP_Vector )
		@Elo_Copy.transformation *= @transform2 #Direction <<<

		if (i%2 == 0) #Rotation <<<
			@transform3 = Geom::Transformation.rotation([0,0,0], X_AXIS, 90.degrees)
			@Elo_Copy.transformation *= @transform3
		end
	end
end

@EloFinal.erase! #Erases base chain link model

Sketchup.active_model.commit_operation
#-------------------------------------- <<<

#------------------------------------------------------------------------------
end

print 'Make Mariner Chain - Debug point 5/5 ok'


 #Timer end ----------------------------------------
@end_time = Time.now
@Total_Time = @end_time - @start_time

#End Message
@EndMessage = "Success. (" + @Total_Time.to_s + " seconds)\n"
print @EndMessage + "\n\n"
Taxsola.soundeffect(1) #0: Installation | 1: Success | 2: Notification
end


	end
end
#-------------------------------------------------------------------------------