=begin
#-------------------------------
Author: Tax, 2021
Description:
#-------------------------------
=end


#-------------------------------------------------------------------------------
module Taxsola
	module Tax_Engineering

def self.railway_along_dialog
# SELECTION FILTER AND CHECK --------------------------------------------------------

model = Sketchup.active_model
entities = model.active_entities

@Raw_Selection = model.selection.to_a #Selection to array
@Raw_Edges = @Raw_Selection.grep(Sketchup::Edge) #Filter only edges

if (@Raw_Edges.length == 0)
UI.messagebox("No edges selected.")
raise Exception.new("Make RailWay Along error: RAxV.000 - Operation aborted due to missing selection.")
end

#------------------------------------------------------------------------------------
@Reverse = "No"
@Make_Ref = "No"
@Timber_Bevel = "Yes"
@Bevel_Factor = 15.0
@Transition = "Linear"

@Rails_distance = 1435.00.mm #(GAUGE)
@Up_Offset = 50.00.mm

@Timber_Length = 2250.00.mm
@Timber_Width = 200.00.mm
@Timber_Thickness = 100.00.mm
@Timber_Spacing = 800.00.mm

@Rail_Color = [202,204,204]
@Timber_Color = [104,85,60]

#Rail profile variables
@HT = 5.75
@BW = 5.75
@HW = 2.75
@W = 0.5625
@HD = 1.703125
@FD = 3.078125
@BD = 0.96875
#------------------------------------------------------------------------------------
@Verification = "NOT OK"
until @Verification == "OK"
#-------------------------------------------------------------------------------
@Verification = "OK"
prompts = ["Rails gauge: ", "Timber length: ", "Timber width: ", "Timber thickness: ", "Timber spacing: ", "Up offset: ", "Reverse? ", "Timber bevel? ","Make Reference? ", "Transition mode: "]
defaults = [@Rails_distance, @Timber_Length, @Timber_Width, @Timber_Thickness, @Timber_Spacing, @Up_Offset, @Reverse, @Timber_Bevel, @Make_Ref, @Transition]
list = ["", "", "", "", "", "", "Yes|No", "Yes|No", "Yes|No", "Linear|Gradient"]

val = UI.inputbox(prompts, defaults, list, "Make Railway Along")
if val
@Rails_distance, @Timber_Length, @Timber_Width, @Timber_Thickness, @Timber_Spacing, @Up_Offset, @Reverse, @Timber_Bevel, @Make_Ref, @Transition = val;
else
 raise Exception.new("Make Railway Along error: RAxV.001 - Operation canceled.\n")
end
#-------------------------------------------------------------------------------


#Log and verification
print 'Make RailWay Along operation:'
print 'Input log: ' + val[0].to_s + ' | ' + val[1].to_s + ' | ' + val[2].to_s + ' | ' + val[3].to_s + ' | ' + val[4].to_s + ' | ' + val[5].to_s + ' | ' + val[6].to_s + ' | ' + val[7].to_s + ' | ' + val[8].to_s
#------------------------------------------------------------------------------------
print 'Make RailWay Along - Debug point 1/10 ok' #DEBUG POINT <<< ################################## <<<


(#VERIFICATIONS

#@Rails_distance > @BW
if (@Rails_distance < @BW)
UI.messagebox("error: RAxV.002 - Rails gauge must be higher than " + @BW.to_s)
warn("Make Parts error: RAxV.002")
@Verification = "NOT OK"
end

if (@Timber_Length < (@Rails_distance + @BW) )
UI.messagebox("error: RAxV.003 - Timber length must be higher than " + (@Rails_distance + @BW).to_s)
warn("Make Parts error: RAxV.003")
@Verification = "NOT OK"
end

if (@Timber_Width > @Timber_Spacing )
UI.messagebox("error: RAxV.004 - Timber width must be smaller than timber spacing.")
warn("Make Parts error: RAxV.004")
@Verification = "NOT OK"
end

if (@Timber_Spacing <= 0 )
UI.messagebox("error: RAxV.005 - Timber spacing must be higher than 0.")
warn("Make Parts error: RAxV.005")
@Verification = "NOT OK"
end
)

end

#-------------------------------------------------------------------------------
print 'Make RailWay Along - Debug point 2/10 ok' #DEBUG POINT <<< ################################## <<<

#Handlers
model = Sketchup.active_model
entities = model.active_entities
#-------------------------------------------------------------------------------

Sketchup.active_model.selection.clear
Sketchup.active_model.selection.add(@Raw_Edges)

#Remove possible duplicates
@Raw_Edges.uniq!
@Edges_Ammount = @Raw_Edges.length

#Register all vertex for each edge
@Raw_Vertex = []
@Path_Length = 0
#------------------------------------------------------------------------------------
 @Raw_Edges.each do |edge|
  @Raw_Vertex << edge.start
  @Raw_Vertex << edge.end
  @Path_Length += edge.length
 end
 #------------------------------------------------------------------------------------

#OPERATION CONFIRMATION
@Sket_Version = Sketchup.version.to_i
#Estimate contruction speed based on my tests (Personal record: 14m/s on SK17 and 55m/s on SK18)
if (@Sket_Version > 17) then @Construction_Speed = 20.0; else @Construction_Speed = 0.5; end

@Total_in_seconds = ((@Path_Length / 1.00.m) / @Construction_Speed)
@Total_minutes_only = (@Total_in_seconds / 60.0).to_i
@Rest_in_seconds = (((@Total_in_seconds / 60.0) - @Total_minutes_only.to_f) * 60.0).round(2)
@Estimated_time_text = @Total_minutes_only.to_s + "min " + @Rest_in_seconds.to_s + "s"

@Text1 = "It can take a few minutes, please wait."
@Text2 = "Estimated time (at " + @Construction_Speed.to_s + "m/s): " + @Estimated_time_text

@Continue = UI.messagebox(@Text1 + "\n" + @Text2, type = MB_OKCANCEL)
if (@Continue != IDOK) then raise Exception.new("Make Railway Along error: RAxV.001 - Operation canceled.\n"); end
#------------------------------------------------------------------------------------

#Timer start
@start_time = Time.now


#Get how many times each vertex is used
@Vertex_Counter = @Raw_Vertex.uniq.collect { |a| @Raw_Vertex.count(a) }

(# VERIFICATION (ends and derivations) ------------------------------------------------
if (@Vertex_Counter.count(1) > 2)
raise Exception.new("Error. More than 2 ends.")
end

if ((@Vertex_Counter.count(1) + @Vertex_Counter.count(2)) < @Vertex_Counter.length)
raise Exception.new("Error. There are derivations.")
end

@MODE = ""

if (@Vertex_Counter.count(1) == 2)
print ("Path type: open path.")
@MODE = "OPEN"
end

if (@Vertex_Counter.count(2) == @Vertex_Counter.length)
print ("Path type: closed loop.")
@MODE = "LOOP"
end
#------------------------------------------------------------------------------------
)


#GET FIRST VERTEX TO START SERIALIZE PROCESS
if (@MODE == "OPEN")
 @Start_Vertex = @Raw_Vertex.uniq[@Vertex_Counter.index(1)]
 @End_Edge_Position = (@Raw_Vertex.index(@Start_Vertex) / 2)
 @Initial_Edge = @Raw_Edges[@End_Edge_Position]
else
 @End_Edge_Position = 0
 @Initial_Edge = @Raw_Edges[@End_Edge_Position]
 @Start_Vertex = @Initial_Edge.start
end


print 'Make RailWay Along - Debug point 3/10 ok' #DEBUG POINT <<< ################################## <<<

#MAKE SERIAL POINTS ARRAY (SERIALIZE PROCESS)
@Current_Vertex = @Start_Vertex
@Serial_Points = []
@Serial_Points << @Current_Vertex.position

@Final_edge = (@Raw_Edges.length - 1)
@Last_Edge = ""
@Counter = 0
@State = ""
#------------------------------------------------------------------------------------#
until ((@Counter > (@Raw_Vertex.uniq.length-1)) or (@State == "End of path"))

@Current_Edge = 0
@State = "Not found"
 until ((@Current_Edge > @Final_edge) or (@State == "Found"))

  if ((@Current_Edge != @Last_Edge) and (@Raw_Edges[@Current_Edge].used_by? @Current_Vertex))
  @State = "Found"

  @Current_Vertex = @Raw_Edges[@Current_Edge].other_vertex @Current_Vertex

   @Serial_Points << @Current_Vertex.position

  @Last_Edge = @Current_Edge
  end

 @Current_Edge += 1
 end

 if (@State == "Not found")
 @State = "End of path"
 end

@Counter += 1
end



#REMOVE POINTS ON STRAIGHT LINES
@Points_To_Remove = []
@Final_Point = (@Serial_Points.length-1)
#------------------------------------------------------------------------------------
@Serial_Points.each do |position|

#Point ID in array
@ID = @Serial_Points.index(position)

#IF NOT FIRST OR LAST POINT
 if ((@ID != 0) and (@ID != @Final_Point))

#POINTS (3)
@Current_Point = position
@Ahead_Point = @Serial_Points[@ID+1]
@Back_Point = @Serial_Points[@ID-1]

#VECTORS (2)
@Last_Vector = (@Back_Point.vector_to( @Current_Point )).normalize
@Next_Vector = (@Current_Point.vector_to( @Ahead_Point )).normalize

 #Register point to delete
 if (@Last_Vector == @Next_Vector)
  @Points_To_Remove << @Current_Point
 end
end

end
#------------------------------------------------------------------------------------
@Serial_Points = @Serial_Points - @Points_To_Remove #Remove the unecessary points


#REVERSE PATH?
if (@Reverse == "yes") then @Serial_Points.reverse!; end

print 'Make RailWay Along - Debug point 4/10 ok' #DEBUG POINT <<< ################################## <<<

#MAKE POINTS AXES
@Points_Axes = [] # <- [Z, SIDE1, SIDE2, AVER, AHEAD]
@Curves_State = ""
@Final_Point = (@Serial_Points.length-1)

#------------------------------------------------------------------------------------
@Serial_Points.each do |position|
#Point ID in array
@ID = @Serial_Points.index(position)

#POINTS (3)
@Current_Point = @Serial_Points[@ID]
@Ahead_Point = @Serial_Points[@ID+1]
@Back_Point = @Serial_Points[@ID-1]


#IF FIRST POINT - # - # - # - # - # - # - # - # - # - # - # - # - #
 if @ID == (0)
@Up_Vector_Basis = Z_AXIS
@Average_Vector = (@Current_Point.vector_to( @Ahead_Point )).normalize
@Ahead_Vector = @Average_Vector

if (@Average_Vector[2].abs == 1)
UI.messagebox("Aborted, first edge direction cannot be on Z_AXIS.")
raise Exception.new("Aborted, 1st edge on Z_AXIS.")
end

@Side_Vector1 = (@Average_Vector.cross(@Up_Vector_Basis)).normalize
@Side_Vector2 = @Side_Vector1.reverse
 
@Up_Vector = (@Average_Vector.cross(@Side_Vector1)).normalize
if (@Up_Vector.z < 0) then @Up_Vector.reverse!; end

#Register axes for current point
@Points_Axes << [@Up_Vector, @Side_Vector1, @Side_Vector2, @Average_Vector, @Ahead_Vector] # <<<<<<<<<< REGISTER (first point)
else

#IF FINAL POINT - # - # - # - # - # - # - # - # - # - # - # - # - #
 if @ID == (@Final_Point)
@Average_Vector = (@Back_Point.vector_to( @Current_Point )).normalize
@Ahead_Vector = @Average_Vector

@Up_Vector_Prediction = @Up_Vector_H
if ((@Up_Vector_Prediction.angle_between @Up_Vector_H).radians > 90) then @Up_Vector_Prediction.reverse!; end

@Side_Vector1 = (@Average_Vector.cross(@Up_Vector_Prediction)).normalize
if ((@Side_Vector1.angle_between @Side_Vector1_H).radians > 90) then @Side_Vector1.reverse!; end
@Side_Vector2 = @Side_Vector1.reverse

@Up_Vector = (@Average_Vector.cross(@Side_Vector1)).normalize
if ((@Up_Vector.angle_between @Up_Vector_H).radians > 90) then @Up_Vector.reverse!; end
 
#Register axes for current point
@Points_Axes << [@Up_Vector, @Side_Vector1, @Side_Vector2, @Average_Vector, @Ahead_Vector] # <<<<<<<<<< REGISTER (final point)
 else # - # - # - # - # - # - # - # - # - # - # - # - # - #

#AVERAGE VECTOR
@Serial_Vector1 = (@Back_Point.vector_to( @Current_Point )).normalize
@Serial_Vector2 = (@Current_Point.vector_to( @Ahead_Point )).normalize
@Average_Vector = (@Serial_Vector1 + @Serial_Vector2).normalize

#PROJECTION VECTORS (2)
@Ahead_Vector = (@Current_Point.vector_to( @Ahead_Point )).normalize
@Back_Vector = (@Current_Point.vector_to( @Back_Point )).normalize

if ((@Ahead_Vector.angle_between @Back_Vector).radians < 120)
@Curves_State = "BAD"
@pt = entities.add_cpoint @Current_Point
end

#TOP PROJECTIONS (2)
@Ahead_Top_Project_Vector = Geom::Vector3d.new(@Ahead_Vector[0], @Ahead_Vector[1], 0).normalize
@Back_Top_Project_Vector = Geom::Vector3d.new(@Back_Vector[0], @Back_Vector[1], 0).normalize
@Curve_Angle = (@Ahead_Top_Project_Vector.angle_between @Back_Top_Project_Vector).radians

#Define up and side vectors
if (@Curve_Angle == 180 or @Curve_Angle == 0 or @Ahead_Top_Project_Vector == [0,0,0] or @Back_Top_Project_Vector == [0,0,0])
 @Up_Vector = (@Back_Vector + @Ahead_Vector).normalize

     #Attempt to solve precision issue with rlly smooth curves
      if (@Up_Vector == [0,0,0]) then @Up_Vector = @Up_Vector_H; end

 if ((@Up_Vector.angle_between @Up_Vector_H).radians > 90) then @Up_Vector.reverse!; end
 @Side_Vector1 = (@Average_Vector.cross(@Up_Vector)).normalize
 if ((@Side_Vector1.angle_between @Side_Vector1_H).radians > 90) then @Side_Vector1.reverse!; end
 @Side_Vector2 = @Side_Vector1.reverse

else

 @Side_Vector1 = (@Back_Top_Project_Vector + @Ahead_Top_Project_Vector).normalize
 
      if (@Side_Vector1 == [0,0,0]) #Attempt to solve precision issue with rlly smooth curves
       @Side_project1 = @Ahead_Top_Project_Vector.cross(@Up_Vector_H)
       @Side_project2 = Back_Top_Project_Vector.cross(@Up_Vector_H)

       @Current_Lateral = [@Current_Point, @Side_project1]
       @Back_Lateral = [@Back_Point, @Side_project2]

       @Intersection = Geom.intersect_line_line(@Current_Lateral, @Back_Lateral)
       @Side_Vector1 = @Current_Point.vector_to(@Intersection).normalize
      end
 
 if ((@Side_Vector1.angle_between @Side_Vector1_H).radians > 90) then @Side_Vector1.reverse!; end
 @Up_Vector = (@Average_Vector.cross(@Side_Vector1)).normalize
 if ((@Up_Vector.angle_between @Up_Vector_H).radians > 90) then @Up_Vector.reverse!; end
 @Side_Vector2 = @Side_Vector1.reverse
end

#Register axes for current point
@Points_Axes << [@Up_Vector, @Side_Vector1, @Side_Vector2, @Average_Vector, @Ahead_Vector] # <<<<<<<<<< REGISTER (middle points)
 end
end

#Past vectors historic
@Up_Vector_H = @Up_Vector
@Side_Vector1_H = @Side_Vector1
end
#------------------------------------------------------------------------------------
if (@Curves_State == "BAD")
@Continue = UI.messagebox("There are hard curves (c < 120°). Consider smoothing the path.\nContinue operation?", type = MB_YESNO)
if (@Continue == IDNO) then raise Exception.new("Operation aborted due to hard curves (c < 120°)."); end
end

print 'Make RailWay Along - Debug point 5/10 ok' #DEBUG POINT <<< ################################## <<<

# Outer group -----------------------------------------------------------------------
@RailWay = Sketchup.active_model.entities.add_group
#------------------------------------------------------------------------------------


#MAKE SIDES PATHS -------------------------------------
Sketchup.active_model.start_operation "Railroad along op.1"

(#Initial definitions for side paths
@Rails_Offset = (@Rails_distance + @HW)/2.00
@Half_Timber = (@Timber_Length / 2.00)

#Intersections status
@Status_1 = ""
@Status_2 = ""
@Status_Up = ""

#Sides Groups
@Side1_ref = @RailWay.entities.add_group
@Side2_ref = @RailWay.entities.add_group

#Initial historic self reference
@Side_Point1_H = @Serial_Points[0].offset(@Points_Axes[0][1], @Rails_Offset)
@Side_Point2_H = @Serial_Points[0].offset(@Points_Axes[0][2], @Rails_Offset)
@Up_Point_H = @Serial_Points[0].offset(@Points_Axes[0][0], @Rails_Offset)
)

@Serial_Points.each do |position|
#Point ID in array
@ID = @Serial_Points.index(position)
@Current_Point = @Serial_Points[@ID]

#Get axes[Z, SIDE1, SIDE2, AHEAD]
@Up_Vector = @Points_Axes[@ID][0]
@Side1_Vector = @Points_Axes[@ID][1]
@Side2_Vector = @Points_Axes[@ID][2]
@Ahead_Vector = @Points_Axes[@ID][3]

#Get current points
@Side_Point1 = @Current_Point.offset(@Side1_Vector, @Rails_Offset)
@Side_Point2 = @Current_Point.offset(@Side2_Vector, @Rails_Offset)
@Up_Point = @Current_Point.offset(@Up_Vector, @Rails_Offset)


#Make lines
@Side_Line_1 = @Side1_ref.entities.add_line(@Side_Point1_H, @Side_Point1)
@Side_Line_2 = @Side2_ref.entities.add_line(@Side_Point2_H, @Side_Point2)
#@Up_Line = entities.add_line(@Up_Point_H, @Up_Point) # <<<<< TEMPORARY DEBUG


#Check intersections -----------------------
if (@ID != 0)
@Side1_line1 = [@Current_Point, (@Current_Point.vector_to @Side_Point1)]
@Side1_line2 = [@Last_Point, (@Last_Point.vector_to @Side_Point1_H)]
@INT1 = Geom.intersect_line_line(@Side1_line1, @Side1_line2)
if ((@INT1 != nil) and (@Current_Point.distance(@INT1) < @Half_Timber)) then @Status_1 = "BAD"; @pt = entities.add_cpoint @Current_Point; end

@Side2_line1 = [@Current_Point, (@Current_Point.vector_to @Side_Point2)]
@Side2_line2 = [@Last_Point, (@Last_Point.vector_to @Side_Point2_H)]
@INT2 = Geom.intersect_line_line(@Side2_line1, @Side2_line2)
if ((@INT2 != nil) and (@Current_Point.distance(@INT2) < @Half_Timber)) then @Status_2 = "BAD"; @pt = entities.add_cpoint @Current_Point; end

@Up_line1 = [@Current_Point, (@Current_Point.vector_to @Up_Point)]
@Up_line2 = [@Last_Point, (@Last_Point.vector_to @Up_Point_H)]
@INT3 = Geom.intersect_line_line(@Up_line1, @Up_line2)
if ((@INT3 != nil) and (@Current_Point.distance(@INT3) < @Half_Timber)) then @Status_Up = "BAD"; @pt = entities.add_cpoint @Current_Point; end
end
#-----------------------------------------------

#Make points historic
@Side_Point1_H = @Side_Point1
@Side_Point2_H = @Side_Point2
@Up_Point_H = @Up_Point
@Last_Point = @Current_Point
end

Sketchup.active_model.commit_operation
#------------------------------------------------------------------------------------

#Intersection advise ----------------------------------------------------------------
if (@Status_1 == "BAD") or (@Status_2 == "BAD") or (@Status_Up == "BAD")
 UI.messagebox("Operation aborted!\nThere are intersections.\nRails distance is too large or curves radius are too low.")
 Sketchup.undo
 raise Exception.new("Operation aborted due to intersections.")
end

if (@Make_Ref == "No")
@Side1_ref.erase!
@Side2_ref.erase!
end
#------------------------------------------------------------------------------------

print 'Make RailWay Along - Debug point 6/10 ok' #DEBUG POINT <<< ################################## <<<


Sketchup.active_model.start_operation "Railroad along op.2"
#MAKE RAIL PROFILE ------------------------------------------------------ <<<<<<<<<<<
@Rails1 = @RailWay.entities.add_group
@Rails2 = @RailWay.entities.add_group
@Rail_Profile = @RailWay.entities.add_group

@Points = []
@Points[0] = [(@BW/2.0), 0, 0]
@Points[1] = [(@BW/2.0), 0, @BD - (((@BW/2.0)-(@W/2.0))/4.0)]
@Points[2] = [(@W/2.0), 0, @BD]
@Points[3] = [(@W/2.0), 0, @BD + @FD]
@Points[4] = [(@HW/2.0), 0, @BD + @FD + (((@HW-@W)/2.0)/4.0)]
@Points[5] = [(@HW/2.0), 0, @HT-(@HD/4.0)]
@Points[6] = [((@HW/2.0)-(@HW/8.0)), 0, @HT]

@Points[14] = [(@BW/2.0), 0, 0]
@Points[13] = [-(@BW/2.0), 0, 0]
@Points[12] = [-(@BW/2.0), 0, @BD - (((@BW/2.0)-(@W/2.0))/4.0)]
@Points[11] = [-(@W/2.0), 0, @BD]
@Points[10] = [-(@W/2.0), 0, @BD + @FD]
@Points[9] = [-(@HW/2.0), 0, @BD + @FD + (((@HW-@W)/2.0)/4.0)]
@Points[8] = [-(@HW/2.0), 0, @HT-(@HD/4.0)]
@Points[7] = [-((@HW/2.0)-(@HW/8.0)), 0, @HT]

@Silhouete = @Rail_Profile.entities.add_line @Points
@Profile_def = @Rail_Profile.definition
#------------------------------------------------------------------------------------

@Last_Point = (@Serial_Points.length-1)
@Last_ID = 0

#Rails side 1 --------------------------------------
@Serial_Points.each do |position|
@ID = @Serial_Points.index(position)

#Add new rail profile
@Rail_Copy = @Rails1.entities.add_instance @Profile_def, Sketchup.active_model.axes.transformation

#Get rails profile points
@Current_Points = []
@Rail_Copy.entities.each do |edge|
@Current_Points << Geom::Point3d.new(edge.end.position)
end

#Put rail profile points on correct position and orientarion
@Side1_Point = @Serial_Points[@ID].offset(@Points_Axes[@ID][1], @Rails_Offset)
@Side1_Point = @Side1_Point.offset(@Points_Axes[@ID][0], @Up_Offset)

@Define_axes = Geom::Transformation.axes(@Side1_Point, @Points_Axes[@ID][1].normalize, @Points_Axes[@ID][4].normalize, @Points_Axes[@ID][0].normalize)
@Current_Points.each do |point|
point.transform!(@Define_axes)
end

#Make ends face
#I know its weird af to convert to string but sketchup was saying (1==1) as false. I swear :|
 if (((@ID == 0) or (@ID.to_s == (@Last_Point).to_s)) and (@MODE == "OPEN")) then @Rails1.entities.add_face @Current_Points; end

#Make lateral faces
 if ( (@ID != 0) or ((@MODE == "LOOP") and (@Last_ID.to_s == (@Last_Point-1).to_s)) )
  @Current_Points.each do |point|
   @SubID = @Current_Points.index(point)

   @Rails1.entities.add_face @Current_Points[@SubID-1], @Current_Points[@SubID], @Last_Points[@SubID-1]
   @Rails1.entities.add_face @Current_Points[@SubID], @Last_Points[@SubID-1], @Last_Points[@SubID]
  end
 end

#Make reference for next points
@Last_Points = @Current_Points
@Rail_Copy.erase!

@Last_ID = @ID
end

#Rails side 2 --------------------------------------
@Serial_Points.each do |position|
@ID = @Serial_Points.index(position)

#Add new rail profile
@Rail_Copy = @Rails2.entities.add_instance @Profile_def, Sketchup.active_model.axes.transformation

#Get rails profile points
@Current_Points = []
@Rail_Copy.entities.each do |edge|
@Current_Points << Geom::Point3d.new(edge.end.position)
end

#Put rail profile points on correct position and orientarion
@Side2_Point = @Serial_Points[@ID].offset(@Points_Axes[@ID][2], @Rails_Offset)
@Side2_Point = @Side2_Point.offset(@Points_Axes[@ID][0], @Up_Offset)

@Define_axes = Geom::Transformation.axes(@Side2_Point, @Points_Axes[@ID][1].normalize, @Points_Axes[@ID][4].normalize, @Points_Axes[@ID][0].normalize)
@Current_Points.each do |point|
point.transform!(@Define_axes)
end

#Make ends face
#I know its weird af to convert to string but sketchup was saying (1==1) as false. I swear :|
 if (((@ID == 0) or (@ID.to_s == (@Last_Point).to_s)) and (@MODE == "OPEN")) then @Rails2.entities.add_face @Current_Points; end

#Make lateral faces
 if ( (@ID != 0) or ((@MODE == "LOOP") and (@Last_ID.to_s == (@Last_Point-1).to_s)) )
  @Current_Points.each do |point|
   @SubID = @Current_Points.index(point)

   @Rails2.entities.add_face @Current_Points[@SubID-1], @Current_Points[@SubID], @Last_Points[@SubID-1]
   @Rails2.entities.add_face @Current_Points[@SubID], @Last_Points[@SubID-1], @Last_Points[@SubID]
  end
 end

#Make reference for next points
@Last_Points = @Current_Points
@Rail_Copy.erase!

@Last_ID = @ID
end

@Rail_Profile.erase!

#Hide rails edges #maybe should use (edge.soft = true)
@Rails1.entities.to_a.grep(Sketchup::Edge).each { |edge| edge.visible = false }
@Rails2.entities.to_a.grep(Sketchup::Edge).each { |edge| edge.visible = false }

@RailWay.material = @Rail_Color
#------------------------------------------------------------------------- <<<<<<<<<<<
Sketchup.active_model.commit_operation


print 'Make RailWay Along - Debug point 7/10 ok' #DEBUG POINT <<< ################################## <<<

#------------------------------------------------------------------------------------
@Timbers_Ammount = ((@Path_Length / @Timber_Spacing).to_i + 1)

@Point = 0
@Last_Position = @Serial_Points[@Point]
@Rest_of_Travel = @Timber_Spacing
@Rest_of_Edge = @Serial_Points[@Point].distance(@Serial_Points[@Point+1])

@Timber_Locations = [] # <<< [[position, ahead, up], [position, ahead, up]]
@Timber_Locations << [@Serial_Points[@Point], @Points_Axes[@Point][4], @Points_Axes[@Point][0], @Point]
@Timber_num = 0
#------------------------------------------------------------------------------------
until (@Timber_num == (@Timbers_Ammount - 1)) or (@Point > (@Serial_Points.length - 2))

   if (@Rest_of_Travel <= @Rest_of_Edge) #TRAVEL END ON THIS EDGE? <<<<<

      #Calculate timber position
      @Position = @Last_Position.offset(@Points_Axes[@Point][4], @Rest_of_Travel)
      @Last_Position = @Position #Update position historic

      @Rest_of_Edge -= @Rest_of_Travel #Update rest of edge
      @Rest_of_Travel = @Timber_Spacing #Reset travel goal since its ended

      #Calculate timber direction
      @Edge_Length = @Serial_Points[@Point].distance(@Serial_Points[@Point+1])
      @Factor = ((@Edge_Length - @Rest_of_Edge) / @Edge_Length)


      if (@Transition == "Linear") #TRANSITION MODE
      #Direct direction change (Option 1)
      @Direction = @Points_Axes[@Point][4]
      @Up_Vec = @Points_Axes[@Point][0]
      else
      #Gradient direction change (Option 2)
      @Direction = Geom::Vector3d.linear_combination(1-@Factor, @Points_Axes[@Point][4], @Factor, @Points_Axes[@Point+1][4])
      @Up_Vec = Geom::Vector3d.linear_combination(1-@Factor, @Points_Axes[@Point][0], @Factor, @Points_Axes[@Point+1][0])
      end
      
      
      #Register timber position values
      @Timber_Locations << [@Position, @Direction, @Up_Vec, @Point]
      @Timber_num += 1
   
   else #TRAVEL DON'T END ON CURRENT EDGE. <<<<<

      @Rest_of_Travel -= @Rest_of_Edge #Update rest of travel
      @Point += 1 #Pass to next edge
      @Last_Position = @Serial_Points[@Point]
      @Rest_of_Edge = @Serial_Points[@Point].distance(@Serial_Points[@Point+1]) #Reset rest of edge to next edge length

   end
end
#------------------------------------------------------------------------------------

print 'Make RailWay Along - Debug point 8/10 ok' #DEBUG POINT <<< ################################## <<<

Sketchup.active_model.start_operation "Railroad along op.3"
(#MAKE TIMBER -------------------------------------------------------------------------
model = Sketchup.active_model
entities = model.active_entities
@Timber = @RailWay.entities.add_group

#MAKE TIMBER BEVEL?
if (@Timber_Bevel == "Yes")
 #Define bevel value
 if (@Timber_Thickness < @Timber_Width) then @Bevel = @Timber_Thickness/@Bevel_Factor; else @Bevel = @Timber_Width/@Bevel_Factor; end
 @transf1 = Geom::Transformation.new([0, 0, -@Bevel])
 @transf2 = Geom::Transformation.new([0, 0, @Bevel])

 #UP SIDE
 @Pts = []
 @Pts[0] = [(@Timber_Length/2.00 - @Bevel), (@Timber_Width/2.00 - @Bevel), 0]
 @Pts[1] = [-(@Timber_Length/2.00 - @Bevel), (@Timber_Width/2.00 - @Bevel), 0]
 @Pts[2] = [-(@Timber_Length/2.00 - @Bevel), -(@Timber_Width/2.00 - @Bevel), 0]
 @Pts[3] = [(@Timber_Length/2.00 - @Bevel), -(@Timber_Width/2.00 - @Bevel), 0]
 @F1 = @Timber.entities.add_face @Pts

 #DOWNSIDE
 @Pts = []
 @Pts[0] = [(@Timber_Length/2.00 - @Bevel), (@Timber_Width/2.00 - @Bevel), -@Timber_Thickness]
 @Pts[1] = [-(@Timber_Length/2.00 - @Bevel), (@Timber_Width/2.00 - @Bevel), -@Timber_Thickness]
 @Pts[2] = [-(@Timber_Length/2.00 - @Bevel), -(@Timber_Width/2.00 - @Bevel), -@Timber_Thickness]
 @Pts[3] = [(@Timber_Length/2.00 - @Bevel), -(@Timber_Width/2.00 - @Bevel), -@Timber_Thickness]
 @F2 = @Timber.entities.add_face @Pts
end

#BASIC TIMBER FORM
 @Pt1 = [(@Timber_Length/2.00), (@Timber_Width/2.00), 0]
 @Pt2 = [-(@Timber_Length/2.00), (@Timber_Width/2.00), 0]
 @Pt3 = [(@Timber_Length/2.00), -(@Timber_Width/2.00), 0]
 @Pt4 = [-(@Timber_Length/2.00), -(@Timber_Width/2.00), 0]

 @Line1 = @Timber.entities.add_line(@Pt1, @Pt2)
 @Line2 = @Timber.entities.add_line(@Pt3, @Pt4)
 @Line3 = @Timber.entities.add_line(@Pt1, @Pt3)
 @Line4 = @Timber.entities.add_line(@Pt2, @Pt4)
 @Base_Countourn1 = @Line1.all_connected

@Base_Face = @Timber.entities.add_face(@Base_Countourn1)
if (@Base_Face.normal != Z_AXIS) then @Base_Face.reverse!; end
@Base_Face.pushpull(-@Timber_Thickness, false)

#Finish bevel
if (@Timber_Bevel == "Yes")
 @Timber.entities.transform_entities(@transf1, @Base_Countourn1)
 
  #DOWNSIDE base Countourn2
 @Pt1 = [(@Timber_Length/2.00), (@Timber_Width/2.00), -@Timber_Thickness]
 @Pt2 = [-(@Timber_Length/2.00), (@Timber_Width/2.00), -@Timber_Thickness]
 @Pt3 = [(@Timber_Length/2.00), -(@Timber_Width/2.00), -@Timber_Thickness]
 @Pt4 = [-(@Timber_Length/2.00), -(@Timber_Width/2.00), -@Timber_Thickness]

 @Line1 = @Timber.entities.add_line(@Pt1, @Pt2)
 @Line2 = @Timber.entities.add_line(@Pt3, @Pt4)
 @Line3 = @Timber.entities.add_line(@Pt1, @Pt3)
 @Line4 = @Timber.entities.add_line(@Pt2, @Pt4)
 
 Sketchup.active_model.selection.clear
 Sketchup.active_model.selection.add(@Line1, @Line2, @Line3, @Line4)
 @Timber.entities.transform_entities(@transf2, Sketchup.active_model.selection)
 Sketchup.active_model.selection.clear
 Sketchup.active_model.selection.add(@Raw_Edges)
end

@Timber = @Timber.to_component
@Timber.name = "Timber"
@T_def = @Timber.definition
#------------------------------------------------------------------------------------
)

print 'Make RailWay Along - Debug point 9/10 ok' #DEBUG POINT <<< ################################## <<<

@All_Timbers = @RailWay.entities.add_group

@Timber_Locations.each do |timber|
   #entities.add_cpoint timber[0] #Timber position indicator
   @Timber_Position = timber[0].offset(timber[2], @Up_Offset)
   
   @Timber_Copy = @All_Timbers.entities.add_instance @T_def, (Geom::Transformation.axes(@Timber_Position, (timber[1].cross(timber[2])).normalize, timber[1], timber[2]))
   
   @Timber_Copy.name = "Timber"
   @Timber_Copy.definition = @T_def
   @Timber_Copy.material = @Timber_Color
end

@Timber.erase! #Delete original timber on origin point
#------------------------------------------------------------------------------------
Sketchup.active_model.commit_operation


print 'Make RailWay Along - Debug point 10/10 ok' #DEBUG POINT <<< ################################## <<<

#------------------------------------------------------------------------------------
@end_time = Time.now
@Total_Time = @end_time - @start_time

@Text1 = ("Success. (" + @Total_Time.round(3).to_s + "s)")
@Text2 = ("Path length: " + (@Path_Length / (1.00.m)).round(2).to_s + "m")
@Text3 = ("Timbers: " + @Timbers_Ammount.to_s + " | Edges: " + @Edges_Ammount.to_s)
@Text4 = ("Construction speed: " + ((@Path_Length / (1.00.m)) / @Total_Time).round(2).to_s + "m/s")

print (@Text1 + "\n" + @Text2 + "\n" + @Text3 + "\n" + @Text4) + "\n\n"
Taxsola.soundeffect(1) #0: Installation | 1: Success | 2: Notification
#------------------------------------------------------------------------------------
end

	end
end
#-------------------------------------------------------------------------------