=begin
#-------------------------------
Author: Tax, 2021
Description:
#-------------------------------
=end


#-------------------------------------------------------------------------------
module Taxsola
	module Tax_Engineering

def self.roller_dialog

(#BASIC INITIAL HANDLERS -----------------------------------------------------
model = Sketchup.active_model
entities = model.active_entities

@selection = model.selection.to_a
@edges = @selection.grep(Sketchup::Edge).uniq
@Unchecked_Edges = @edges
@Checked_Edges = []
@Segments = []

@MODE = ""
if (@edges.length == 0)
print ("Roller Chain along debug: RCAxD.000 - No edges selected. Default mode operation.")
UI.messagebox("No path selected.\nDefault mode operation.")
@MODE = "no path"
else
print ("Roller Chain along debug: RCAxD.001 - Follow path along operation.")
@MODE = "follow path"
end
#-----------------------------------------------------------------------------
)


if (@MODE == "follow path")

(#VERTEX TYPE CLASSIFICATION --------------------------------------------------
@Raw_vertexes = []
@edges.each { |edge| @Raw_vertexes << edge.start; @Raw_vertexes << edge.end;}

@Single_vertexes = @Raw_vertexes.uniq
@Vertex_Counter = @Single_vertexes.collect { |v| @Raw_vertexes.count(v) }


@End_Vertexes = []
@Middle_Vertexes = [] 
@Intersection_Vertexes = []

@ID = 0
@Vertex_Counter.each do |counter|
	if (counter == 1) then @End_Vertexes << @Single_vertexes[@ID]; end
	if (counter == 2) then @Middle_Vertexes << @Single_vertexes[@ID]; end
	if (counter > 2) then @Intersection_Vertexes << @Single_vertexes[@ID]; end
	@ID += 1
end
#-----------------------------------------------------------------------------
print 'Make Roller Chain - VERTEX TYPE CLASSIFICATION OK'
)

#SEGMENT MAKER ----------------------------------------------------
def self.make_segment(initialvertex, initialedge)
@Segment_Vertexes = []

@Vertex = initialvertex
@Edge = initialedge

	while ((@Middle_Vertexes.include?(@Vertex) == true) || (@Vertex == initialvertex)) && (@Checked_Edges.include?(@Edge) == false)
		@Segment_Vertexes << @Vertex
		@Checked_Edges << @Edge
		@Unchecked_Edges -= [@Edge]

		#Define new vertex
		@Vertex = @Edge.other_vertex @Vertex

		#Finds new edge
		@n = 0
		loop do
			if (@Unchecked_Edges.length == 0) then break end
			if (@Unchecked_Edges[@n].used_by? @Vertex) && (@Unchecked_Edges[@n] != @Edge) then @Edge = @Unchecked_Edges[@n]; break; end
			if (@n == @Unchecked_Edges.length-1) then break end
			@n += 1
		end
	end
@Segment_Vertexes << @Vertex

return @Segment_Vertexes
end
#------------------------------------------------------------------

#1st round - segments making from intersections -------------------------------
@Int_Seg_Ind = []
@Unchecked_Edges_Cache = @Unchecked_Edges

if (@Intersection_Vertexes.length > 0)
	@Intersection_Vertexes.each do |vertex|
		@Edges_Cache = []
		@Unchecked_Edges_Cache.each do |edge| #Didn't know about '.edges' method. Could work better.
			if (edge.used_by? vertex)
				@Edges_Cache << edge
				@Unchecked_Edges_Cache -= [edge]
			end
		end

		@Int_Seg_Ind << @Edges_Cache
	end

	@Int_Seg_Ind.each do |intersection|
		intersection.each do |segment_ind|
			if (@Checked_Edges.include?(segment_ind) == false)
				@Segments << self.make_segment(@Intersection_Vertexes[@Int_Seg_Ind.index(intersection)],segment_ind)
			end
		end
	end

end
print 'Make Roller Chain - Segments making (1st round OK)'
#------------------------------------------------------------------------------


#2nd round - segments making from end points ----------------------------------
if (@End_Vertexes.length > 0)
	@End_Vertexes.each do |vertex|
		@Unchecked_Edges.each do |edge|
			if (edge.used_by? vertex)
				@Segments << self.make_segment(vertex, edge)
				break
			end
		end
	end
end
print 'Make Roller Chain - Segments making (2nd round OK)'
#------------------------------------------------------------------------------


#3rd round - segments making loose loops --------------------------------------
until (@Unchecked_Edges.length == 0)
@Segments << self.make_segment(@Unchecked_Edges[0].start,@Unchecked_Edges[0])
end
print 'Make Roller Chain - Segments making (3rd round OK)'
#------------------------------------------------------------------------------


#Remove aligned (in line) vertexes
@Seg_Num = 0
@Segments.each do |segment|
	@To_Remove = []
	for n in 1..(segment.length-2)
		if @Middle_Vertexes.include?(segment[n])
			@Last_Vector = segment[n-1].position.vector_to(segment[n].position)
			@Next_Vector = segment[n].position.vector_to(segment[n+1].position)
			@Last_Vector.length = 10000 #High value to prevent precision issues
			@Next_Vector.length = 10000

			if (@Last_Vector == @Next_Vector) then @To_Remove << segment[n]; end
		end
	end
	@Segments[@Seg_Num] = segment - @To_Remove
	@Seg_Num += 1
end
print 'Make Roller Chain - Remove aligned OK'
#---------------------------------

@msg = "There are " + @Segments.length.to_s + " path segments on selection.\nThey are going to be flatted on axial planes."
	if UI.messagebox(@msg, type = MB_OKCANCEL) != IDOK
		UI.messagebox('Operation canceled.', type = MB_OK)
		raise Exception.new("Roller Chain along error: RCAxE.000 - Operation canceled.")
	end


#Vertex to point conversion
@New_Segments = []
@Segments.each do |segment|
	@Seg_Points = []
	segment.each {|vertex| @Seg_Points << vertex.position}
	@New_Segments << @Seg_Points
end
@Segments = @New_Segments
#---------------------------


(#Find axis to flat
@Flat_Axis = []

@Segments.each do |segment|
	@Seg_X = []
	@Seg_Y = []
	@Seg_Z = []

	segment.each do |point|
		@Seg_X << point[0]
		@Seg_Y << point[1]
		@Seg_Z << point[2]
	end

	@Max_X = @Seg_X.max
	@Max_Y = @Seg_Y.max
	@Max_Z = @Seg_Z.max

	@Min_X = @Seg_X.min
	@Min_Y = @Seg_Y.min
	@Min_Z = @Seg_Z.min

	@Delta_X = @Max_X - @Min_X
	@Delta_Y = @Max_Y - @Min_Y
	@Delta_Z = @Max_Z - @Min_Z

	@Deltas = []
	@Deltas << @Delta_X << @Delta_Y << @Delta_Z

	@Min_Var_Axis = @Deltas.index(@Deltas.min) #0, 1, 2 (X, Y, Z)

	@Flat_Axis << @Min_Var_Axis
end
#-------------------------------------------------------
)


(#Flatting segments
for segment in 0..(@Segments.length-1)
@To_flat = @Flat_Axis[segment] #@To_flat = 0, 1, 2 (X, Y, Z)
@Flat_coord = @Segments[segment][0][@To_flat]

	for point in 0..(@Segments[segment].length-1)
		@Segments[segment][point][@To_flat] = @Flat_coord
	end
end
#------------------ '@Segments' is now flatted on better fitting axial planes
)

end



#Default chain dimensions values
@Choosen_Preset = "Custom"
@Pitch = 8.00
@RollerDiameter = 4.70
@RollerWidth = 4.80
@RollerThickness = 0.20
@PinDiameter = 3.25
@PlateThickness = 1.00
@Detail = 32
@RollerOutRatio = 1.60
@R1R2Ratio = 1.00
@Smooth = "Yes"
@bevelfactor = 0.20
@Links = 10
@MakeRef2 = "Yes"
#-------------------------------

#Presets setup
@Presets = {
'Custom': [],
'08A-1': [12.7,7.93,7.85,0.9875,3.98,1.5,1.45],
'10A-1': [15.875,10.15,9.55,1.265,5.09,2.03,1.35],
'12A-1': [19.05,11.91,12.65,1.4875,5.96,2.42,1.36],
'16A-1': [25.4,15.88,15.88,1.985,7.94,3.25,1.31],
'20A-1': [31.75,19.05,19.05,2.38,9.53,4,1.33],
'24A-1': [38.1,22.22,25.4,2.78,11.1,4.8,1.58],
'28A-1': [44.45,25.4,25.4,3.175,12.7,5.6,1.65],
'32A-1': [50.8,28.58,31.75,3.5775,14.27,6.4,1.69],
'40A-1': [63.5,39.67,38.1,4.955,19.85,8,1.46],
'48A-1': [76.2,47.63,47.6,5.9575,23.8,9.5,1.51],
'04B-1': [6,4,2.8,0.5375,1.85,0.6,1.25],
'05B-1': [8,5,3,0.6725,2.31,0.8,1.42],
'#06B-1': [9.525,6.35,5.72,0.77,3.27,1.3,1.3],
'08B-1': [12.7,8.51,7.75,1.015,4.45,1.6,1.39],
'10B-1': [15.875,10.16,9.65,1.27,5.08,1.7,1.35],
'12B-1': [19.05,12.07,11.68,1.5875,5.72,1.85,1.34],
'16B-1': [25.4,15.88,17.02,1.9,8.28,3.1,1.31],
'20B-1': [31.75,19.05,19.56,2.215,10.19,3.5,1.33],
'24B-1': [38.1,25.4,25.4,2.6925,14.63,4.8,1.27],
'28B-1': [44.45,27.94,30.95,3.01,15.9,6,1.32],
'32B-1': [50.8,29.21,30.99,2.85,17.81,6,1.45],
'40B-1': [63.5,39.37,38.1,4.12,22.89,8,1.34],
'48B-1': [76.2,48.26,45.72,4.755,29.24,10,1.33]
}
@Presets_String = ''
@Presets.each {|key,value| @Presets_String += key.to_s; @Presets_String += '|'}
@Presets_String.chomp!'|'


@Verification = "NOT OK"
until @Verification == "OK"
#-------------------------------------------------------------------------------
@Verification = "OK"
prompts = ["Preset:", "Pitch:", "Roller Diameter:", "Roller Width:", "Roller Thickness:", "Pin Diameter:", "Plate Thickness:", "Detail (sides):", "Roller-Out Ratio:", "Rin-Rout Ratio:", "Smooth?", "Bevel Factor:", "Copies in row:", "Construct ref.?:"]
defaults = [@Choosen_Preset, @Pitch, @RollerDiameter, @RollerWidth, @RollerThickness, @PinDiameter, @PlateThickness, @Detail, @RollerOutRatio, @R1R2Ratio, @Smooth, @bevelfactor, @Links, @MakeRef2]
list = [@Presets_String, "", "", "", "", "", "", "", "", "", "Yes|No", "", "", "Yes|No"]
input = UI.inputbox(prompts, defaults, list, "Make Roller Chain")

#Get input values
if input
@Choosen_Preset, @Pitch, @RollerDiameter, @RollerWidth, @RollerThickness, @PinDiameter, @PlateThickness, @Detail, @RollerOutRatio, @R1R2Ratio, @Smooth, @bevelfactor, @Links, @MakeRef2 = input;
else
 raise Exception.new("Roller Chain along error: RCAxV.000 - Operation canceled.")
end

#LOAD PRESET
if (@Choosen_Preset != "Custom") #Get selected preset values into the variables
	puts ("Loading preset")
	@Preset_Values = @Presets[@Choosen_Preset.to_sym]
	@Pitch, @RollerDiameter, @RollerWidth, @RollerThickness, @PinDiameter, @PlateThickness, @RollerOutRatio = @Preset_Values
end


#Print input log
@Input_Log = ""
@Input_Log += @Choosen_Preset.to_s + ' | '
@Input_Log += @Pitch.to_s + ' | '
@Input_Log += @RollerDiameter.to_s + ' | '
@Input_Log += @RollerWidth.to_s + ' | '
@Input_Log += @RollerThickness.to_s + ' | '
@Input_Log += @PinDiameter.to_s + ' | '
@Input_Log += @PlateThickness.to_s + ' | '
@Input_Log += @Detail.to_s + ' | '
@Input_Log += @RollerOutRatio.to_s + ' | '
@Input_Log += @R1R2Ratio.to_s + ' | '
@Input_Log += @Smooth.to_s + ' | '
@Input_Log += @bevelfactor.to_s + ' | '
@Input_Log += @Links.to_s + ' | '
@Input_Log += @MakeRef2.to_s

print 'Make Roller Chain operation:'
print 'Input log: ' + @Input_Log


print 'Make Roller Chain - Debug point 1/7 ok'

	if @MODE == "no path"
		#Define camera view
		camera = Sketchup::Camera.new([-2, -2, 2], [0, 0, 0], Z_AXIS)
		view = Sketchup.active_model.active_view
		view.camera = camera
	end

(
#Get variables input
@Pitch = @Pitch.to_f
@RollerDiameter = @RollerDiameter.to_f
@RollerWidth = @RollerWidth.to_f
@RollerThickness = @RollerThickness.to_f
@PinDiameter = @PinDiameter.to_f
@PlateThickness = @PlateThickness.to_f
@Detail = @Detail.to_i
@RollerOutRatio = @RollerOutRatio.to_f
@R1R2Ratio = @R1R2Ratio.to_f
@Smooth = @Smooth.to_s
@bevelfactor = @bevelfactor.to_f
@Links = @Links.to_i
@MakeRef2 = @MakeRef2.to_s

#math
@BrushingWidth = (@RollerWidth + (2*@PlateThickness))
@PinLength = (@RollerWidth + (4*@PlateThickness))
@BrushingDiameter = (@RollerDiameter - (@RollerThickness*2))
@BrushingThickness = (@BrushingDiameter - @PinDiameter)/2

	if (@Smooth == "No") then @bevelfactor = 0.25; end
)

print 'Make Roller Chain - Debug point 2/7 ok'
(#VERIFICATIONS
if (@Pitch <= 0) or (@Pitch <= @RollerDiameter) or ((@RollerDiameter * @RollerOutRatio) > @Pitch)
UI.messagebox("error: RCAxV.001 - Pitch is to small or roller diameter is too large.")
warn("error: RCAxV.001")
@Verification = "NOT OK"
end

if (@PlateThickness <= 0) or (((@RollerDiameter * @RollerOutRatio)-@RollerDiameter)/2 <= 0)
UI.messagebox("error: RCAxV.002 - Plate thickness or roller out ratio is too low.")
warn("error: RCAxV.002")
@Verification = "NOT OK"
end

if (@Smooth == "Yes") and ( (((@RollerDiameter * @RollerOutRatio)-@RollerDiameter)/4) <= (@PlateThickness * @bevelfactor))
UI.messagebox("error: RCAxV.003 - Plate thickness or bevel factor is too large or roller out ratio is too low.")
warn("error: RCAxV.003")
@Verification = "NOT OK"
end

if (@RollerWidth <= 0)
UI.messagebox("error: RCAxV.004 - Roller width is too small.")
warn("error: RCAxV.004")
@Verification = "NOT OK"
end

if (@PinDiameter <= 0)
UI.messagebox("error: RCAxV.005 - Pin diameter is too small.")
warn("error: RCAxV.005")
@Verification = "NOT OK"
end

if (@BrushingDiameter <= @PinDiameter)
UI.messagebox("error: RCAxV.006 - No space for brushes; reduce pin diameter and/or roller thickness or increase roller diameter")
warn("error: RCAxV.006")
@Verification = "NOT OK"
end

if (@RollerDiameter <= 0) or (@RollerDiameter <= @BrushingDiameter) or (@RollerDiameter >= @Pitch)
UI.messagebox("error: RCAxV.007 - Roller diameter too short or too big.")
warn("error: RCAxV.007")
@Verification = "NOT OK"
end

if (@Detail <= 3)
UI.messagebox("error: RCAxV.008 - Min. detail level = 3.")
warn("error: RCAxV.008")
@Verification = "NOT OK"
end

if (@R1R2Ratio <= 0)
UI.messagebox("error: RCAxV.009 - Rin-Rout ratio is too low.")
warn("error: RCAxV.009")
@Verification = "NOT OK"
end

if (@Smooth == "Yes") and ((@bevelfactor <= 0) or (@bevelfactor > 0.5))
UI.messagebox("error: RCAxV.010 - Bevel factor out of (0 < x <= 0.5) range.")
warn("error: RCAxV.010")
@Verification = "NOT OK"
end


@OuterDiameter = @RollerDiameter * @RollerOutRatio
@R1 = @OuterDiameter / 2
@R2 = @R1 * @R1R2Ratio

if ( (@R1+@R2) < (@Pitch/2) )
UI.messagebox("error: RCAxV.011 - Rin-Rout ratio is to low.")
warn("error: RCAxV.011")
@Verification = "NOT OK"
end

begin
	@IntersectAngle = Math.acos(@Pitch / (2*(@R1+@R2))).radians
	@R2y = Math.sin(@IntersectAngle.degrees) * (@R1+@R2)
rescue
	puts ("Can't calculate intersection angle.")
else
	if ( ((@R2y - @R2) - (@PlateThickness * @bevelfactor)) < @PinDiameter/4 )
		UI.messagebox("error: RCAxV.012 - Pitch is too high or Rin-Rout ratio is to low")
		warn("error: RCAxV.012")
		@Verification = "NOT OK"
	end
end
)

puts ("Make Roller Chain - Verifications: " + @Verification.to_s)
end

print 'Make Roller Chain - Debug point 3/7 ok'
UI.messagebox("It can take a few minutes, please wait.")

#Timer start ------------------------------------------------------------------------------
@start_time = Time.now

(#Conversion
@Pitch = @Pitch.mm
@RollerDiameter = @RollerDiameter.mm
@RollerWidth = @RollerWidth.mm
@RollerThickness = @RollerThickness.mm
@PinDiameter = @PinDiameter.mm
@PlateThickness = @PlateThickness.mm
@BrushingWidth = @BrushingWidth.mm
@PinLength = @PinLength.mm
@BrushingDiameter = @BrushingDiameter.mm
@BrushingThickness = @BrushingThickness.mm
)


#MODELING

 def self.plates(pitch, brushingdiameter, rollerdiameter, platethickness, rolleroutratio, r1r2ratio, detail, bevelfactor, smooth)

@HoleRadius = (brushingdiameter/2)
@Bevel = platethickness * bevelfactor
@OuterDiameter = rollerdiameter * rolleroutratio
@R1 = @OuterDiameter / 2
@R2 = @R1 * r1r2ratio
@IntersectAngle = Math.acos(pitch / (2*(@R1+@R2))).radians
@R2y = Math.sin(@IntersectAngle.degrees) * (@R1+@R2)

#Make group
model = Sketchup.active_model
entities = model.active_entities
@Plate = Sketchup.active_model.entities.add_group

#outers of 1st side
@C1 = @Plate.entities.add_circle([pitch/2, 0, 0], [0, 1, 0] , (brushingdiameter/2), detail)
@C2 = @Plate.entities.add_circle([-pitch/2, 0, 0], [0, 1, 0] , (brushingdiameter/2), detail)
@OuterArc1 = @Plate.entities.add_arc [pitch/2, 0, 0], [-1,0,0], [0,1,0], @R1, @IntersectAngle.degrees, (360-@IntersectAngle).degrees, detail
@OuterArc2 = @Plate.entities.add_arc [-pitch/2, 0, 0], [1,0,0], [0,1,0], @R1, @IntersectAngle.degrees, (360-@IntersectAngle).degrees, detail
@SuperiorArc = @Plate.entities.add_arc [0, 0, @R2y], [1,0,0], [0,1,0], @R2, @IntersectAngle.degrees, (180-@IntersectAngle).degrees, detail/2
@InferiorArc = @Plate.entities.add_arc [0, 0, -@R2y], [1,0,0], [0,-1,0], @R2, @IntersectAngle.degrees, (180-@IntersectAngle).degrees, detail/2

@OuterPath = @SuperiorArc[0].all_connected
@PlateFace = @Plate.entities.add_face(@OuterPath)

@FTR1 = @Plate.entities.add_face(@C1)
status = @FTR1.erase!
@FTR2 = @Plate.entities.add_face(@C2)
status = @FTR2.erase!

if @PlateFace.normal.y == -1
@PlateFace.reverse!
end

status = @PlateFace.pushpull(platethickness, true)



if (@Smooth == "Yes")
model = Sketchup.active_model
entities = model.active_entities

@HoleRadius = (brushingdiameter/2) + @Bevel
@R1 = @R1 - @Bevel
@R2 = @R2 + @Bevel

#inners of 1st side
@C1s = @Plate.entities.add_circle([pitch/2, 0, 0], [0, 1, 0] , @HoleRadius, detail)
@C2s = @Plate.entities.add_circle([-pitch/2, 0, 0], [0, 1, 0] , @HoleRadius, detail)
@OuterArc1s = @Plate.entities.add_arc [pitch/2, 0, 0], [-1,0,0], [0,1,0], @R1, @IntersectAngle.degrees, (360-@IntersectAngle).degrees, detail
@OuterArc2s = @Plate.entities.add_arc [-pitch/2, 0, 0], [1,0,0], [0,1,0], @R1, @IntersectAngle.degrees, (360-@IntersectAngle).degrees, detail
@SuperiorArcs = @Plate.entities.add_arc [0, 0, @R2y], [1,0,0], [0,1,0], @R2, @IntersectAngle.degrees, (180-@IntersectAngle).degrees, detail/2
@InferiorArcs = @Plate.entities.add_arc [0, 0, -@R2y], [1,0,0], [0,-1,0], @R2, @IntersectAngle.degrees, (180-@IntersectAngle).degrees, detail/2

@OuterPaths = @SuperiorArcs[0].all_connected
@PlateFaces = @Plate.entities.add_face(@OuterPaths)

#Make bevel 1st side
transform = Geom::Transformation.new([0,@Bevel,0])
@Plate.entities.transform_entities(transform, @OuterPath)
@Plate.entities.transform_entities(transform, @C1)
@Plate.entities.transform_entities(transform, @C2)


#inners of 2nd side
@C1ss = @Plate.entities.add_circle([pitch/2, platethickness, 0], [0, 1, 0] , @HoleRadius, detail)
@C2ss = @Plate.entities.add_circle([-pitch/2, platethickness, 0], [0, 1, 0] , @HoleRadius, detail)
@OuterArc1ss = @Plate.entities.add_arc [pitch/2, platethickness, 0], [-1,0,0], [0,1,0], @R1, @IntersectAngle.degrees, (360-@IntersectAngle).degrees, detail
@OuterArc2ss = @Plate.entities.add_arc [-pitch/2, platethickness, 0], [1,0,0], [0,1,0], @R1, @IntersectAngle.degrees, (360-@IntersectAngle).degrees, detail
@SuperiorArcss = @Plate.entities.add_arc [0, platethickness, @R2y], [1,0,0], [0,1,0], @R2, @IntersectAngle.degrees, (180-@IntersectAngle).degrees, detail/2
@InferiorArcss = @Plate.entities.add_arc [0, platethickness, -@R2y], [1,0,0], [0,-1,0], @R2, @IntersectAngle.degrees, (180-@IntersectAngle).degrees, detail/2

@OuterPathss = @SuperiorArcss[0].all_connected
@PlateFacess = @Plate.entities.add_face(@OuterPathss)


#Outers of 2nd side
@R1 = @OuterDiameter / 2
@R2 = @R1 * r1r2ratio

@C12 = @Plate.entities.add_circle([pitch/2, platethickness, 0], [0, 1, 0] , (brushingdiameter/2), detail)
@C22 = @Plate.entities.add_circle([-pitch/2, platethickness, 0], [0, 1, 0] , (brushingdiameter/2), detail)
@OuterArc12 = @Plate.entities.add_arc [pitch/2, platethickness, 0], [-1,0,0], [0,1,0], @R1, @IntersectAngle.degrees, (360-@IntersectAngle).degrees, detail
@OuterArc22 = @Plate.entities.add_arc [-pitch/2, platethickness, 0], [1,0,0], [0,1,0], @R1, @IntersectAngle.degrees, (360-@IntersectAngle).degrees, detail
@SuperiorArc2 = @Plate.entities.add_arc [0, platethickness, @R2y], [1,0,0], [0,1,0], @R2, @IntersectAngle.degrees, (180-@IntersectAngle).degrees, detail/2
@InferiorArc2 = @Plate.entities.add_arc [0, platethickness, -@R2y], [1,0,0], [0,-1,0], @R2, @IntersectAngle.degrees, (180-@IntersectAngle).degrees, detail/2


#Make bevel 2nd side
Sketchup.active_model.selection.clear
Sketchup.active_model.selection.add(@SuperiorArc2)
Sketchup.active_model.selection.add(@InferiorArc2)
Sketchup.active_model.selection.add(@OuterArc12)
Sketchup.active_model.selection.add(@OuterArc22)

transform = Geom::Transformation.new([0,-@Bevel,0])
@Plate.entities.transform_entities(transform, Sketchup.active_model.selection)
@Plate.entities.transform_entities(transform, @C12)
@Plate.entities.transform_entities(transform, @C22)

Sketchup.active_model.selection.clear
end

@Plate = @Plate.move! [0, -(platethickness/2), 0]

#Changing axis
@Temp = Sketchup.active_model.entities.add_group
@L1 = @Temp.entities.add_line [0,0,0], [1,1,1]
@Temp.entities.add_instance(@Plate.entities.parent, @Plate.transformation*@Temp.transformation)
status = @L1.erase!
status = @Plate.erase!
@Temp.entities[0].explode
@Plate = @Temp
end

 def self.tube(diameter, thickness, width, detail, bevelfactor, smooth)
 
   #get variables
   @Bevel = thickness * bevelfactor
   @Radius = diameter/2
   
   #Make group
   model = Sketchup.active_model
   entities = model.active_entities
   @Tube = Sketchup.active_model.entities.add_group

   @Tc1 = @Tube.entities.add_circle([0,-width/2,0], [0, 1, 0] , @Radius, detail)
   @Tc2 = @Tube.entities.add_circle([0,-width/2,0], [0, 1, 0] , (@Radius-thickness), detail)
   @Tf2 = @Tube.entities.add_face(@Tc2)
   @Tf1 = @Tube.entities.add_face(@Tc1)
   status = @Tf2.erase!
   status = @Tf1.pushpull(width, true)
    
   if (@Smooth == "Yes")
   #internal bevel circles face 1
    @C1 = @Tube.entities.add_circle([0,-width/2,0], [0, 1, 0] , (@Radius-@Bevel), detail)
    @C2 = @Tube.entities.add_circle([0,-width/2,0], [0, 1, 0] , (@Radius-thickness+@Bevel), detail)
   #internal bevel circles face 2
    @C3 = @Tube.entities.add_circle([0,width/2,0], [0, 1, 0] , (@Radius-@Bevel), detail)
    @C4 = @Tube.entities.add_circle([0,width/2,0], [0, 1, 0] , (@Radius-thickness+@Bevel), detail)

    #move external bevels face 1
    transform = Geom::Transformation.new([0,@Bevel,0])
    @Tube.entities.transform_entities(transform, @Tc1)
    @Tube.entities.transform_entities(transform, @Tc2)

    #move external bevels face 2
    @Tc12 = @Tube.entities.add_circle([0,width/2,0], [0, 1, 0] , @Radius, detail)
    @Tc22 = @Tube.entities.add_circle([0,width/2,0], [0, 1, 0] , (@Radius-thickness), detail)
    Sketchup.active_model.selection.clear
    Sketchup.active_model.selection.add(@Tc12)
    Sketchup.active_model.selection.add(@Tc22)
    transform = Geom::Transformation.new([0,-@Bevel,0])
    @Tube.entities.transform_entities(transform, Sketchup.active_model.selection)
    Sketchup.active_model.selection.clear
   end
   
#Changing axis
@Temp = Sketchup.active_model.entities.add_group
@L1 = @Temp.entities.add_line [0,0,0], [1,1,1]
@Temp.entities.add_instance(@Tube.entities.parent, @Tube.transformation*@Temp.transformation)
status = @L1.erase!
status = @Tube.erase!
@Temp.entities[0].explode
@Tube = @Temp
 end

 def self.pin(diameter, length, detail, bevelfactor, smooth)
 @ExtentionFactor = 0.5
 @Extension = @ExtentionFactor * diameter
 @Scale = 100.00

  @Bevel1 = 0.0
  @Bevel2 = bevelfactor * (diameter/2)
  if (@Smooth == "Yes")
  @Bevel1 = bevelfactor * (diameter/2)
  end
 

 #Make group
 model = Sketchup.active_model
 entities = model.active_entities
 @Pin = Sketchup.active_model.entities.add_group


 @ProfileContourn = [
 [0, -((length/2)+@Extension)* @Scale, 0],
 [0, (length/2)* @Scale, 0],
 [((diameter/2)-@Bevel1)* @Scale, (length/2)* @Scale, 0],
 [(diameter/2)* @Scale, ((length/2)-@Bevel2)* @Scale, 0],

 [(diameter/2)* @Scale, -(length/2)* @Scale, 0],
 [(diameter/3)* @Scale, -(length/2)* @Scale, 0],
 [(diameter/3)* @Scale, -((length/2)+(diameter/6))* @Scale, 0],
 [(diameter/2)* @Scale, -((length/2)+(diameter/6))* @Scale, 0],

 [(diameter/2)* @Scale, -((length/2)+@Extension-@Bevel2)* @Scale, 0],
 [((diameter/2)-@Bevel1)* @Scale, -((length/2)+@Extension)* @Scale, 0],
 [0, -((length/2)+@Extension)* @Scale, 0]
 ]

 @Pf = @Pin.entities.add_face(@ProfileContourn)
 
 @Pc = @Pin.entities.add_circle([0,0,0], [0, 1, 0] , (diameter* @Scale), detail)
 @Pfollow = @Pf.followme(@Pc)

   @Pin_Counter = 0
   while @Pin_Counter < @Pc.length
   status = @Pc[@Pin_Counter].erase!
   @Pin_Counter += 1
   end
 
#Changing axis
@Temp = Sketchup.active_model.entities.add_group
@L1 = @Temp.entities.add_line [0,0,0], [1,1,1]
@Temp.entities.add_instance(@Pin.entities.parent, @Pin.transformation*@Temp.transformation)
status = @L1.erase!
status = @Pin.erase!
@Temp.entities[0].explode
@Pin = @Temp

#Make hole
@Hole = Sketchup.active_model.entities.add_group
@C1 = @Hole.entities.add_circle([0,-((length/2)+(diameter/6)) * @Scale, -diameter * @Scale], [0, 0, 1] , (diameter/6) * @Scale, detail)
@F1 = @Hole.entities.add_face(@C1)
status = @F1.pushpull((diameter*2) * @Scale, true)
@Pin = @Hole.subtract(@Pin)

Sketchup.active_model.selection.clear
Sketchup.active_model.selection.add(@Pin)
@Pin.transformation = Geom::Transformation.scaling([0,0,0], (1.0/@Scale).to_f) * @Pin.transformation

#Changing axis
@Temp = Sketchup.active_model.entities.add_group
@L1 = @Temp.entities.add_line [0,0,0], [1,1,1]
@Temp.entities.add_instance(@Pin.entities.parent, @Pin.transformation*@Temp.transformation)
status = @L1.erase!
status = @Pin.erase!
@Temp.entities[0].explode
@Pin = @Temp
 end

 def self.lock(inrad, outrad, thickness, detail, bevelfactor, smooth)
 #Make group
 model = Sketchup.active_model
 entities = model.active_entities
 @Lock = Sketchup.active_model.entities.add_group
 @Scale = 100.00

@R1 = (inrad)
@R2 = (outrad)
@AngA = 100.degrees
@AngB = 360.degrees
@AngC = 10.degrees
@Thick = thickness
@Detail = detail
@Bevel = @Thick * bevelfactor

@A1 = @Lock.entities.add_arc [0, 0, 0], [0,0,-1], [0,1,0], @R2* @Scale, @AngA, @AngB, detail
@A2 = @Lock.entities.add_arc [0, 0, 0], [0,0,-1], [0,1,0], @R1* @Scale, @AngA, @AngB, detail
@L1 = @Lock.entities.add_line [0,0,-@R1* @Scale], [0,0,-@R2* @Scale]
@L2 = @Lock.entities.add_line [(-Math.cos(@AngC)*@R1)* @Scale,0, (Math.sin(@AngC)*@R1)* @Scale], [-(Math.cos(@AngC)*@R2)* @Scale,0, (Math.sin(@AngC)*@R2)* @Scale]

@Path = @L1.all_connected
@F1 = @Lock.entities.add_face(@Path)


if @F1.normal.y == -1
@F1.reverse!
end

status = @F1.pushpull(-@Thick* @Scale, true)

  if (@Smooth == "Yes")
  @A1o = @Lock.entities.add_arc [0, -thickness* @Scale, 0], [0,0,-1], [0,1,0], @R2* @Scale, @AngA, @AngB, detail
  @A1b = @Lock.entities.add_arc [0, -thickness* @Scale, 0], [0,0,-1], [0,1,0], (@R2-@Bevel)* @Scale, @AngA, @AngB, detail
  Sketchup.active_model.selection.clear
  Sketchup.active_model.selection.add(@A1o)
  transform = Geom::Transformation.new([0,@Bevel* @Scale,0])
  @Lock.entities.transform_entities(transform, @A1o)
  Sketchup.active_model.selection.clear
  end

Sketchup.active_model.selection.clear
Sketchup.active_model.selection.add(@Lock)
@Lock.transformation = Geom::Transformation.scaling([0,0,0], (1.0/@Scale).to_f) * @Lock.transformation

#Changing axis
@Temp = Sketchup.active_model.entities.add_group
@L1 = @Temp.entities.add_line [0,0,0], [1,1,1]
@Temp.entities.add_instance(@Lock.entities.parent, @Lock.transformation*@Temp.transformation)
status = @L1.erase!
status = @Lock.erase!
@Temp.entities[0].explode
@Lock = @Temp
 end


(#ASSEMBLE
#Speed boost by disable UI updates ---- <<<
model.start_operation('Chain link - Assemble', true)

model = Sketchup.active_model
entities = model.active_entities


print 'Make Roller Chain - Debug point 4/7 ok'
#Make BnR elements - - - - - - - - - - - - - - - - - - -
@PlateX1 = self.plates(@Pitch, @BrushingDiameter, @RollerDiameter, @PlateThickness, @RollerOutRatio, @R1R2Ratio, @Detail, @bevelfactor, @Smooth)
@PlateX1 = @PlateX1.to_component
@PlateX1 = @PlateX1.move! [0, (-(@RollerWidth+@PlateThickness)/2), 0]
@PlateX2 = @PlateX1.copy
@PlateX2 = @PlateX2.move! [0, (@RollerWidth+@PlateThickness)/2, 0]

@TubeX1 = self.tube(@BrushingDiameter, @BrushingThickness, @BrushingWidth, @Detail, @bevelfactor, @Smooth)
@TubeX1 = @TubeX1.to_component
@TubeX1 = @TubeX1.move! [(-@Pitch/2), 0, 0]
@TubeX2 = @TubeX1.copy
@TubeX2 = @TubeX2.move! [(@Pitch/2), 0, 0]

@TubeX3 = self.tube(@RollerDiameter, @RollerThickness, @RollerWidth, @Detail, @bevelfactor, @Smooth)
@TubeX3 = @TubeX3.to_component
@TubeX3 = @TubeX3.move! [(-@Pitch/2), 0, 0]
@TubeX4 = @TubeX3.copy
@TubeX4 = @TubeX4.move! [(@Pitch/2), 0, 0]


	(#Make group 1 Brushes and Rollers
	@BnR = Sketchup.active_model.entities.add_group
	@L1 = @BnR.entities.add_line [0,0,0], [1,1,1]
	@BnR.entities.add_instance(@PlateX1.definition, @PlateX1.transformation*@BnR.transformation)
	@BnR.entities.add_instance(@PlateX2.definition, @PlateX2.transformation*@BnR.transformation)
	@BnR.entities.add_instance(@TubeX1.definition, @TubeX1.transformation*@BnR.transformation)
	@BnR.entities.add_instance(@TubeX2.definition, @TubeX2.transformation*@BnR.transformation)
	@BnR.entities.add_instance(@TubeX3.definition, @TubeX3.transformation*@BnR.transformation)
	@BnR.entities.add_instance(@TubeX4.definition, @TubeX4.transformation*@BnR.transformation)
	@L1.erase!
	@PlateX1.erase!
	@PlateX2.erase!
	@TubeX1.erase!
	@TubeX2.erase!
	@TubeX3.erase!
	@TubeX4.erase!

	#Put colors
	@BnR.entities[0].material = [100, 40, 255]
	@BnR.entities[1].material = [100, 40, 255]
	@BnR.entities[2].material = [70, 100, 150]
	@BnR.entities[3].material = [70, 100, 150]
	@BnR.entities[4].material = [170, 140, 90]
	@BnR.entities[5].material = [170, 140, 90]

	@BnR = @BnR.to_component
	@BnR.name = "RC link 1"
	@Link_1_def = @BnR.definition
	)

print 'Make Roller Chain - Debug point 5/7 ok'

#Make PnL elements - - - - - - - - - - - - - - - - - - -
@PlateX1 = self.plates(@Pitch, @PinDiameter, @RollerDiameter, @PlateThickness, @RollerOutRatio, @R1R2Ratio, @Detail, @bevelfactor, @Smooth)
@PlateX1 = @PlateX1.to_component
@PlateX1 = @PlateX1.move! [0, -((@RollerWidth/2)+(@PlateThickness*1.5)), 0]
@PlateX2 = @PlateX1.copy
@PlateX2 = @PlateX2.move! [0, ((@RollerWidth/2)+(@PlateThickness*1.5)), 0]

@PinX1 = self.pin(@PinDiameter, @PinLength, @Detail, @bevelfactor, @Smooth)
@PinX1 = @PinX1.to_component
@PinX1 = @PinX1.move! [(-@Pitch/2), 0, 0]
@PinX2 = @PinX1.copy
@PinX2 = @PinX2.move! [(@Pitch/2), 0, 0]

@LockX1 = self.lock((@PinDiameter/3), @PinDiameter*0.8, (@PinDiameter/6), @Detail, @bevelfactor, @Smooth)
@LockX1 = @LockX1.to_component
@LockX1 = @LockX1.move! [(-@Pitch/2), -(@PinLength/2), 0]
@LockX2 = @LockX1.copy
@LockX2 = @LockX2.move! [(@Pitch/2), -(@PinLength/2), 0]

	(#Make group 2 Plates, Pins and Locks
	@PnL = Sketchup.active_model.entities.add_group
	@L1 = @PnL.entities.add_line [0,0,0], [1,1,1]
	@PnL.entities.add_instance(@PlateX1.definition, @PlateX1.transformation*@PnL.transformation)
	@PnL.entities.add_instance(@PlateX2.definition, @PlateX2.transformation*@PnL.transformation)
	@PnL.entities.add_instance(@PinX1.definition, @PinX1.transformation*@PnL.transformation)
	@PnL.entities.add_instance(@PinX2.definition, @PinX2.transformation*@PnL.transformation)
	@PnL.entities.add_instance(@LockX1.definition, @LockX1.transformation*@PnL.transformation)
	@PnL.entities.add_instance(@LockX2.definition, @LockX2.transformation*@PnL.transformation)

	@PnL = @PnL.move! [@Pitch, 0, 0]

	@L1.erase!
	@PlateX1.erase!
	@PlateX2.erase!
	@PinX1.erase!
	@PinX2.erase!
	@LockX1.erase!
	@LockX2.erase!

	#Put colors
	@PnL.entities[0].material = [100, 40, 255]
	@PnL.entities[1].material = [100, 40, 255]
	@PnL.entities[2].material = [250, 180, 230]
	@PnL.entities[3].material = [250, 180, 230]
	@PnL.entities[4].material = [80, 80, 80]
	@PnL.entities[5].material = [80, 80, 80]

	@PnL = @PnL.to_component
	@PnL.name = "RC link 2"
	@Link_2_def = @PnL.definition
	)

Sketchup.active_model.commit_operation
#-------------------------------------- <<<
)

print 'Make Roller Chain - Debug point 6/7 ok'


if (@MODE == "no path")

#Make Unit group 0
@PatternUnit = Sketchup.active_model.entities.add_group
@L1 = @PatternUnit.entities.add_line [0,0,0], [1,1,1]
@PatternUnit.entities.add_instance(@BnR.definition, @BnR.transformation*@PatternUnit.transformation)
@PatternUnit.entities.add_instance(@PnL.definition, @PnL.transformation*@PatternUnit.transformation)
@L1.erase!
@BnR.erase!
@PnL.erase!


#Make contruction references
if (@MakeRef2 == "Yes")
model = Sketchup.active_model
entities = model.active_entities
@Refs = @PatternUnit.entities.add_group
@CenterPoint0 = @Refs.entities.add_cpoint [-@Pitch/2, 0, 0]
@CenterPoint1 = @Refs.entities.add_cpoint [0, 0, 0]
@CenterPoint2 = @Refs.entities.add_cpoint [@Pitch/2, 0, 0]
@CenterPoint3 = @Refs.entities.add_cpoint [@Pitch, 0, 0]
@CenterPoint4 = @Refs.entities.add_cpoint [@Pitch*1.5, 0, 0]
@LineSize = (@RollerWidth + @PlateThickness*4)
@RefLine0 = @Refs.entities.add_line [-@Pitch/2,@LineSize,0], [-@Pitch/2,-@LineSize,0]
@RefLine1 = @Refs.entities.add_line [@Pitch/2,@LineSize,0], [@Pitch/2,-@LineSize,0]
@RefLine2 = @Refs.entities.add_line [@Pitch*1.5,@LineSize,0], [@Pitch*1.5,-@LineSize,0]
end

@PatternUnit = @PatternUnit.to_component


(# links in row
if @Links > 0
model.start_operation('RCA - links row', true)

@Whole = Sketchup.active_model.entities.add_group
@Whole.entities.add_instance(@PatternUnit.definition, @PatternUnit.transformation*@Whole.transformation)
 for step in 1..@Links
 @Extend = @PatternUnit.copy
 @Extend = @Extend.move! [(@Pitch*2)*step, 0, 0]
 @Whole.entities.add_instance(@Extend.definition, @Extend.transformation*@Whole.transformation)
 status = @Extend.erase!
 end
 status = @PatternUnit.erase!
 
model.commit_operation
end
)

else #Put chain links along path
print 'Make Roller Chain - Follow path mode'



#Angled offset math -----------------------------------------------
def self.find_new_point(current_point, reference_point, next_point, interval)

#Vectors
@Back_Vec = reference_point.vector_to(current_point)
@Front_Vec = reference_point.vector_to(next_point)

#Math
@Angle_A = @Back_Vec.angle_between @Front_Vec
@Angle_B = Math.asin((Math.sin(@Angle_A)*@Back_Vec.length)/interval)
@Angle_C = (180.0 - (@Angle_A.radians + @Angle_B.radians)).degrees
@Ref_Offset = (Math.sin(@Angle_C)*interval)/Math.sin(@Angle_A)

#Final result
@New_Point_Vector = @Front_Vec
@New_Point_Vector.length = @Ref_Offset
@New_Point = reference_point.offset(@New_Point_Vector)

return @New_Point
end
#------------------------------------------------------------------

def self.collinear_check(point1, point2, point3)
	@Array = [point1, point2, point3].uniq
	if (@Array.length == 2) then return true end
	@Vector1 = point1.vector_to(point2)
	@Vector2 = point1.vector_to(point3)
	@Vector1.length = 1
	@Vector2.length = 1
	if (@Vector1 == @Vector2) || (@Vector1 == @Vector2.reverse) then return true end
	return false
end


#Normalizes path segments
print 'Make Roller Chain - Starting segments normalizing'
@Link_Interval = @Pitch
@Normalized_Segments = []

@Segments.each do |segment|
	@Seg_Points = segment

	@Current_Point = @Seg_Points[0]
	@Next_Point = @Seg_Points[1]
	
	@Normalized_Segment = []
	@Normalized_Segment << Geom::Point3d.new(@Current_Point)

	@STATUS = "STARTING"
	until (@STATUS == "FINISHED")
		@Distance = @Current_Point.distance(@Next_Point)

		until (@Link_Interval <= @Distance) || ( @Seg_Points.index(@Next_Point) == (@Seg_Points.length-1) ) || ( @Seg_Points.index(@Next_Point) == 0 )
		@Next_Point = @Seg_Points[@Seg_Points.index(@Next_Point)+1]
		@Distance = @Current_Point.distance(@Next_Point)
		end

		if (@Link_Interval > @Distance) then
			#End projection
			if self.collinear_check(@Current_Point, @Seg_Points[@Seg_Points.index(@Next_Point)-1], @Next_Point)
				#Linear
				@New_Point_Offset = @Current_Point.vector_to(@Next_Point)
				@New_Point_Offset.length = @Link_Interval
				@New_Point = @Current_Point.offset(@New_Point_Offset)
				@Normalized_Segment << Geom::Point3d.new(@New_Point)
				@Current_Point = @New_Point
				@STATUS = "FINISHED"
			else
				#Angular
				@New_Point = self.find_new_point(@Current_Point, @Seg_Points[@Seg_Points.index(@Next_Point)-1], @Next_Point, @Link_Interval)
				@Normalized_Segment << Geom::Point3d.new(@New_Point)
				@Current_Point = @New_Point
				@STATUS = "FINISHED"
			end
		end

		if (@Link_Interval < @Distance)
			if self.collinear_check(@Current_Point, @Seg_Points[@Seg_Points.index(@Next_Point)-1], @Next_Point)
				#Linear
				@New_Point_Offset = @Current_Point.vector_to(@Next_Point)
				@New_Point_Offset.length = @Link_Interval
				@New_Point = @Current_Point.offset(@New_Point_Offset)
				@Normalized_Segment << Geom::Point3d.new(@New_Point)
				@Current_Point = @New_Point
			else
				#Angular
				@New_Point = self.find_new_point(@Current_Point, @Seg_Points[@Seg_Points.index(@Next_Point)-1], @Next_Point, @Link_Interval)
				@Normalized_Segment << Geom::Point3d.new(@New_Point)
				@Current_Point = @New_Point
			end
		end

		if (@Link_Interval == @Distance)
			@New_Point = @Next_Point
			@Normalized_Segment << Geom::Point3d.new(@New_Point)
			@Current_Point = @New_Point
			
			if (@Seg_Points.index(@Next_Point) == (@Seg_Points.length-1)) then @STATUS = "FINISHED"; end
		end
	end


@Normalized_Segments << @Normalized_Segment
end
print 'Make Roller Chain - Segments normalized'
#--------------------------------


#Chain links ammount - CONTINUE CHECK
@Chain_Links = 0
@Normalized_Segments.each do |segment|
@Chain_Links += (segment.length-1)
end

if UI.messagebox(@Chain_Links.to_s + " chain links to be created.", type = MB_OKCANCEL) != IDOK
 UI.messagebox("Operation canceled.", type = MB_OK)
 raise Exception.new("Make Roller Chain: Operation canceled.")
end
print ("Make Roller Chain - " + @Chain_Links.to_s + " chain links to be created.")
#-----------------------------------------------------------------------------


#Speed boost by disable UI updates ---- <<<
model.start_operation('Chain link - segments instances', true)

#Puts chain link instances along normalized segments
@All_Chains = model.entities.add_group
@Seg_ID = 0
@Normalized_Segments.each do |segment|

	if (@Normalized_Segments.length > 1)
		@Group = @All_Chains.entities.add_group
	else
		@Group = model.entities.add_group
	end

	#Define Side Vector <<<
	case @Flat_Axis[@Seg_ID]
		when 0
		@Side_Vector = Geom::Vector3d.new(1,0,0)
		when 1
		@Side_Vector = Geom::Vector3d.new(0,1,0)
		when 2
		@Side_Vector = Geom::Vector3d.new(0,0,1)
	end

	@Last_UP_Vector = Geom::Vector3d.new(0,0,0)
	for i in 0..(segment.length-2)

		@Direction = segment[i].vector_to(segment[i+1]) #Define Direction Vector <<<
		@Direction.normalize!

		@Offset = segment[i].vector_to(segment[i+1])
		@Offset.length = @Offset.length/2.0
		@Position = segment[i].offset(@Offset)

		@UP_Vector = @Direction.cross(@Side_Vector) #Define Up Vector <<<
		@UP_Vector.normalize!

		@UPs_Angle = @UP_Vector.angle_between @Last_UP_Vector
		if (i != 0)
			if (@UPs_Angle.radians > 90.0) then @UP_Vector.reverse!; end
		end

		if (i%2 == 0)
			@Elo_def = @Link_1_def
		else
			@Elo_def = @Link_2_def
		end

		@Elo_Copy = @Group.entities.add_instance @Elo_def, Geom::Transformation.scaling(@Position, 1) #Position & Scale <<<
		@transform2 = Geom::Transformation.axes( @Position, @Direction, @Side_Vector, @UP_Vector )
		@Elo_Copy.transformation *= @transform2 #Direction <<<

		@Last_UP_Vector = @UP_Vector
	end
	
	@Seg_ID += 1
end

#Erases base chain link model
@BnR.erase!
@PnL.erase!

Sketchup.active_model.commit_operation
#-------------------------------------- <<<
end

print 'Make Roller Chain - Debug point 7/7 ok'

#--------------------------------------------------------------------------

 #Timer end
@end_time = Time.now
@Total_Time = @end_time - @start_time

#End Message
print "Success. (" + @Total_Time.to_s + " seconds) \n \n \n"
Taxsola.soundeffect(1) #0: Installation | 1: Success | 2: Notification
end

	end
end
#-------------------------------------------------------------------------------