=begin
#-------------------------------
Author: Tax, 2021
Description:
#-------------------------------
=end


#-------------------------------------------------------------------------------
module Taxsola
	module Tax_Engineering

def self.spheric_spiral_dialog
#-------------------------------------------------------------------------------
prompts = ["Sphere radius: ", "Coils: ", "Segments per coil: ", "Cutoff (%): ", "Stretch factor: ", "Draw wire?: ", "Wire diameter: ", "Wire profile segments: ", "Draw reference?: "]
defaults = [50.00.mm, 10, 24, 5, 1.00, "Yes", 5.00.mm, 12, "Yes"]
list = ["", "", "", "", "", "Yes|No", "", "", "Yes|No"]
val = UI.inputbox(prompts, defaults, list, "Spheric Helix")
if val
@Sphere_radius, @Revolutions, @Edges_per_revolution, @Cutoff, @Stretch_factor, @Make_tube, @Tube_diameter, @Tube_detail, @MakeRef = val;
else
 raise Exception.new("Spheric Spring error: SSxV.000 - Operation canceled.")
end
#-------------------------------------------------------------------------------

#Timer start
@start_time = Time.now

#Log and verification
print 'Spheric Spring operation:'
print 'Input log: ' + val[0].to_s + ' | ' + val[1].to_s + ' | ' + val[2].to_s + ' | ' + val[3].to_s + ' | ' + val[4].to_s + ' | ' + val[5].to_s + ' | ' + val[6].to_s + ' | ' + val[7].to_s + ' | ' + val[8].to_s


(#Verifications

# Sphere radius > 0
if (@Sphere_radius <= 0)
UI.messagebox("error: SSxV.001 - Sphere radius must be higher than 0.")
raise Exception.new("Spheric Spring error: SSxV.001")
end

# Revolutions > 0
if (@Revolutions <= 0)
UI.messagebox("error: SSxV.002 - Revolutions must be higher than 0.")
raise Exception.new("Spheric Spring error: SSxV.002")
end

# Edges per rev > 0
if (@Edges_per_revolution <= 0)
UI.messagebox("error: SSxV.003 - Edges per revolution must be higher than 0.")
raise Exception.new("Spheric Spring error: SSxV.003")
end

# Cutoff 0 > c < 100
if (@Cutoff <= 0) or (@Cutoff >= 100)
UI.messagebox("error: SSxV.004 - Cutoff must be in range (0 > C < 100).")
raise Exception.new("Spheric Spring error: SSxV.004")
end

# Strech > 0
if (@Stretch_factor <= 0)
UI.messagebox("error: SSxV.005 - Stretch factor must be higher than 0.")
raise Exception.new("Spheric Spring error: SSxV.005")
end

# Tube radius range
if (@Tube_diameter <= 0)
UI.messagebox("error: SSxV.006 - Tube diameter must be higher than 0.")
raise Exception.new("Spheric Spring error: SSxV.006")
end

# Tube radius range
if ( (@Revolutions * @Tube_diameter /4) >= (@Sphere_radius * @Stretch_factor) )
UI.messagebox("error: SSxV.007 - Tube diameter is too large.")
raise Exception.new("Spheric Spring error: SSxV.007")
end

# Tube detail > 0
if (@Tube_detail <= 0)
UI.messagebox("error: SSxV.008 - Tube detail must be higher than 0.")
raise Exception.new("Spheric Spring error: SSxV.008")
end
)

print 'Spheric Spring - Debug point 1/5 ok'

(#-------------------------------------------------------------------------------

#Define camera view
camera = Sketchup::Camera.new([10, -10, 10], [0, 0, 0], Z_AXIS)
view = Sketchup.active_model.active_view
view.camera = camera

#Speed boost by disable UI updates
model = Sketchup.active_model
entities = model.active_entities
model.start_operation('Make Parts', true)

#-------------------------------------------------------------------------------
)


#Spheric Spring
def self.spiral_spring(radius, revolutions, edgesperrev, cutoff, stretchfactor, maketube, tubediameter, detail, makereferences)
#Defining group
model = Sketchup.active_model
entities = model.active_entities
@Spring = Sketchup.active_model.entities.add_group
@Tube = @Spring.entities.add_group
@Reference = @Spring.entities.add_group

#Defining variables
@Scale = 100.00
radius = (radius * @Scale).to_f
revolutions = revolutions.to_i
edgesperrev = edgesperrev.to_i
cutoff = cutoff.to_i
tubediameter = (tubediameter * @Scale).to_f
detail = detail.to_i
@SpringColor = [40, 40, 40]


@TotalEdges = revolutions * edgesperrev
@VerticalStep = 180.00.degrees / @TotalEdges
@HorizontalStep = 360.00.degrees / edgesperrev

#CutOff (0% - 100%)
@cut = (@TotalEdges / 100.00 * cutoff)
@start = (@cut).to_i
@ending = (@TotalEdges - @cut).to_i

print 'Spheric Spring - Debug point 2/5 ok'

#Defining 1st point
@LocalRadius = Math.sin(@VerticalStep * @start) * radius
@x = Math.cos(@HorizontalStep * @start) * @LocalRadius
@y = Math.sin(@HorizontalStep * @start) * @LocalRadius
@z = Math.cos(@VerticalStep * @start) * radius * stretchfactor
@point1 = [@x, @y, @z]

@PointRef = @point1

#Making spiral
 for edge in @start+1..@ending
 @LocalRadius = Math.sin(@VerticalStep * edge) * radius

 @x = Math.cos(@HorizontalStep * edge) * @LocalRadius
 @y = Math.sin(@HorizontalStep * edge) * @LocalRadius
 @z = Math.cos(@VerticalStep * edge) * radius * stretchfactor

 @point2 = [@x, @y, @z]
 @line = @Reference.entities.add_line @point1, @point2
 @point1 = @point2
 end

print 'Spheric Spring - Debug point 3/5 ok'

#Get edge and path reference
@RefEdge = @Reference.entities[0]
@Vector = @RefEdge.start.position.vector_to(@RefEdge.end.position)
@Path = @RefEdge.all_connected


	if (maketube.to_s == "Yes")
		#Make tube face
		@C1 = @Tube.entities.add_circle(@PointRef, @Vector , tubediameter/2, detail)
		@F1 = @Tube.entities.add_face(@C1)

		#Set face direction
		@D = 3
		@VectorModule = ((@Vector.x**2) + (@Vector.y**2) + (@Vector.z**2))**0.5
		@UnitVector = [ (@Vector.x/@VectorModule).round(@D), (@Vector.y/@VectorModule).round(@D), (@Vector.z/@VectorModule).round(@D) ]

		@FaceVectorModule = ((@F1.normal.x**2) + (@F1.normal.y**2) + (@F1.normal.z**2))**0.5
		@FaceUnitVector = [ (@F1.normal.x/@FaceVectorModule).round(@D), (@F1.normal.y/@FaceVectorModule).round(@D), (@F1.normal.z/@FaceVectorModule).round(@D) ]

		if @UnitVector == @FaceUnitVector
		@F1.reverse!
		end

		print 'Spheric Spring - Debug point 4/5 ok'

		#Apply face color
		@F1.material = @SpringColor
		@F1.back_material = @SpringColor

		#Make tube
		@Structure = @F1.followme(@Path)
	end


	if makereferences == "No"
		@Reference.erase!
	else
		@CenterPoint = @Reference.entities.add_cpoint [0, 0, 0]
	end

scale_transform = Geom::Transformation.scaling([0, 0, 0], 1/@Scale)
@Spring.transformation *= scale_transform
end


self.spiral_spring(@Sphere_radius, @Revolutions, @Edges_per_revolution, @Cutoff, @Stretch_factor, @Make_tube.to_s, @Tube_diameter, @Tube_detail, @MakeRef)

print 'Spheric Spring - Debug point 5/5 ok'
#-------------------------------------------------------------------------------


 #Timer end
@end_time = Time.now
@Total_Time = @end_time - @start_time
print "Success. (" + @Total_Time.to_s + " seconds) \n \n"
Taxsola.soundeffect(1) #0: Installation | 1: Success | 2: Notification
#-------------------------------------------------------------------------------
end



	end
end
#-------------------------------------------------------------------------------