=begin
#-------------------------------
Author: Tax, 2021
Description:
#-------------------------------
=end


#-------------------------------------------------------------------------------
module Taxsola
	module Tax_Engineering

def self.spur_dialog
#SOME RESOURCE HERE: https://www.engineersedge.com/calculators/spur_gear_calculator_and_generator_15506.htm


#DEFAULT VALUES
@Choosen_Preset = "Custom"
@GearColor = [100,100,100]
@nTeeth = 24
@Module = 2.5465
@PressureAngle = 20.00.degrees
@Clearance = 1.0
@Width = 4.80
@HoleDiameter = 2.50
@Detail = 12
@MakeRef = "Yes"

#Presets setup obs: presets pressure angle must be in degrees
@Presets = {
'Custom': [],
'KMSGA1-18': [18,1,20],
'KMSGA1-20': [20,1,20],
'KMSGA1-24': [24,1,20],
'KMSGA1-25': [25,1,20],
'KMSGA1-30': [30,1,20],
'KMSGA1-35': [35,1,20],
'KMSGA1-36': [36,1,20],
'KMSGA1-40': [40,1,20],
'KMSGA1-45': [45,1,20],
'KMSGA1-48': [48,1,20],
'KMSGA1-50': [50,1,20],
'KMSGA1-55': [55,1,20],
'KMSGA1-60': [60,1,20],
'KMSGA1-70': [70,1,20],
'KMSGA1-80': [80,1,20],
'KMSGA1-100': [100,1,20]
}
@Presets_String = ''
@Presets.each {|key,value| @Presets_String += key.to_s; @Presets_String += '|'}
@Presets_String.chomp!'|'


@Verification = "NOT OK"
until @Verification == "OK"
#-------------------------------------------------------------------------------
@Verification = "OK"

#INPUT BOX
@PressureAngle = @PressureAngle.radians.round(3).to_s + "degrees"

prompts		=	["Preset: ", "Teeths: ", "Module (Pitch/PI): ", "Pressure angle: ", "Clearance: ", "Width: ", "Hole diameter: ", "Detail (sides): ", "Construct ref.?: "]
defaults	=	[@Choosen_Preset, @nTeeth, @Module, @PressureAngle, @Clearance, @Width, @HoleDiameter, @Detail, @MakeRef]
list		=	[@Presets_String, "", "", "", "", "", "", "", "Yes|No"]
@Input 		=	UI.inputbox(prompts, defaults, list, "Make Spur gear")

if @Input
	@Choosen_Preset, @nTeeth, @Module, @PressureAngle, @Clearance, @Width, @HoleDiameter, @Detail, @MakeRef = @Input;
else
	raise Exception.new("Make Spur Gear error: SxV.000 - Operation canceled.")
end

print 'Make Spur Gear operation:'

#LOAD PRESET
if (@Choosen_Preset != "Custom") #Get selected preset values into the variables
	puts ("Loading preset")
	@Preset_Values = @Presets[@Choosen_Preset.to_sym]
	@nTeeth, @Module, @PressureAngle = @Preset_Values
end

@PressureAngle = Taxsola.to_angle(@PressureAngle)	#Personal method for improving UI experience with angles units
@Pitch = Math::PI * @Module


(#INPUT LOG
@Input_Log = "Input log:\n"
@Input_Log += @Choosen_Preset.to_s + ' | '
@Input_Log += @nTeeth.to_s + ' | '
@Input_Log += @Module.to_s + ' | '
@Input_Log += @PressureAngle.to_s + ' | '
@Input_Log += @Clearance.to_s + ' | '
@Input_Log += @Width.to_s + ' | '
@Input_Log += @HoleDiameter.to_s + ' | '
@Input_Log += @Detail.to_s + ' | '
@Input_Log += @MakeRef.to_s

print @Input_Log
)


(#Precaution protocols
@nTeeth = @nTeeth.to_i
@Detail = @Detail.to_i

@Module = @Module.to_f
@PressureAngle = @PressureAngle.to_f
@Clearance = @Clearance.to_f
@Width = @Width.to_f
@HoleDiameter = @HoleDiameter.to_f
#------------------------------------
if (@nTeeth <= 2) then @nTeeth = 3; end
if (@Detail <= 2) then @Detail = 3; end
if (@Module <= 0) then @Module = 0.1; end
if (@PressureAngle <= 0) then @PressureAngle = 1; end
if (@PressureAngle > 32.0) then @PressureAngle = 32.0; end
if (@Clearance <= 0) then @Clearance = 0.01; end
if (@Width <= 0) then @Width = 1; end
if (@HoleDiameter <= 0) then @HoleDiameter = 1; end

@ProfileShift = 0 #SHOULD IT BE A PARAM??? maybe in future update
#if (@ProfileShift < 0) then @ProfileShift = 0; end
)


(#VERIFICATIONS
@msg = ""

if (@Clearance >= @Pitch)
@msg = "error: SGxV.002 - Clearance is too large."
UI.messagebox(@msg); puts(@msg);
@Verification = "NOT OK"
end


#Pitch
@PitchDiameter = (@Pitch * @nTeeth) / Math::PI
@PitchRadius = @PitchDiameter/2

#BASE
@BaseDiameter = @PitchDiameter * Math.cos( @PressureAngle )
@BaseRadius = @BaseDiameter / 2

if (@HoleDiameter/2 >= @BaseRadius-@Clearance)
@msg = "error: SGxV.003 - HoleDiameter is too large."
UI.messagebox(@msg); puts(@msg);
@Verification = "NOT OK"
end
)

print("Verifications: " + @Verification.to_s + "\n")
#-------------------------------------------------------------------------------
end



#Conversion
@Scale = 100.000
@Pitch = (@Pitch * @Scale).mm
# @ProfileShift = (@ProfileShift* @Scale).mm
@Clearance = (@Clearance* @Scale).mm
@Width = (@Width* @Scale).mm
@HoleDiameter = (@HoleDiameter* @Scale).mm
@PitchAngle = (360.000 / @nTeeth).degrees
@MakeRef = @MakeRef.to_s
#-------------------------------------------------------------------------------



#-------------------------------------------------------------------------------
#Define camera view
camera = Sketchup::Camera.new([-2, -2, 2], [0, 0, 0], Z_AXIS)
view = Sketchup.active_model.active_view
view.camera = camera
sleep(1)

#Speed boost by disable UI updates
model = Sketchup.active_model
entities = model.active_entities
model.start_operation('Spur gear', true)
#-------------------------------------------------------------------------------



#Calculating gear basic parameters

#Pitch
@PitchDiameter = (@Pitch * @nTeeth) / Math::PI
@PitchRadius = @PitchDiameter/2

#BASE
@BaseDiameter = @PitchDiameter * Math.cos( @PressureAngle )
@BaseRadius = @BaseDiameter / 2

@Module = @PitchDiameter / @nTeeth

#TIP
@TipDiameter = @PitchDiameter + 2 * @Module * (1 + @ProfileShift)
@TipRadius = @TipDiameter / 2

@TipPressureAngle = Math.acos(@BaseDiameter / @TipDiameter).degrees
@InvAlpha = Math.tan(@PressureAngle) - @PressureAngle
@InvAlphaA = Math.tan(@TipPressureAngle.radians) - @TipPressureAngle.radians

@TopThickness = Math::PI / (2 * @nTeeth) + (2 * @ProfileShift * Math.tan(@PressureAngle)) / @nTeeth + @InvAlpha - @InvAlphaA
@TopThicknessDegrees = @TopThickness.degrees


@CrestWidth = @TipDiameter * @TopThickness
@tooththickness = 2 * ((90 / @nTeeth) + ( 360 * @ProfileShift * Math.tan(@PressureAngle) ) / Math::PI / @nTeeth)

#Angular tooth width along the base diameter.
@umax = Math.sqrt( @TipRadius * @TipRadius / @BaseRadius / @BaseRadius - 1) * 1.000 # Math.sqrt( ((R**2) / (r**2)) - 1 )
	@x1 = @BaseRadius
	@y1 = 0
	@x2 = @BaseRadius * ( Math.cos( @umax ) + @umax * Math.sin( @umax ) )
	@y2 = @BaseRadius * ( Math.sin( @umax ) - @umax * Math.cos( @umax ) )
#Distance between beginning and end of the involute curve
@d = Math.sqrt( ( @x1 - @x2 ) * ( @x1 - @x2 ) + ( @y1 - @y2 ) * ( @y1 - @y2 ) )
@cosx = (@BaseRadius * @BaseRadius + @TipRadius * @TipRadius - @d * @d) / 2 / @BaseRadius / @TipRadius
@basetooththickness = (2 * @TopThickness + 2 * Math.acos(@cosx))

print 'Make Spur Gear - Debug point 1/5 ok'

#-------------------------------------------------------------------------------

#Involute making
model = Sketchup.active_model
entities = model.active_entities
@Involute = Sketchup.active_model.entities.add_group
@Points = []


#Some math to find basic parameters of involute

#Tip arc
@AngleOffset = Math.asin(@y2/@TipRadius) #replace: ((@basetooththickness-@TopThickness)/2)
@TipArc = @Involute.entities.add_arc [0,0,0], [1, 0, 0] , [0,0,1], @TipRadius, -(@TopThickness/2), @TopThickness/2, @Detail


#involute A
for step in 0.. (@Detail)
 @angle = ((@umax / @Detail) * step) * 1.000
 @Points[step] = [@BaseRadius * (Math.cos(@angle) + @angle * Math.sin(@angle)), @BaseRadius * (Math.sin(@angle) - @angle * Math.cos(@angle)), 0]
end
@TempInv = @Involute.entities.add_group
@Inva = @TempInv.entities.add_line @Points
transformation = Geom::Transformation.rotation([0,0,0], [0,0,1], -(@AngleOffset + @TopThickness/2) )
@TempInv.transform!(transformation)
@TempInv.explode

#involute B
for a in 0.. (@Detail)
 @angle = ((@umax / @Detail) * a) * 1.000
 @Points[a] = [@BaseRadius * ( Math.cos(@angle) + ((@angle) * Math.sin(@angle)) ), -@BaseRadius * ( Math.sin(@angle) - ((@angle) * Math.cos(@angle)) ), 0]
end
@TempInv = @Involute.entities.add_group
@Invb = @TempInv.entities.add_line @Points
transformation = Geom::Transformation.rotation([0,0,0], [0,0,1], @AngleOffset + (@TopThickness/2) )
@TempInv.transform!(transformation)
@TempInv.explode

print 'Make Spur Gear - Debug point 2/5 ok'

#-------------------------------------------------------------------------------
model = Sketchup.active_model
entities = model.active_entities


#Some math to find basic parameters of clearance arc
@ClearanceAngle = @PitchAngle - (2*@AngleOffset + @TopThickness)

@Pointa = [Math.cos( @AngleOffset + (@TopThickness/2) )*@BaseRadius, Math.sin( @AngleOffset + (@TopThickness/2) )*@BaseRadius, 0]
@Pointb = [Math.cos( @AngleOffset + (@TopThickness/2) + @ClearanceAngle )*@BaseRadius, Math.sin( @AngleOffset + (@TopThickness/2) + @ClearanceAngle )*@BaseRadius, 0]

@Corda = BigMath.sqrt( BigDecimal(((@Pointb[0] - @Pointa[0])**2) + ((@Pointb[1] - @Pointa[1])**2), 16), 20 )
@Semi_Corda = @Corda/2

@CordaCenter = [(@Pointb[0] - @Pointa[0])/2 + @Pointa[0], (@Pointb[1] - @Pointa[1])/2 + @Pointa[1], 0]
@ClearancePoint = [ Math.cos(@PitchAngle/2)*(@BaseRadius-@Clearance), Math.sin(@PitchAngle/2)*(@BaseRadius-@Clearance), 0]

@Flecha = BigMath.sqrt(BigDecimal(( (@CordaCenter[0] - @ClearancePoint[0])**2 + (@CordaCenter[1] - @ClearancePoint[1])**2 ), 16), 20 )

@Raio = (@Flecha**2 + @Semi_Corda**2) / (2*@Flecha)

@CenterPoint1 = [ Math.cos(@PitchAngle/2)*(@BaseRadius-@Clearance+@Raio), Math.sin(@PitchAngle/2)*(@BaseRadius-@Clearance+@Raio), 0]

#Calculating angle between 2 vectors (Clearance detail)
@Vector1 = [(@CenterPoint1[0] - @ClearancePoint[0]), (@CenterPoint1[1] - @ClearancePoint[1]), 0]
@Vector2 = [(@CenterPoint1[0] - @Pointb[0]), (@CenterPoint1[1] - @Pointb[1]), 0]
@DotProduct = (@Vector1[0] * @Vector2[0]) + (@Vector1[1] * @Vector2[1])
@AngleT = Math.acos( @DotProduct / (@Raio**2) ).radians
@TotalT = @AngleT*2

(#Making clearance curve 1st side

#_________________________________________________________________________________
@TempInv1 = @Involute.entities.add_group
@Arc1 = @TempInv1.entities.add_arc( @CenterPoint1, [(@Pointb[0] - @Pointa[0]), (@Pointb[1] - @Pointa[1]), 0], [0,0,1], @Raio, ((180-@TotalT)/2 +@AngleT).degrees, ((180-@TotalT)/2 +@TotalT).degrees, @Detail )
@TempInv1.explode
#_________________________________________________________________________________
)


@Pointb = [Math.cos( @AngleOffset + (@TopThickness/2) )*@BaseRadius, -Math.sin( @AngleOffset + (@TopThickness/2) )*@BaseRadius, 0]
@Pointa = [Math.cos( @AngleOffset + (@TopThickness/2) + @ClearanceAngle )*@BaseRadius, -Math.sin( @AngleOffset + (@TopThickness/2) + @ClearanceAngle )*@BaseRadius, 0]
@CenterPoint2 = [ Math.cos(@PitchAngle/2)*(@BaseRadius-@Clearance+@Raio), -Math.sin(@PitchAngle/2)*(@BaseRadius-@Clearance+@Raio), 0]

(#Making clearance curve 2nd side

#_________________________________________________________________________________
@TempInv2 = @Involute.entities.add_group
@Arc2 = @TempInv2.entities.add_arc( @CenterPoint2, [(@Pointb[0] - @Pointa[0]), (@Pointb[1] - @Pointa[1]), 0], [0,0,1], @Raio, ((180-@TotalT)/2).degrees, ((180-@TotalT)/2 +@AngleT).degrees, @Detail )
@TempInv2.explode
#_________________________________________________________________________________
)


print 'Make Spur Gear - Debug point 3/5 ok'


#Making base arc to close involute
@baseArc = @Involute.entities.add_arc [0,0,0], [1, 0, 0] , [0,0,1], @BaseRadius-@Clearance, -@PitchAngle/2, @PitchAngle/2, @Detail

#Making involute face
@InvSilhouete = @baseArc[1].all_connected
@InvFace = @Involute.entities.add_face(@InvSilhouete)
if @InvFace.normal.z == -1
@InvFace.reverse!
end
status = @InvFace.material = @GearColor
status = @InvFace.back_material = @GearColor

#Extrusion of involute face
status = @InvFace.pushpull(@Width, true)
@Involute = @Involute.to_component

#Making external gear group
@Gear = Sketchup.active_model.entities.add_group
@All_inv = @Gear.entities.add_group
@All_inv.entities.add_instance(@Involute.definition, @Involute.transformation*@All_inv.transformation)

(#Copy and rotation of teeths
 for step in 1..(@nTeeth-1)
 @Extend = @Involute.copy
 
transformation = Geom::Transformation.rotation([0,0,0], [0,0,1], @PitchAngle*step )
@Extend.transform!(transformation)

 @All_inv.entities.add_instance(@Extend.definition, @Extend.transformation*@All_inv.transformation)
 status = @Extend.erase!
 end
 status = @Involute.erase!
)

print 'Make Spur Gear - Debug point 4/5 ok'
#-------------------------------------------------------------------------------

(#Body making
Sketchup.active_model.selection.clear
model = Sketchup.active_model
entities = model.active_entities
@BaseGroup = @Gear.entities.add_group

@Bodyc1 = @BaseGroup.entities.add_circle([0,0,0], [0, 0, 1] , @BaseRadius-@Clearance, @Detail*@nTeeth)
@Bodyc2 = @BaseGroup.entities.add_circle([0,0,0], [0, 0, 1] , @HoleDiameter/2, @Detail)
@Bodyf1 = @BaseGroup.entities.add_face(@Bodyc1)
@Bodyf2 = @BaseGroup.entities.add_face(@Bodyc2)
status = @Bodyf2.erase!

if @Bodyf1.normal.z == -1
@Bodyf1.reverse!
end
status = @Bodyf1.material = @GearColor
status = @Bodyf1.back_material = @GearColor
status = @Bodyf1.pushpull(@Width, true)
)


#Make contruction references
if (@MakeRef == "Yes")
model = Sketchup.active_model
entities = model.active_entities
@Refs = @Gear.entities.add_group
@CenterPoint = @Refs.entities.add_cpoint [0, 0, @Width/2]
@RefLine = @Refs.entities.add_line [0,0,-@PitchRadius+@Width/2], [0,0,@PitchRadius+@Width/2]
@RefCircle = @Refs.entities.add_circle([0,0,@Width/2], [0, 0, 1] , @PitchRadius, @nTeeth)
end

#Reset gear for final scale
scale_transform = Geom::Transformation.scaling([0,0,0], 1/@Scale)
@Gear.transformation *= scale_transform

model.commit_operation
#-------------------------------------------------------------------------------
print 'Make Spur Gear - Debug point 5/5 ok'


#End Message
print "Success. \n \n"
Taxsola.soundeffect(1) #0: Installation | 1: Success | 2: Notification
#-------------------------------------------------------------------------------
end


	end
end
#-------------------------------------------------------------------------------