=begin
#-------------------------------
Author: Tax, 2021
Description: Advanced pallete feature
#-------------------------------
=end


#-------------------------------------------------------------------------------
module Taxsola
	module Tax_Engineering

def self.pallete_dialog

	pallete_dialog = UI::HtmlDialog.new({
	  :dialog_title => "TAX - Pallete (HSV)",
	  :preferences_key => "com.sample.plugin",
	  :resizable => false,
	  :style => UI::HtmlDialog::STYLE_DIALOG
	})
	html = File.read(@This_File_Path.to_s + "/Main/Pallete/Pallete_UI.html")
	html.gsub!("DEPENDENCIES", @@HtmlDialog_Dependencies.to_s) #SET DEPENDENCIES
	html.gsub!("MAIN_FILE_PATH", @This_File_Path.to_s) #SET JS UX

	pallete_dialog.set_html(html)
	pallete_dialog.set_size(650, 430)
	pallete_dialog.show


#Print from UX js
pallete_dialog.add_action_callback("PrintText"){|action_context, text| puts(text.to_s); }


#MAKE PALLETE
pallete_dialog.add_action_callback("getUserInput"){|action_context, pallete_name, type, main, c1, c2, c3, c4|

@Colors = [c1, c2, c3, c4]
@Counter = 0


model = Sketchup.active_model
materials = model.materials
matl = materials.current
materials.current = nil if !matl.nil?

if (UI.messagebox("Do you wanna purge unused materials first?", MB_YESNO) == IDYES)
@Before = Sketchup.active_model.materials.length

model.start_operation('Purge materials', true)
	materials.purge_unused
model.commit_operation

@After = Sketchup.active_model.materials.length
UI.messagebox((@Before - @After).to_s + " materials have been purged.", MB_OK)
end

materials.current = matl if !matl.nil? && matl.valid?


#-------------------------------------------
model.start_operation('Color pallete', true)

#Add main material
@material = materials.add(pallete_name + " - " + type + " MAIN ")
@material.color = main

@Colors.each do |color|
	if (color != 'n/a')
		@Counter =+ 1
		@material = materials.add(pallete_name + " - " + type + " " + @Counter.to_s)
		@material.color = color
	end
end

model.commit_operation
#-------------------------------------------

print("Sucess.\n\nPallete name: " + pallete_name + "\nColor rule: " + type + "\n\n")
Taxsola.soundeffect(1) #0: Installation | 1: Success | 2: Notification
}
#------------------------------------------------------------------------------------


#RECIEVE HTML REQUEST TO USE ACTIVE MATERIAL AS MAIN COLOR
pallete_dialog.add_action_callback("GetActive"){|action_context, action|

if (action.to_s != "get active") then raise Exception.new("Color pallete error: CPxV.000 - Command not recognized.") end
print "CPxE.000 - HTML request: " + action.to_s 

model = Sketchup.active_model
materials = model.materials
@mat = materials.current

if @mat.nil?
UI.messagebox("No active material, select one in 'materials' tray.", MB_OK)
raise Exception.new("Color pallete error: CPxV.001 - Operation aborted due to missing active material.")
end

@rgb_color = @mat.color.to_a
@R = @rgb_color[0]
@G = @rgb_color[1]
@B = @rgb_color[2]

#CONVERT RGB TO HSV
(
@r = @R/255.0
@g = @G/255.0
@b = @B/255.0

@rgb_array = [@r, @g, @b]
@max = @rgb_array.max
@min = @rgb_array.min

@h = @max
@s = @max
@v = @max

@d = @max - @min

if (@max == 0) then @s = 0 else @s = (@d / @max) end

if(@max == @min)
	@h = 0 #Achromatic
else
	case @max
		when @r
			@gb = 0
			if (@g < @b) then @gb = 6 else @gb = 0 end
			@h = (@g - @b) / @d + @gb

		when @g
			@h = (@b - @r) / @d + 2

		when @b
			@h = (@r - @g) / @d + 4
	end

	@h /= 6.0
end

#scale conversion
@h *= 360.0
@s *= 100.0
@v *= 100.0
@hsv_color = [@h, @s, @v] #(0-360, 0-100, 0-100)
#------------------------------------------------------
)

#SEND HSV COLOR TO HTML SIDE
@js_command = "returncolor(" + @h.to_s + ", " + @s.to_s + ", " + @v.to_s + ")"
pallete_dialog.execute_script(@js_command)
}
#------------------------------------------------------------------------------------



#IMPORT SKM
pallete_dialog.add_action_callback("Import_skm"){|action_context|
UI.messagebox("Under development.", MB_OK)
}

#REPLACE TEXTURES
pallete_dialog.add_action_callback("Replace_tex"){|action_context|
	puts "-------------------------------"
	puts Time.now.to_s
	puts ("Replace textures operation.\n\n")

	#Get in model materials
	@Materials = Sketchup.active_model.materials
	@msg = "There are no materials on model."
	if (@Materials.length == 0) then UI.messagebox(@msg); raise Exception.new(@msg); end

	@To_Replace = []
	for i in 0..(@Materials.length-1) #Loop on all materials in model
	@Curr_Mat = @Materials[i]
		if (@Curr_Mat.texture != nil) #Only if it has texture
			@Tex_Name = @Curr_Mat.texture.filename.reverse.split("/")[0].reverse
			@To_Replace << [@Curr_Mat, @Tex_Name]
		end
	end


	@msg = "There are no textured materials on model."
	if (@To_Replace.length == 0) then UI.messagebox(@msg); raise Exception.new(@msg); end

	#Confirm operation
	if UI.messagebox("There are " + @To_Replace.length.to_s + " textured materials in model.\n\nReplace textures?", type = MB_YESNO) != IDYES then raise Exception.new("Aborted operation."); end


	#Get new textures folder
	@Folder = UI.select_directory()

	if (@Folder != nil)
		@Files = Dir.glob("#{@Folder}/**/*")

		@Replaced_Counter = 0
		@Failed = []

		@To_Replace.each do |mat| #For each textured material in model
				@Status = "not found"

			for a in 0..(@Files.length-1) #Search in all files in selected folder
				if ( @Files[a].end_with?(mat[1]) )
					mat[0].texture = @Files[a] #.to_s.tr("/","\\")
					@Replaced_Counter += 1
					@Status = "found"
					break
				end
			end

			if (@Status == "not found") then @Failed << mat[0].texture.filename; end
		end

	@Replace_Rate = ((@Replaced_Counter.to_f/@To_Replace.length.to_f)*100.0).round(2)
	@msg = @Replaced_Counter.to_s + " of " + @To_Replace.length.to_s + " textures has been found and replaced. (" + @Replace_Rate.to_s + "%)"
	UI.messagebox(@msg)
	puts @msg

	#Report fails
	if (@Failed.length > 0) then puts "\nFailed to replace:\n"; @Failed.each { |fail| puts fail}; end
	puts "-------------------------------"
	end
}


	
	model = Sketchup.active_model
	materials = model.materials

#PURGE UNUSED MATERIALS
pallete_dialog.add_action_callback("Purge_un"){|action_context|
	if (UI.messagebox("Purge unused materials?", MB_YESNO) == IDYES)
		@Before = Sketchup.active_model.materials.length

		model.start_operation('Purge materials', true)
			materials.purge_unused
		model.commit_operation

		@After = Sketchup.active_model.materials.length
		UI.messagebox((@Before - @After).to_s + " materials have been purged.", MB_OK)
	end
}

#DELETE ALL MATERIAS IN SCENE
pallete_dialog.add_action_callback("Del_all"){|action_context|

	if (materials.length < 1)
		UI.messagebox("There are no materials.", MB_OK)
		raise Exception.new("Delete materials DMxV000 - There are no materials.\n\n")
	end

	if (UI.messagebox("There are " + materials.length.to_s + " materials.\nAre you sure you want to delete them all?", MB_YESNO) == IDYES)
		matl = materials.current
		materials.current = nil if !matl.nil?

		@Before = materials.length

		model.start_operation('Delete materials', true)
			for i in 0..materials.length-1
				materials.remove(materials[0])
			end
		model.commit_operation

		@After = materials.length
		@Deleted = (@Before-@After)
		@Ratio = (@Deleted/@Before*100).round(2).to_s

		UI.messagebox(@Deleted.to_s + " materials got deleted. (" + @Ratio + "%)", MB_OK)
		print ("Delete materials operation: " + @Deleted.to_s + " materials got deleted. (" + @Ratio + "%)\n\n")

	else
		UI.messagebox("Delete materials operation canceled.", MB_OK)
		raise Exception.new("Delete materials DMxE000 - Operation canceled by user.\n\n")
	end
}



end

	end #end of extension space
end #end of Taxsola's space