=begin
#-------------------------------
Author: Tax, 2021-2022
Description: Initial procedures, loads all files and create menus.
#-------------------------------
=end


require 'sketchup.rb'

module Taxsola
	module Tax_Engineering

#LIBRARIES
require 'json' unless defined? JSON
require "bigdecimal/math"
include BigMath
#----------------------------------

#GENERAL INSTANCE VARIABLES
@Ext_Name = PLUGIN_NAME.to_s
@Author = EXTENSION.creator.to_s
@Ext_version = EXTENSION.version.to_s

@This_File_Path = File.dirname(__FILE__)
@@Menus_Path = @This_File_Path
@Tax_Files = Dir.glob("#{@This_File_Path}/**/*.rb")
@Sub_Extensions = @Tax_Files - [@This_File_Path]
@@Settings_Path = @This_File_Path.to_s + "/Dependencies/Settings/Settings_Register.json"
@Extension_web_link = "https://sketchucation.com/plugin/2444-tax_tools"
@Donation_web_link = "https://www.paypal.com/donate/?business=UXBBY5RD98KSQ&no_recurring=0&item_name=SU+extensions+development&currency_code=USD"

@@HtmlDialog_Dependencies = "
<link href='" + @This_File_Path.to_s + "/Dependencies/modus.css' rel='stylesheet'>
<link href='" + @This_File_Path.to_s + "/Dependencies/modus-icons.css' rel='stylesheet'>
<script src='" + @This_File_Path.to_s + "/Dependencies/jquery.js'></script>
<script src='" + @This_File_Path.to_s + "/Dependencies/bootstrap.bundle.min.js'></script>
"

@PI = Math::PI
@Alarm_State = "free" #Used for 'Vray alarm'
#----------------------------------

#INITIAL PROCEDURES
#Loads all sub extensions (including Tax_lib) if not loaded already
@Sub_Extensions.each{|file| require file;}

@Settings_data = Taxsola.check_settings() #READS SETTINGS FILE

#CONSOLE ON START UP?
if (@Settings_data['open_console_on_start']) then SKETCHUP_CONSOLE.show; puts "TAX - Start up Console"; end
#-----------------------------------------------------


#SUBMENUS
unless file_loaded?(__FILE__)
	@Menu = UI.menu("Plugins")
	@TaxSubmenu = @Menu.add_submenu(@Ext_Name)
	@TE_toolbar = UI::Toolbar.new(@Ext_Name) #TOOLBAR CREATION

	#Mechanicals submenu
	@TaxSubmenu.add_item("Mechanicals"){self.mech_dialog}
	#Toolbar setup
	@mech = UI::Command.new(@Ext_Name) {self.mech_dialog}
	@mech.small_icon = @This_File_Path.to_s + "/Dependencies/Toolbar_Icons/Tools.svg"
	@mech.large_icon = @This_File_Path.to_s + "/Dependencies/Toolbar_Icons/Tools.svg"
	@mech.tooltip = "Mechanicals"
	@mech.status_bar_text = "Opens mechanicals dialogbox."
	@TE_toolbar = @TE_toolbar.add_item @mech

	def self.mech_dialog
		mechs_dialog = UI::HtmlDialog.new({
		  :dialog_title => "TAX - Mechanicals",
		  :preferences_key => "com.sample.plugin",
		  :resizable => false,
		  :style => UI::HtmlDialog::STYLE_DIALOG
		})
		(html = "<!DOCTYPE html>
		<html>

			<head>
				"+@@HtmlDialog_Dependencies+"

				<script>function Redirect(num) {sketchup.Redirect(num);}</script>
			</head>

			<body class='m-3 bg-panel-background'>


				<div class='modal-footer'>
					<button class='btn btn-block btn-tertiary' onclick='Redirect(10)'>Displacer</button>
					<button class='btn btn-block btn-tertiary' onclick='Redirect(1)'>Follow-Rotate-Scale</button>
					<button class='btn btn-block btn-tertiary' onclick='Redirect(2)'>Railway Along</button>
					<button class='btn btn-block btn-tertiary' onclick='Redirect(3)'>Roller Chain Along</button>
					<button class='btn btn-block btn-tertiary' onclick='Redirect(4)'>Mariner Chain Along</button>
					<button class='btn btn-block btn-tertiary' onclick='Redirect(5)'>Bicycle Wheel</button>
					<button class='btn btn-block btn-tertiary' onclick='Redirect(6)'>Spur Gear</button>
					<button class='btn btn-block btn-tertiary' onclick='Redirect(7)'>Adjustable Suspension</button>
					<button class='btn btn-block btn-tertiary' onclick='Redirect(8)'>Spheric Helix</button>
					<button class='btn btn-block btn-tertiary' onclick='Redirect(9)'>Linear Helix</button>
				</div>

		  </body>
		</html>

		")

		mechs_dialog.set_html(html)
		mechs_dialog.set_size(300, 540)
		mechs_dialog.show

		#Redirect
		mechs_dialog.add_action_callback("Redirect"){|action_context, num|
			case num.to_s
				when "1"
					self.FRS_dialog

				when "2"
					self.railway_along_dialog

				when "3"
					self.roller_dialog

				when "4"
					self.mariner_dialog

				when "5"
					self.bicycle_wheel_dialog

				when "6"
					self.spur_dialog

				when "7"
					self.adjustable_suspension_dialog

				when "8"
					self.spheric_spiral_dialog

				when "9"
					self.linear_spiral_dialog

				when "10"
					self.displacer_dialog
					
				else
					puts("error: Mechanical dialog ID not recognized.")
			end
		}
	end


	#Pallete submenu
	@TaxSubmenu.add_item("Pallete"){self.pallete_dialog}
	#Toolbar setup
	@pal = UI::Command.new(@Ext_Name) {self.pallete_dialog}
	@pal.small_icon = @This_File_Path.to_s + "/Dependencies/Toolbar_Icons/Palette.svg"
	@pal.large_icon = @This_File_Path.to_s + "/Dependencies/Toolbar_Icons/Palette.svg"
	@pal.tooltip = "Color pallete"
	@pal.status_bar_text = "Opens harmonic color pallete dialogbox."
	@TE_toolbar = @TE_toolbar.add_item @pal
	
	#SU links
	@TaxSubmenu.add_item("SU links"){self.su_links}
	#Toolbar setup
	@links = UI::Command.new(@Ext_Name) {self.su_links}
	@links.small_icon = @This_File_Path.to_s + "/Dependencies/Toolbar_Icons/Internet.svg"
	@links.large_icon = @This_File_Path.to_s + "/Dependencies/Toolbar_Icons/Internet.svg"
	@links.tooltip = "SU links"
	@links.status_bar_text = "Opens SU web links list."
	@TE_toolbar = @TE_toolbar.add_item @links


	#UTILITIES MENUS
	@UtilitiesMenu = @TaxSubmenu.add_submenu("Utilities")

	(
	#SELECTION REPORT
	@SR_Menu_Counter = 1
	@SR_Menu = @UtilitiesMenu.add_item("Selection report"){self.report_dialog}
	@SR_Menu_Status = @UtilitiesMenu.set_validation_proc(@SR_Menu) {if (@SR_Menu_Counter%2 == 0) then MF_CHECKED; else MF_UNCHECKED; end}

	#VRAY ALARM
	@UtilitiesMenu.add_item("V-ray alarm"){self.alarm_dialog}

	#AGROUP BY LOOSE
	@UtilitiesMenu.add_item("Agroup by loose"){self.abyloose_dialog}

	#Multiple push pull
	@UtilitiesMenu.add_item("Multiple push pull"){self.multpushpull_dialog}

	#Disguise pattern
	@UtilitiesMenu.add_item("Disguise pattern"){self.disguise_dialog}

	#Edge groups
	@UtilitiesMenu.add_item("Edge groups"){self.edge_groups}
	
	#Del faces
	@UtilitiesMenu.add_item("Del faces"){self.del_faces_dialog}
	
	#Remove instance naming
	@UtilitiesMenu.add_item("Remove instance naming"){
		puts ("Starting to remove instance names.")
		Sketchup.active_model.definitions.each{ |d| d.instances.each{ |i| i.name = '' } }
		puts ("Remove instance names complete.")
		
		#Success sound
		@Settings_data = Taxsola.check_settings()
		if (@Settings_data['success_sound']) then Taxsola.soundeffect(1); end #1: Success
	}
	)



	#SETTINGS & HELP submenus
	@TaxSubmenu.add_separator
	@TaxSubmenu.add_item("Settings"){self.settings}
	#Toolbar setup
	@TE_toolbar = @TE_toolbar.add_separator
	@sett = UI::Command.new(@Ext_Name) {self.settings}
	@sett.small_icon = @This_File_Path.to_s + "/Dependencies/Toolbar_Icons/Settings.svg"
	@sett.large_icon = @This_File_Path.to_s + "/Dependencies/Toolbar_Icons/Settings.svg"
	@sett.tooltip = "Settings"
	@sett.status_bar_text = "Opens extension settings."
	@TE_toolbar = @TE_toolbar.add_item @sett

	def self.settings
		@Settings_data = Taxsola.check_settings() #READS SAVED SETTINGS
		@Sound_Path = @Settings_data['vray_alarm_sound'] #GET VRAY ALARM SOUND PATH
		
		#GET CHECKBOXES - SWITCHES --------------------------------------------
		@checked1 = @Settings_data['open_console_on_start']
		@checked2 = @Settings_data['check_updates_on_start']
		@checked3 = @Settings_data['success_sound']
		@checked4 = @Settings_data['notification_sound']
		
		#DEFINE CHECKBOXES - SWITCHES
		if (@checked1) then @checked1="checked"; else @checked1=""; end
		if (@checked2) then @checked2="checked"; else @checked2=""; end
		if (@checked3) then @checked3="checked"; else @checked3=""; end
		if (@checked4) then @checked4="checked"; else @checked4=""; end
		#----------------------------------------------------------------------
		
		@params = {
		  :dialog_title => "TAX - Settings",
		  :preferences_key => "com.sample.plugin",
		  :resizable => false,
		  :style => UI::HtmlDialog::STYLE_DIALOG
		}
		#GET HTML FILE
		@html = File.read(@This_File_Path.to_s + "/Dependencies/Settings/Settings_UI.html")
		
		#REPLACING MARKERS ---------------------------------------------------------
		@html.gsub!("DEPENDENCIES", @@HtmlDialog_Dependencies.to_s) #SET DEPENDENCIES
		@html.gsub!("REGISTERED_PATH", @Sound_Path.to_s) #SET VRAY ALARM SOUND PATH
		
		#SET CHECKBOXES - SWITCHES
		@html.gsub!("IS_CHECKED1", @checked1.to_s)
		@html.gsub!("IS_CHECKED2", @checked2.to_s)
		@html.gsub!("IS_CHECKED3", @checked3.to_s)
		@html.gsub!("IS_CHECKED4", @checked4.to_s)
		
		
		@settings_dialog = UI::HtmlDialog.new(@params)
		@settings_dialog.set_html(@html)
		@settings_dialog.set_size(300, 420)
		@settings_dialog.show
		#----------------------------------------------------------------------------


		#CALLBACKS --------------------------------------------------------------------------------
		
		#Callback 1 - Select Vray alarm sound
		@settings_dialog.add_action_callback("Pick_Sound"){|action_context, id|
			@sound_path = UI.openpanel("Select Vray alarm sound", "", "Audio Files|*.wav;*.mp3;||").tr("\\","/")
			@settings_dialog.execute_script( "Return_Sound('" + @sound_path.to_s + "')" )
		}
		
		#Callback 2 - Save buttom
		@settings_dialog.add_action_callback("Save"){|action_context, oc, cu, ss, ns, vasp|
			@Settings_data['open_console_on_start'] = oc
			@Settings_data['check_updates_on_start'] = cu
			@Settings_data['success_sound'] = ss
			@Settings_data['notification_sound'] = ns
			@Settings_data['vray_alarm_sound'] = vasp

			if not(File.file?(vasp)) && (vasp != "") #Check if sound file exists
				@msg = "error: Vray-alarm sound path not found."
				UI.messagebox(@msg)
				raise Exception.new(@msg)
			end
			
			Taxsola.write_settings(@Settings_data) #Register settings
			
			#Success
			print "TAX - Settings has been saved."
			@Settings_data = Taxsola.check_settings()
			if (@Settings_data['success_sound']) then Taxsola.soundeffect(1); end #1: Success
		}
	end



	@TaxSubmenu.add_item("Help"){self.help_dialog}
	#Toolbar setup
	@hel = UI::Command.new(@Ext_Name) {self.help_dialog}
	@hel.small_icon = @This_File_Path.to_s + "/Dependencies/Toolbar_Icons/Help.svg"
	@hel.large_icon = @This_File_Path.to_s + "/Dependencies/Toolbar_Icons/Help.svg"
	@hel.tooltip = "Help"
	@hel.status_bar_text = "Opens extension help."
	@TE_toolbar = @TE_toolbar.add_item @hel

	def self.help_dialog
		@ECA_link = "https://www.gov.br/mdh/pt-br/assuntos/noticias/2021/julho/trinta-e-um-anos-do-estatuto-da-crianca-e-do-adolescente-confira-as-novas-acoes-para-fortalecer-o-eca/ECA2021_Digital.pdf"
	
		help_box = UI::HtmlDialog.new({
		  :dialog_title => "TAX - Help",
		  :preferences_key => "com.sample.plugin",
		  :resizable => false,
		  :style => UI::HtmlDialog::STYLE_DIALOG
		})
		(html = "<!DOCTYPE html>
		<html>

			<head>
				"+@@HtmlDialog_Dependencies+"

				<script>function WebPage() {sketchup.OpenUrl('"+@Extension_web_link+"');}</script>
				<script>function Docs() {sketchup.Docs();}</script>
				<script>function Folder() {sketchup.Folder();}</script>
				<script>function Reload() {sketchup.Reload();}</script>
				<script>function Donate() {sketchup.OpenUrl('"+@Donation_web_link+"');}</script>
				<script>function CheckUpdate() {sketchup.check_update();}</script>
				<script>function Laws() {sketchup.OpenUrl('"+@ECA_link+"');}</script>
			</head>

			<body class='m-3 bg-panel-background'>


			<div class='card' style='width: 24rem;'>
				<div class='card-body'>

					<h4 class='card-title' id='card-title-2'>Help</h4>
					<h5 class='card-subtitle mb-2 text-muted' id='card-subtitle-2'>About</h5>

					<p class='card-text'>
						Extension: " + @Ext_Name + "<br>
						Author: " + @Author + "<br>
						Version: " + @Ext_version + "<a href='#' onclick='CheckUpdate(); return false;'> (check update)</a><br>
						License: " + "free/open source" + "<br>
					</p>

					<p class='card-text'>
						A tribute to all young victims of connivance.<br>
						
						<a href='#' onclick='Laws(); return false;'>
							ECA Art. 4, 5, 13, 17, 18 & 18A(II).<br>
							LEI 13.431 Art. 4(II a) & 13.
						</a>
					</p>

					<a href='#' onclick='WebPage(); return false;'>WebPage</a> | 
					<a href='#' onclick='Docs(); return false;'>Docs</a> | 
					<a href='#' onclick='Folder(); return false;'>Open folder</a> | 
					<a href='#' onclick='Reload(); return false;'>Reload me</a> | 
					<a href='#' onclick='Donate(); return false;'>Donate</a>

				</div>
			</div>


		  </body>
		</html>
		")

		help_box.set_html(html)
		help_box.set_size(433, 385)
		help_box.show



		#Open Url on external browser (used for WebPage and Donate)
		help_box.add_action_callback("OpenUrl"){|action_context, url| UI.openURL(url) }

		#check_update
		help_box.add_action_callback("check_update"){|action_context|
			#Success sound
			@Settings_data = Taxsola.check_settings()
			if (@Settings_data['success_sound']) then Taxsola.soundeffect(1); end #1: Success
			
			Taxsola.update_check(@Ext_version, @Extension_web_link, "check")
		}
		
		#DOCS
		help_box.add_action_callback("Docs"){|action_context| Tax_Engineering.docs_dialog; }

		#PLUGINS FOLDER
		help_box.add_action_callback("Folder"){|action_context|
			@Plugins_Folder = Sketchup.find_support_file("Plugins").to_s.tr("/","\\")
			system("explorer #{@Plugins_Folder}")
		}

		#RELOAD
		help_box.add_action_callback("Reload"){|action_context|
			@This_File_Path = File.dirname(__FILE__)
			@Tax_Files = Dir.glob("#{@This_File_Path}/**/*.rb") #Loads '.rb' files only

			print "-----------------------------------------"
			print "Reloading " + @Ext_Name + ": " + Time.now.to_s + "\n\n"

			@a=[]
			@i=0
				@Tax_Files.each do |file|
					@Status = load(file)
					if (@Status) #Loading Sucess
						@i+=1
						print file
					else #Loading Fail
						@a << file
					end
				end

			print "(" + @i.to_s + "/" + @Tax_Files.length.to_s + ") files loaded."

				if (@a.length > 0) #There is loading errors
					print "\nErrors: (" + @a.length.to_s + ")"
					@a.each{|file| print file;}
					UI.messagebox("(" + @i.to_s + "/" + @Tax_Files.length.to_s + ") files loaded.", type = MB_OK)
				else
					#Success sound
					@Settings_data = Taxsola.check_settings()
					if (@Settings_data['success_sound']) then Taxsola.soundeffect(1); end #1: Success
				end
			print "-----------------------------------------"

		}

	end



	#DOCUMENTATIONS
	def self.docs_dialog
		docs_dialog = UI::HtmlDialog.new({
		  :dialog_title => "TAX - Docs (under development)",
		  :preferences_key => "com.sample.plugin",
		  :resizable => false,
		  :style => UI::HtmlDialog::STYLE_DIALOG
		})
		html = File.read(@This_File_Path.to_s + "/Docs/Docs.html")
		html.gsub!("DEPENDENCIES", @@HtmlDialog_Dependencies.to_s)
		html.gsub!("DONATE_LINK", @Donation_web_link.to_s)
		
		docs_dialog.set_html(html)
		docs_dialog.set_size(800, 600)
		docs_dialog.show
		
		#Open Url on external browser
		docs_dialog.add_action_callback("OpenUrl"){|action_context, url| UI.openURL(url.to_s) }
	end



	#INSTALLATION REGISTER
	if not(@Settings_data['installed'])
		@alert = @Ext_Name + " extension\n\nALERT FOR MAC USERS:\nAlways save your project before using it.\nReason: I (dev) can't test for mac on a regular basis, so I can't guarantee a stable behavior. Thx for your comprehension.\n\nEnjoy!"
		if (Sketchup.platform == :platform_osx) then UI.messagebox(@alert.to_s,MB_OK); end
		
@Tax_Logo = <<END
#####    #     #   #
  #     # #     # #
  #    #####     #
  #   #     #   # #
  #  #       # #   #
END
puts @Tax_Logo

		puts (@Ext_Name.to_s + " v" + @Ext_version.to_s + " has been installed.")
		Taxsola.soundeffect(0) #Plays installation soudeffect
		
		@Settings_data['installed'] = true; #Defines as installed
		Taxsola.write_settings(@Settings_data) #WRITE TO SETTINGS FILE
	end

	#Check for extension update (current_version, weblink, mode)- @Ext_version
	if (@Settings_data['check_updates_on_start']) then Taxsola.update_check(@Ext_version, @Extension_web_link, "start"); end


	@TE_toolbar.show
	file_loaded(__FILE__)
end

		@TE_toolbar.show
	end #end of extension space
end #end of Taxsola's space

