=begin
#-------------------------------
Author: Tax, 2021
Description:
#-------------------------------
=end


#-------------------------------------------------------------------------------
module Taxsola
	module Tax_Engineering

def self.abyloose_dialog

model = Sketchup.active_model
entities = model.active_entities


#SELECTION FILTERS
@Selection = model.selection.to_a #Selection to array

@Faces = @Selection.grep(Sketchup::Face)
@Edges = @Selection.grep(Sketchup::Edge)

#IS THERE VALID SELECTION?
if (@Faces.length + @Edges.length == 0)  # < CHECK 1
UI.messagebox("No valid element is selected (face or edge).")
raise Exception.new("Agroup by loose error: ABLxV.000 - Operation aborted due to missing valid selection.")
end



#SELECTION REPORT
@Groups = @Selection.grep(Sketchup::Group)
@Components = @Selection.grep(Sketchup::ComponentInstance)
@Sections = @Selection.grep(Sketchup::SectionPlane)

@ConsideredText = "CONSIDERED:\nFaces: " + @Faces.length.to_s + "\nEdges: " + @Edges.length.to_s + "\n\n"
@IgnoredText = "IGNORED:\nGroups: " + @Groups.length.to_s + "\nComponents: " + @Components.length.to_s + "\nSections: " + @Sections.length.to_s + "\n\n"

if UI.messagebox(@ConsideredText + @IgnoredText + "\nContinue?", type = MB_YESNO) == IDNO # < CHECK 2
raise Exception.new("Agroup by loose error: ABLxV.001 - Operation aborted by user.")
end
model.selection.clear


#Print input log
print 'Agroup by loose operation:'
print "Input log: " + @Faces.length.to_s + " faces, " + @Edges.length.to_s + " edges, " + @Groups.length.to_s + " groups, " + @Components.length.to_s + " components, " + @Sections.length.to_s + " sections."


#Timer start
@start_time = Time.now
#-------------------------------------------
model.start_operation('Agroup by loose', true)


#VALID SELECTION
@ValidSelection = @Edges
@Faces.each { |entity| @ValidSelection << entity }

#TOO LARGE SELECTION CHECK
@Faces_Selection_Limit = 3000
@Edges_Selection_Limit = 10000
if (@Faces.length > @Faces_Selection_Limit) || (@Edges.length > @Edges_Selection_Limit)
	if UI.messagebox("Too large selection.\nIt's recomended to make a single group instead.", type = MB_YESNO) != IDNO # < CHECK 3
	@SingleX = entities.add_group(@ValidSelection)
	@ValidSelection = []
	UI.messagebox("Single group made.", type = MB_OK)
	model.selection.add(@SingleX)
	raise Exception.new("Agroup by loose: ABLxV.002 - Single group made due to large selection.")
	end
end

#Groups ammount calculation
if UI.messagebox("Groups ammount calculation. It may take a while.\nPlease wait.\n\n Continue?", MB_YESNO) == IDNO  # < CHECK 4
model.selection.add(@ValidSelection)
raise Exception.new("Agroup by loose error: ABLxV.003 - Operation aborted by user.")
end

@Groups = []
until (@ValidSelection.length == 0)
@LooseX = @ValidSelection[0].all_connected
@Groups << @LooseX
@ValidSelection -= @LooseX
end
#--------------------------

#Groups creating
if UI.messagebox(@Groups.length.to_s + " groups to be created.\n\nContinue?", MB_YESNO) == IDNO  # < CHECK 5
model.selection.add(@ValidSelection)
raise Exception.new("Agroup by loose error: ABLxV.004 - Operation aborted by user.")
end

#Agroup by loose
@All_Them = model.entities.add_group
@All_Them.name = "Agroup by loose"

@Agrouping_Limit = 50
@Counter = 0
@Groups.each do |group|
	@GroupX = entities.add_group(group)
	@All_Them.entities.add_instance(@GroupX.entities.parent, @GroupX.transformation*@All_Them.transformation)
	@GroupX.erase!
	@Counter += 1

	#Agrouping limit (It may freeze SU on large selections)
	if (@Counter % @Agrouping_Limit == 0) && (@Counter != @Groups.length)
		@advance = (@Counter*100/@Groups.length).round(2)
		if UI.messagebox("(Freeze prevention)\n\n" + @Counter.to_s + " groups created until now (" + @advance.to_s + "%).\n\nContinue?", MB_YESNO) == IDNO then break end
	end
end
#--------------------------
model.selection.add(@All_Them)

model.commit_operation
#-------------------------------------------
 #Timer end
@end_time = Time.now
@Total_Time = @end_time - @start_time


#End Message
@advance = (@Counter*100/@Groups.length).round(2)
@EndMessage = "Success.\n" + @Counter.to_s + " groups have been created (" + @advance.to_s + "%).\n\nTotal time:\n" + (@Total_Time/60.0).truncate.to_s + " min " + (@Total_Time%60.0).round(2).to_s + " seconds\n"
print @EndMessage + "\n\n"
Taxsola.soundeffect(1) #0: Installation | 1: Success | 2: Notification
end

	end
end
#-------------------------------------------------------------------------------