=begin
#-------------------------------
Author: Tax, 2021
Description:
#-------------------------------
=end


#-------------------------------------------------------------------------------
module Taxsola
	module Tax_Engineering

def self.disguise_dialog
model = Sketchup.active_model
entities = model.active_entities

@Selection = model.selection.to_a #Selection to array
@Groups = @Selection.grep(Sketchup::Group)
@Components = @Selection.grep(Sketchup::ComponentInstance)

#IS THERE VALID SELECTION?
if (@Groups.length + @Components.length < 1)
UI.messagebox("No valid element is selected (groups or components).")
raise Exception.new("Disguise pattern error: DPxV.000 - Operation aborted due to missing valid selection.\n\n")
end

#Selection report
if UI.messagebox("Selected itens:\n" + @Groups.length.to_s + " groups\n" + @Components.length.to_s + " components\n\nContinue?", type = MB_YESNO) == IDNO
raise Exception.new("Disguise pattern error: DPxV.001 - Operation aborted by user.\n\n")
end

@ValidSelection = @Groups
@Components.each { |instance| @ValidSelection << instance }


#DEFAULT VARIABLES VALUES
@MinXmov = -2.0
@MaxXmov = 2.0

@MinYmov = -2.0
@MaxYmov = 2.0

@MinZmov = 0.0
@MaxZmov = 0.0

#----

@MinXrot = 0.0
@MaxXrot = 0.0

@MinYrot = 0.0
@MaxYrot = 0.0

@MinZrot = -15.0
@MaxZrot = 15.0

#----

@MinScale = 1.0
@MaxScale = 1.0
#-------------------


@Verification = "NOT OK"
#-------------------------------------------------------------------------------
until @Verification == "OK"
@Verification = "OK"

prompts = ["Xaxis min move (cm):", "Xaxis max move (cm):", "Yaxis min move (cm):", "Yaxis max move (cm):", "Zaxis min move (cm):", "Zaxis max move (cm):", "Xaxis min rot (deg):", "Xaxis max rot (deg):", "Yaxis min rot (deg):", "Yaxis max rot (deg):", "Zaxis min rot (deg):", "Zaxis max rot (deg):", "Min Scale:", "Max Scale:"]
defaults = [@MinXmov, @MaxXmov, @MinYmov, @MaxYmov, @MinZmov, @MaxZmov, @MinXrot, @MaxXrot, @MinYrot, @MaxYrot, @MinZrot, @MaxZrot, @MinScale, @MaxScale]
list = ["", "", "", "", "", "", "", "", "", "", "", "", "", ""]
input = UI.inputbox(prompts, defaults, list, "Disguise pattern")

#Get input values
if input
@MinXmov, @MaxXmov, @MinYmov, @MaxYmov, @MinZmov, @MaxZmov, @MinXrot, @MaxXrot, @MinYrot, @MaxYrot, @MinZrot, @MaxZrot, @MinScale, @MaxScale = input;
else
 raise Exception.new("Disguise pattern error: DPxV.002 - Operation canceled.\n\n")
end

#Print input log
print 'Disguise pattern operation:'
print "Input log: " + @MinXmov.to_s + " | " + @MaxXmov.to_s + " | " + @MinYmov.to_s + " | " + @MaxYmov.to_s + " | " + @MinZmov.to_s + " | " + @MaxZmov.to_s + " | " + @MinXrot.to_s + " | " + @MaxXrot.to_s + " | " + @MinYrot.to_s + " | " + @MaxYrot.to_s + " | " + @MinZrot.to_s + " | " + @MaxZrot.to_s + " | " + @MinScale.to_s + " | " + @MaxScale.to_s


#VERIFICATIONS
(
if (@MinXmov > @MaxXmov)
UI.messagebox("error: MPPxV.003 - Xaxis min move can't be higher than Xaxis max move.")
warn("Disguise pattern error: DPxV.003 - Xaxis min move can't be higher than Xaxis max move.\n\n")
@Verification = "NOT OK"
end

if (@MinYmov > @MaxYmov)
UI.messagebox("error: MPPxV.004 - Yaxis min move can't be higher than Yaxis max move.")
warn("Disguise pattern error: DPxV.004 - Yaxis min move can't be higher than Yaxis max move.\n\n")
@Verification = "NOT OK"
end

if (@MinZmov > @MaxZmov)
UI.messagebox("error: MPPxV.005 - Zaxis min move can't be higher than Zaxis max move.")
warn("Disguise pattern error: DPxV.005 - Zaxis min move can't be higher than Zaxis max move.\n\n")
@Verification = "NOT OK"
end


if (@MinXrot > @MaxXrot)
UI.messagebox("error: MPPxV.006 - Xaxis min rot can't be higher than Xaxis max rot.")
warn("Disguise pattern error: DPxV.006 - Xaxis min rot can't be higher than Xaxis max rot.\n\n")
@Verification = "NOT OK"
end

if (@MinYrot > @MaxYrot)
UI.messagebox("error: MPPxV.007 - Yaxis min rot can't be higher than Yaxis max rot.")
warn("Disguise pattern error: DPxV.007 - Yaxis min rot can't be higher than Yaxis max rot.\n\n")
@Verification = "NOT OK"
end

if (@MinZrot > @MaxZrot)
UI.messagebox("error: MPPxV.008 - Zaxis min rot can't be higher than Zaxis max rot.")
warn("Disguise pattern error: DPxV.008 - Zaxis min rot can't be higher than Zaxis max rot.\n\n")
@Verification = "NOT OK"
end

if (@MinScale > @MaxScale)
UI.messagebox("error: MPPxV.009 - Min Scale can't be higher than Max Scale.")
warn("Disguise pattern error: DPxV.009 - Min Scale can't be higher than Max Scale.\n\n")
@Verification = "NOT OK"
end


)

end


#CONVERSIONS
@MinXmov = @MinXmov.cm
@MaxXmov = @MaxXmov.cm

@MinYmov = @MinYmov.cm
@MaxYmov = @MaxYmov.cm

@MinZmov = @MinZmov.cm
@MaxZmov = @MaxZmov.cm
#----
@MinXrot = @MinXrot.degrees
@MaxXrot = @MaxXrot.degrees

@MinYrot = @MinYrot.degrees
@MaxYrot = @MaxYrot.degrees

@MinZrot = @MinZrot.degrees
@MaxZrot = @MaxZrot.degrees
#-----------


#INTERVALS
@XmovI = @MaxXmov - @MinXmov
@YmovI = @MaxYmov - @MinYmov
@ZmovI = @MaxZmov - @MinZmov

@XrotI = @MaxXrot - @MinXrot
@YrotI = @MaxYrot - @MinYrot
@ZrotI = @MaxZrot - @MinZrot

@ScaleI = @MaxScale - @MinScale
#--------



#ATTEMPTS START -------------------------------------------------------------------------------
@Again = IDYES
@Attempt = 0
while @Again == IDYES

#Timer start ------------------------------------------------------------------------------
@start_time = Time.now
#-------------------------------------------
model.start_operation('Disguise pattern', true)


@Counter = 0
@ValidSelection.each do |object|
@Origin = object.transformation.origin
@Xpos, @Ypos, @Zpos = @Origin.to_a

#MOVEMENT
@Xmov = ((rand()*@XmovI) + @MinXmov)
@Ymov = ((rand()*@YmovI) + @MinYmov)
@Zmov = ((rand()*@ZmovI) + @MinZmov)

@NewPosition = [(@Xpos+@Xmov), (@Ypos+@Ymov), (@Zpos+@Zmov)]
object = object.move! (@NewPosition)


#ROTATION
@Xrot = ((rand()*@XrotI) + @MinXrot)
@Yrot = ((rand()*@YrotI) + @MinYrot)
@Zrot = ((rand()*@ZrotI) + @MinZrot)

@transformation = Geom::Transformation.rotation(@NewPosition, X_AXIS, @Xrot)
object.transform!(@transformation)

@transformation = Geom::Transformation.rotation(@NewPosition, Y_AXIS, @Yrot)
object.transform!(@transformation)

@transformation = Geom::Transformation.rotation(@NewPosition, Z_AXIS, @Zrot)
object.transform!(@transformation)


#SCALE
@Scale = ((rand()*@ScaleI) + @MinScale)
scale_transform = Geom::Transformation.scaling(@NewPosition, @Scale)
object.transformation *= scale_transform


@Counter += 1
end

model.commit_operation
#-------------------------------------------

 #Timer end
@end_time = Time.now
@Total_Time = @end_time - @start_time


#--------------------- Attempt handler
@Attempt += 1
print ("Attempt nº" + @Attempt.to_s + " (Success in " + @Total_Time.round(5).to_s + " seconds)\n")
@Again = UI.messagebox("Try other variation?", MB_YESNO)
if (@Again == IDYES) then Sketchup.undo end
#---------------------
end #END OF ATTEMPTS --------------------------------------------------------


#End Message
@Average = @Counter.to_f / @Total_Time.to_f
@EndMessage = "Success.\n" + @Counter.to_s + " disguises in " + @Total_Time.round(5).to_s + " seconds.\n (Average: " + @Average.round(2).to_s + " disguises/sec)"
print @EndMessage + "\n\n"
Taxsola.soundeffect(1) #0: Installation | 1: Success | 2: Notification
end

	end
end
#-------------------------------------------------------------------------------