=begin
#-------------------------------
Author: Tax, 2021
Description:
#-------------------------------
=end


#-------------------------------------------------------------------------------
module Taxsola
	module Tax_Engineering

def self.multpushpull_dialog

model = Sketchup.active_model
entities = model.active_entities

@Selection = model.selection.to_a #Selection to array
@Faces = @Selection.grep(Sketchup::Face)

#IS THERE VALID SELECTION?
if (@Faces.length == 0)
UI.messagebox("No valid element is selected (faces).")
raise Exception.new("Multiple push pull error: MPPxV.000 - Operation aborted due to missing valid selection.")
end

if UI.messagebox("There is " + @Faces.length.to_s + " selected faces.\nobs: be sure about reversed faces.\n\nContinue?", type = MB_YESNO) == IDNO # < CHECK 2
raise Exception.new("Multiple push pull error: MPPxV.001 - Operation aborted by user.")
end


@MinPushPull = 300.0
@MaxPushPull = 300.0
@Agroup = "Yes"

@Verification = "NOT OK"
#-------------------------------------------------------------------------------
until @Verification == "OK"
@Verification = "OK"

prompts = ["Min (cm):", "Max (cm):", "Agroup?"]
defaults = [@MinPushPull, @MaxPushPull, @Agroup]
list = ["", "", "Yes|No"]
input = UI.inputbox(prompts, defaults, list, "Multiple push pull")

#Get input values
if input
@MinPushPull, @MaxPushPull, @Agroup = input;
else
 raise Exception.new("Multiple push pull error: MPPxV.002 - Operation canceled.")
end

#Print input log
print 'Multiple push pull operation:'
print "Input log: " + @Faces.length.to_s + " faces, " + @MinPushPull.to_s + " min, " + @MaxPushPull.to_s + " max, agroup = " + @Agroup.to_s

#VERIFICATIONS
if (@MinPushPull > @MaxPushPull)
UI.messagebox("error: MPPxV.002 - min value can't be higher than max value.")
warn("Multiple push pull error: MPPxV.003 - min was higher than max value.\n\n")
@Verification = "NOT OK"
end

#Timer start ------------------------------------------------------------------------------
@start_time = Time.now
end

#-------------------------------------------
model.start_operation('Agroup by loose', true)

#Conversion
@MinPushPull = @MinPushPull.cm
@MaxPushPull = @MaxPushPull.cm

@Interval = @MaxPushPull - @MinPushPull
@Counter = 0
@GP_Limit = 50
@EX_Limit = 500

if (@Agroup == "Yes")
	@Faces.each do |face|
	 @GroupX = entities.add_group(face)
	 @GroupX.name = "MMP-" + @Counter.to_s
	 @Counter += 1
	 face.pushpull( (rand()*@Interval) + @MinPushPull )

		#Operation limit (It may freeze SU on large selections)
		if (@Counter % @GP_Limit == 0) && (@Counter != @Faces.length)
			@advance = (@Counter*100/@Faces.length).round(2)
			if UI.messagebox("(Freeze prevention)\n\n" + @Counter.to_s + " groups created until now (" + @advance.to_s + "%).\n\nContinue?", MB_YESNO) == IDNO then break end
		end
	end
else
	@Faces.each do |face|
	 face.pushpull( (rand()*@Interval) + @MinPushPull )
	 @Counter += 1

		#Operation limit (It may freeze SU on large selections)
		if (@Counter % @EX_Limit == 0) && (@Counter != @Faces.length)
			@advance = (@Counter*100/@Faces.length).round(2)
			if UI.messagebox("(Freeze prevention)\n\n" + @Counter.to_s + " extrusions until now (" + @advance.to_s + "%).\n\nContinue?", MB_YESNO) == IDNO then break end
		end
	end
end

model.commit_operation
#-------------------------------------------

 #Timer end
@end_time = Time.now
@Total_Time = @end_time - @start_time


#End Message
@EndMessage = "Success. (" + @Total_Time.to_s + " seconds)\n" + @Counter.to_s + " operations."
print @EndMessage + "\n\n"
Taxsola.soundeffect(1) #0: Installation | 1: Success | 2: Notification
end

	end
end
#-------------------------------------------------------------------------------